clear
clc

 % **************************************************************************************************************


%                                $$$$$$$$$$$$$$$$$$$$$$$$$$$$
                     
                                    latitude_injection= 70;                                      % En degrs

%                                $$$$$$$$$$$$$$$$$$$$$$$$$$$$

 
% *************************************  VARIABLES GLOBALES  **************************************************

global Ir It Il Irl Itl Irt Asat etat_init vitesse_rotation altitude points_trace tau vitesse_angul_mini
global instant_injection T Ta Tu pulse periode period_sans period_reg
global KPD_laurens FPD_laurens mom_max_bobine cpl_max_roue J_ROUE J pas_mini pas_maxi
global gain_PD_laurens K_PD_laurens F_PD_laurens consigne_tangage KB KRG KFG Hy
global H ang zenith temps_energie Pmax jour rayon_orbital inclinaison temps_fin_simu periode_orbitale
global beta_fix P_opt_fix deltaT Cpl_roue_init

% ********************************** Donnes pour les calculs d'nergie **************************************

H=21.5;                                          % Heure locale au noeud ascendant
Pmax=18;ang=35*pi/180;                           % Puissance maximum des panneaux
jour=355;jour=181;                               % Jour de l'anne 81=quinoxes 181=soltice t  355=solstice hiver;
delta=23.45*pi/180*sin(2*pi/365*(280+jour));     % Dclinaison solaire du jour choisi

% -----------------------------------------  Choix pointage nadir ou znith  --------------------------------
zenith=-1;
% -----------------------------------------------------------------------------------------------------------  
 
% ************************************ Coefficients de la rgulation PD de Laurens *************************
KPD_laurens=1.6e-5;                              %  Coefficient de rappel sur champ de consigne
FPD_laurens=8.4e-3;                              %  Coefficient de rappel sur drive du champ de consigne

% **************************************** Seuils des stops ************************************************
 
vitesse_angul_mini=2e-3;     % Vitesse angulaire au dessous de laquelle on passe au mode fin
angle_quater_maxi=10*pi/180; % Test d'arrt de l'acquisition grossire en rd ( 10 ou autre )
angles_cardan=5;             % Test d'arrt de l'acquisition grossire (5 ou autre )          

T=5;Ta=0.1;Tu=4.9;pulse=1;   % Nouvel essai

K_poinsot=1;                 % Permet par la mise  0 de simuler un mouvement de poinsot, sans moment externe
                             % ce qui permet de vrifier une bonne partie de la simulation ( intgration)
% ****************************************  ERREUR DE POSITION ******************************************
decal_angle_orbital=14;                                                   % En degrs
deltaT=decal_angle_orbital*pi/180/vitesse_rotation; deltaT,              % En temps
% *******************************************************************************************************
% Nombre de points du trac

 points_trace=50000;

% *************************** LIMITATIONS PHYSIQUES DES COMPOSANTS ************************

 mom_max_bobine=0.2;        % Am  puissance maxi d'une bobine
 
 cpl_max_roue=4e-6;         % Nm   couple maxi dlivr par la roue
 couple_max_roue=4e-6;      % Pour la premire roue que j'ai essaye
 Cpl_roue_init=-1.6e-6*zenith; % Couple roue sur satellite, de lancement de la roue, durant le dtumbling
% ***************************************************************************************
% Ce fichier initialise la simulation regderiv.m ou regdesat.m ou reg_derb.m ou boussol0.m ....

%  *******************   Matrice d''inertie satellite  *********************************

Ir=32e-3;It=36e-3;Il=22e-3;Irt=5.8e-5;Irl=3e-4;Itl=4e-3;

Asat=[Ir -Irt -Irl;
         -Irt It -Itl;
            -Irl -Itl Il];

% ***************************  Caractristiques de l'orbite du satellite ***************************

altitude_orbite=817;		% altitude satellite en km
rayon_orbital=(6378+altitude_orbite)*1000;
vitesse_rotation=sqrt(39.86e13/rayon_orbital^3);                % pulsation orbitale en rd/sinclinaison=98.7*pi/180;
inclinaison=98.7*pi/180;
periode_orbitale=2*pi/vitesse_rotation;                         % priode
latitude_injection=latitude_injection*pi/180;                   % 70 convertis en rd
instant_injection=asin(sin( latitude_injection)/sin(inclinaison))/vitesse_rotation; 
instant_injection=fix(instant_injection);              % Instant injection calcul  1 seconde prs
                                                       % aprs survol quateur suppos  t=0 seconde

% *********************************  Temps spciaux de la simulation  ********************************

temps_energie=instant_injection;                               % Temps  partir duquel l'nergie reue est mesure
temps_fin_simu=temps_energie+5*periode_orbitale;    % pour avoir une priode complte,  partir de temps_energie

                             %   **********  VALEURS INITIALES   ******

% Initialisation en vitesses (rd/s)

v_roulis0=0.19;
v_tangage0=-0.15;               
v_lacet0=0.21;

% Initialisation en position (rd)

roulis0=0.2;
lacet0=-0.8;
tangage0=1.2;




                             % ***************** DONNEES PERTURBATIONS********************

mom_magn_res=0.01;                              %  Moment magntique rsiduel en Am
alpha=sqrt(1/3);beta=sqrt(1/3);gamma=sqrt(1/3); % Direction unitaire /satellite du moment rsiduel

cpl_aero1=0;
cpl_aero2=-3.6e-8;% COUPLE AERODYNAMIQUE
cpl_aero3=0;

                             % **************** DONNEES ROUE *********************
J_ROUE=3.2e-5;      		        % Moment d'inertie de la roue en mkg
J=3.2e-5;        		       

vitesseroue=0*zenith;   		% Roue  en dbut de rgulation, elle pourra ensuite voluer  
rotation_initiale=0*zenith;             % Vitesse de rotation initiale de la roue  0 t/mn, au lancement satellite
rotation_maximale= 360;                 % Vitesse de rotation maximale de la roue environ 3600 t/mn

%                    **********************
                     Hy=J_ROUE*vitesseroue;                  % Moment cintique roue
%                    **********************

% ************ CALCULS D'INITIALISATION DU VECTEUR D'ETAT (  ne jamais changer) **************

% Rotation absolue compose de la rotation par rapport au repre orbital
% et de la rotation orbitale  vitesse_rotation porte par Y

vp0=v_roulis0+v_lacet0*sin(tangage0)+...
    vitesse_rotation*sin(lacet0)*cos(tangage0);

vq0=v_lacet0*cos(tangage0)*sin(roulis0)+v_tangage0*cos(roulis0)+...
    vitesse_rotation*(cos(lacet0)*cos(roulis0)+...
    sin(tangage0)*sin(roulis0)*sin(lacet0));

vr0=v_lacet0*cos(tangage0)*cos(roulis0)-v_tangage0*sin(roulis0)+...
    vitesse_rotation*(-cos(lacet0)*sin(roulis0)+...
    sin(tangage0)*cos(roulis0)*sin(lacet0));
   

% -------------- Calcul du quaternion initial connaissant les angles initiaux ---------------------

% Matrice de passage P de la base fixe XYZ  la base satellite xyz
%  l'instant initial

Pmat(1,1)=cos(tangage0)*cos(lacet0);
Pmat(1,2)=-sin(lacet0)*cos(roulis0)+cos(lacet0)*sin(roulis0)*sin(tangage0);
Pmat(1,3)=sin(lacet0)*sin(roulis0)+cos(lacet0)*cos(roulis0)*sin(tangage0);

Pmat(2,1)=cos(tangage0)*sin(lacet0);
Pmat(2,2)=cos(lacet0)*cos(roulis0)+sin(lacet0)*sin(roulis0)*sin(tangage0);
Pmat(2,3)=-cos(lacet0)*sin(roulis0)+sin(lacet0)*cos(roulis0)*sin(tangage0);

Pmat(3,1)=-sin(tangage0);
Pmat(3,2)=sin(roulis0)*cos(tangage0);
Pmat(3,3)=cos(roulis0)*cos(tangage0);

                                                 % determinant=det(Pmat),% ************* Bien vrifi ***********

% ------------------------  Calcul du quaternion initial ------------------------
%
% Nous prenons toujours q0 >0 ou nul

trac=trace(Pmat);

qt00=(1+trac)^0.5/2;

qt10=(1-trac+2*Pmat(1,1))^0.5/2;

qt20=(1-trac+2*Pmat(2,2))^0.5/2;

qt30=(1-trac+2*Pmat(3,3))^0.5/2;

if abs(qt00)>1e-8, 
qt10=qt10*sign((Pmat(3,2)-Pmat(2,3))/4/qt00);
qt20=qt20*sign((Pmat(1,3)-Pmat(3,1))/4/qt00);
qt30=qt30*sign((-Pmat(1,2)+Pmat(2,1))/4/qt00);
end

if abs(qt10)>1e-8, 

qt20=qt20*sign((Pmat(1,2)+Pmat(2,1))/4/qt10);
qt30=qt30*sign((Pmat(1,3)+Pmat(3,1))/4/qt10);
qt00=qt00*sign((Pmat(3,2)-Pmat(2,3))/4/qt10);
end

if abs(qt20)>1e-8, 
qt10=qt10*sign((Pmat(1,2)+Pmat(2,1))/4/qt20);
qt30=qt30*sign((Pmat(2,3)+Pmat(3,2))/4/qt20);
qt00=qt00*sign((Pmat(1,3)-Pmat(3,1))/4/qt20);
end

if abs(qt30)>1e-8, 
qt20=qt20*sign((Pmat(3,2)+Pmat(2,3))/4/qt30);
qt10=qt10*sign((Pmat(1,3)+Pmat(3,1))/4/qt30);
qt00=qt00*sign((-Pmat(1,2)+Pmat(2,1))/4/qt30);
end

                                           % Q=[qt00 qt10 qt20 qt30], %  ********* devrait avoir une norme 1

Q=[qt00 qt10 qt20 qt30]/norm([qt00 qt10 qt20 qt30]);

                                           % normQ=norm(Q),% ******** vrifi *******

                                           % energie=0.5*[vp0 vq0 vr0]*Asat*[vp0 vq0 vr0]',  
                                           % Energie devrait rester constante

etat_init=[vp0 vq0 vr0 Q(1) Q(2) Q(3) Q(4)]; 

% *****************************************************************************************************
          % Affichage espace de travail
disp(['       Moment cintique de la roue Hy= ' num2str(Hy) ' N-m']);disp('  ');
disp(['       Heure locale au noeud ascendant H=  ' num2str(H) ' heure']);disp('  ');
% ***********  Transmission de l'angle optimal ( rd ) et sortie de la puissance optimale **************

balayOpt;      % Programme transmettant ponctuellement l'angle beta optimal, pour le jour choisi,
               % ainsi que la puissance de ce jour l

disp(['       L''angle beta optimal du jour ' num2str(jour) ' est de ' num2str(beta_fix) ' degrs']);

disp(' ');

disp(['       Puissance optimale du jour ' num2str(jour) ': = ' num2str(P_opt_fix) ' Watts']);

beta_fix=beta_fix*pi/180;               % Transmission de l'angle beta optimal en radian

%*******************************************************************************************************

end








