function [ret,x0,str,ts,xts]=nanosim5(t,x,u,flag);
%NANOSIM5	is the M-file description of the SIMULINK system named NANOSIM5.
%	The block-diagram can be displayed by typing: NANOSIM5.
%
%	SYS=NANOSIM5(T,X,U,FLAG) returns depending on FLAG certain
%	system values given time point, T, current state vector, X,
%	and input vector, U.
%	FLAG is used to indicate the type of output to be returned in SYS.
%
%	Setting FLAG=1 causes NANOSIM5 to return state derivatives, FLAG=2
%	discrete states, FLAG=3 system outputs and FLAG=4 next sample
%	time. For more information and other options see SFUNC.
%
%	Calling NANOSIM5 with a FLAG of zero:
%	[SIZES]=NANOSIM5([],[],[],0),  returns a vector, SIZES, which
%	contains the sizes of the state vector and other parameters.
%		SIZES(1) number of states
%		SIZES(2) number of discrete states
%		SIZES(3) number of outputs
%		SIZES(4) number of inputs
%		SIZES(5) number of roots (currently unsupported)
%		SIZES(6) direct feedthrough flag
%		SIZES(7) number of sample times
%
%	For the definition of other parameters in SIZES, see SFUNC.
%	See also, TRIM, LINMOD, LINSIM, EULER, RK23, RK45, ADAMS, GEAR.

% Note: This M-file is only used for saving graphical information;
%       after the model is loaded into memory an internal model
%       representation is used.

% the system will take on the name of this mfile:
sys = mfilename;
new_system(sys)
simver(1.3)
if (0 == (nargin + nargout))
     set_param(sys,'Location',[4,42,1265,968])
     open_system(sys)
end;
set_param(sys,'algorithm',     'RK-45')
set_param(sys,'Start time',    'instant_injection')
set_param(sys,'Stop time',     '20000')
set_param(sys,'Min step size', '0.04')
set_param(sys,'Max step size', '0.1')
set_param(sys,'Relative error','1e-3')
set_param(sys,'Return vars',   '')

add_block('built-in/Note',[sys,'/',['DETUMBLING AVEC QUATERNION D''ATTITUDE',13,'PHASE FINALE D''ACQUISITION DES ANGLES ',13,'',13,'',13,'']])
set_param([sys,'/',['DETUMBLING AVEC QUATERNION D''ATTITUDE',13,'PHASE FINALE D''ACQUISITION DES ANGLES ',13,'',13,'',13,'']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',26)
set_param([sys,'/',['DETUMBLING AVEC QUATERNION D''ATTITUDE',13,'PHASE FINALE D''ACQUISITION DES ANGLES ',13,'',13,'',13,'']],...
		'position',[640,0,645,5])

add_block('built-in/Note',[sys,'/',['ACQUISITION DES ANGLES PAR REGULATION AVEC L''AIDE D''UNE ROUE A VITESSE CONSTANTE',13,'PRISE EN COMPTE D''UN RETARD DE POSITION SUR LA TRAJECTOIRE']])
set_param([sys,'/',['ACQUISITION DES ANGLES PAR REGULATION AVEC L''AIDE D''UNE ROUE A VITESSE CONSTANTE',13,'PRISE EN COMPTE D''UN RETARD DE POSITION SUR LA TRAJECTOIRE']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold')
set_param([sys,'/',['ACQUISITION DES ANGLES PAR REGULATION AVEC L''AIDE D''UNE ROUE A VITESSE CONSTANTE',13,'PRISE EN COMPTE D''UN RETARD DE POSITION SUR LA TRAJECTOIRE']],...
		'Font Size',16,...
		'position',[595,160,600,165])

add_block('built-in/Note',[sys,'/',['Stabilisation de M Philippe Laurens par une rgulation PD',13,' sur un champ de consigne, adapte au cas puls']])
set_param([sys,'/',['Stabilisation de M Philippe Laurens par une rgulation PD',13,' sur un champ de consigne, adapte au cas puls']],...
		'ForeGround',5,...
		'Font Weight','bold',...
		'Font Size',22,...
		'position',[623,80,628,85])


%     Subsystem  'Subsystem3'.

new_system([sys,'/','Subsystem3'])
set_param([sys,'/','Subsystem3'],'Location',[4,44,1244,1004])

add_block('built-in/Note',[sys,'/',['Subsystem3/Suppression du test',13,'sur les vitesses']])
set_param([sys,'/',['Subsystem3/Suppression du test',13,'sur les vitesses']],...
		'Font Name','Arial',...
		'position',[625,240,630,245])


%     Subsystem  'Subsystem3/Subsystem2'.

new_system([sys,'/','Subsystem3/Subsystem2'])
set_param([sys,'/','Subsystem3/Subsystem2'],'Location',[559,482,1161,775])

add_block('built-in/Note',[sys,'/','Subsystem3/Subsystem2/Pour mettre le stop hors course, fixer le gain  0'])
set_param([sys,'/','Subsystem3/Subsystem2/Pour mettre le stop hors course, fixer le gain  0'],...
		'ForeGround',7,...
		'Font Name','Arial',...
		'Font Size',18,...
		'position',[345,140,350,145])

add_block('built-in/Gain',[sys,'/','Subsystem3/Subsystem2/Gain'])
set_param([sys,'/','Subsystem3/Subsystem2/Gain'],...
		'BackGround',7,...
		'Gain','0',...
		'position',[185,63,265,97])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/Subsystem2/To Workspace'])
set_param([sys,'/','Subsystem3/Subsystem2/To Workspace'],...
		'mat-name','stop',...
		'position',[370,27,420,43])

add_block('built-in/Stop Simulation',[sys,'/','Subsystem3/Subsystem2/Stop Simulation'])
set_param([sys,'/','Subsystem3/Subsystem2/Stop Simulation'],...
		'position',[350,63,390,97])

add_block('built-in/Inport',[sys,'/','Subsystem3/Subsystem2/in_1'])
set_param([sys,'/','Subsystem3/Subsystem2/in_1'],...
		'position',[15,70,35,90])
add_line([sys,'/','Subsystem3/Subsystem2'],[270,80;345,80])
add_line([sys,'/','Subsystem3/Subsystem2'],[300,80;300,35;365,35])
add_line([sys,'/','Subsystem3/Subsystem2'],[40,80;180,80])
set_param([sys,'/','Subsystem3/Subsystem2'],...
		'Mask Display','STOP')


%     Finished composite block 'Subsystem3/Subsystem2'.

set_param([sys,'/','Subsystem3/Subsystem2'],...
		'ForeGround',5,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[995,518,1080,572])

add_block('built-in/Note',[sys,'/',['Subsystem3/ARRT SUR',13,'LES ANGLES']])
set_param([sys,'/',['Subsystem3/ARRT SUR',13,'LES ANGLES']],...
		'position',[785,685,790,690])

add_block('built-in/Note',[sys,'/',['Subsystem3/ARRT SUR LE',13,'QUATERNION']])
set_param([sys,'/',['Subsystem3/ARRT SUR LE',13,'QUATERNION']],...
		'position',[795,500,800,505])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant2'])
set_param([sys,'/','Subsystem3/Constant2'],...
		'ForeGround',1,...
		'BackGround',3,...
		'Value','2',...
		'position',[440,874,515,896])

add_block('built-in/Gain',[sys,'/','Subsystem3/Gain3'])
set_param([sys,'/','Subsystem3/Gain3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'position',[600,870,645,900])

add_block('built-in/Note',[sys,'/',['Subsystem3/Suppression du test',13,'sur le quaternion']])
set_param([sys,'/',['Subsystem3/Suppression du test',13,'sur le quaternion']],...
		'position',[570,605,575,610])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant4'])
set_param([sys,'/','Subsystem3/Constant4'],...
		'ForeGround',1,...
		'BackGround',2,...
		'Value','2',...
		'position',[465,649,540,671])

add_block('built-in/Gain',[sys,'/','Subsystem3/Gain2'])
set_param([sys,'/','Subsystem3/Gain2'],...
		'ForeGround',1,...
		'BackGround',0,...
		'position',[580,645,625,675])

add_block('built-in/Relational Operator',[sys,'/',['Subsystem3/Relational',13,'Operator1']])
set_param([sys,'/',['Subsystem3/Relational',13,'Operator1']],...
		'ForeGround',1,...
		'BackGround',3,...
		'Operator','<=',...
		'position',[615,723,645,792])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant1'])
set_param([sys,'/','Subsystem3/Constant1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Value','angles_cardan*pi/180',...
		'position',[410,764,550,786])

add_block('built-in/Sum',[sys,'/','Subsystem3/Sum1'])
set_param([sys,'/','Subsystem3/Sum1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'inputs','+-',...
		'position',[705,739,740,821])

add_block('built-in/Relational Operator',[sys,'/',['Subsystem3/Relational',13,'Operator']])
set_param([sys,'/',['Subsystem3/Relational',13,'Operator']],...
		'ForeGround',1,...
		'BackGround',2,...
		'Operator','<=',...
		'position',[620,498,645,547])

add_block('built-in/MATLAB Fcn',[sys,'/',['Subsystem3/Dcalage',13,'angulaire',13,'R//R0']])
set_param([sys,'/',['Subsystem3/Dcalage',13,'angulaire',13,'R//R0']],...
		'ForeGround',1,...
		'BackGround',2,...
		'MATLAB Fcn','decalRR0(u)',...
		'Output Width','1',...
		'position',[395,438,455,472])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace6'])
set_param([sys,'/','Subsystem3/To Workspace6'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'mat-name','angle_rot_R0_to_R',...
		'buffer','points_trace',...
		'position',[505,441,625,469])

add_block('built-in/Sum',[sys,'/','Subsystem3/Sum2'])
set_param([sys,'/','Subsystem3/Sum2'],...
		'ForeGround',1,...
		'BackGround',2,...
		'position',[700,504,735,586])

add_block('built-in/Sum',[sys,'/','Subsystem3/Sum3'])
set_param([sys,'/','Subsystem3/Sum3'],...
		'ForeGround',1,...
		'BackGround',4,...
		'position',[705,354,740,436])

add_block('built-in/Gain',[sys,'/','Subsystem3/Gain4'])
set_param([sys,'/','Subsystem3/Gain4'],...
		'ForeGround',1,...
		'BackGround',0,...
		'position',[605,285,650,315])

add_block('built-in/Product',[sys,'/','Subsystem3/Product'])
set_param([sys,'/','Subsystem3/Product'],...
		'inputs','3',...
		'position',[860,365,885,725])

add_block('built-in/Note',[sys,'/',['Subsystem3/ARRT SUR',13,'LES VITESSES']])
set_param([sys,'/',['Subsystem3/ARRT SUR',13,'LES VITESSES']],...
		'position',[795,350,800,355])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant3'])
set_param([sys,'/','Subsystem3/Constant3'],...
		'ForeGround',1,...
		'BackGround',4,...
		'Value','2',...
		'position',[460,289,535,311])

add_block('built-in/Demux',[sys,'/','Subsystem3/Demux'])
set_param([sys,'/','Subsystem3/Demux'],...
		'outputs','[3 4]',...
		'position',[220,440,270,560])

add_block('built-in/Inport',[sys,'/','Subsystem3/in_1'])
set_param([sys,'/','Subsystem3/in_1'],...
		'position',[105,490,125,510])

add_block('built-in/Note',[sys,'/','Subsystem3/Entre pqrQ'])
set_param([sys,'/','Subsystem3/Entre pqrQ'],...
		'position',[130,535,135,540])

add_block('built-in/Note',[sys,'/','Subsystem3/Q'])
set_param([sys,'/','Subsystem3/Q'],...
		'position',[325,535,330,540])

add_block('built-in/Note',[sys,'/','Subsystem3/pqr'])
set_param([sys,'/','Subsystem3/pqr'],...
		'position',[295,450,300,455])

add_block('built-in/MATLAB Fcn',[sys,'/',['Subsystem3/fonction ',13,'d''arrt']])
set_param([sys,'/',['Subsystem3/fonction ',13,'d''arrt']],...
		'ForeGround',1,...
		'BackGround',4,...
		'MATLAB Fcn','stopderv(u)',...
		'Output Width','1',...
		'position',[360,348,420,382])

add_block('built-in/Note',[sys,'/',['Subsystem3/Suppression du test ',13,'sur les angles']])
set_param([sys,'/',['Subsystem3/Suppression du test ',13,'sur les angles']],...
		'position',[605,830,610,835])

add_block('built-in/Note',[sys,'/','Subsystem3/STOP SIMULATION'])
set_param([sys,'/','Subsystem3/STOP SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',24,...
		'position',[595,5,600,10])

add_block('built-in/Note',[sys,'/',['Subsystem3/NB : La mise  1 d''un des gains 2, 3, 4 mettra hors jeu le contrle correspondant ',13,'La mise  1 de tous les gains empche la simulation de dmarrer']])
set_param([sys,'/',['Subsystem3/NB : La mise  1 d''un des gains 2, 3, 4 mettra hors jeu le contrle correspondant ',13,'La mise  1 de tous les gains empche la simulation de dmarrer']],...
		'ForeGround',3,...
		'Font Angle','italic')
set_param([sys,'/',['Subsystem3/NB : La mise  1 d''un des gains 2, 3, 4 mettra hors jeu le contrle correspondant ',13,'La mise  1 de tous les gains empche la simulation de dmarrer']],...
		'Font Size',18,...
		'position',[600,70,605,75])

add_block('built-in/Fcn',[sys,'/','Subsystem3/Fcn'])
set_param([sys,'/','Subsystem3/Fcn'],...
		'ForeGround',1,...
		'BackGround',3,...
		'Expr','(u(1)^2+u(2)^2+u(3)^2)^0.5',...
		'position',[965,674,1040,696])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem3/MATLAB Fcn2'])
set_param([sys,'/','Subsystem3/MATLAB Fcn2'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','angles(u)',...
		'Output Width','3',...
		'position',[330,721,420,759])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem3/MATLAB Fcn'])
set_param([sys,'/','Subsystem3/MATLAB Fcn'],...
		'hide name',0,...
		'MATLAB Fcn','anglmax(u)',...
		'Output Width','1',...
		'position',[500,723,560,757])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace7'])
set_param([sys,'/','Subsystem3/To Workspace7'],...
		'hide name',0,...
		'mat-name','rien',...
		'buffer','points_trace',...
		'position',[1070,670,1125,700])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace8'])
set_param([sys,'/','Subsystem3/To Workspace8'],...
		'hide name',0,...
		'mat-name','stop_multiple',...
		'buffer','points_trace',...
		'position',[1010,452,1135,478])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace9'])
set_param([sys,'/','Subsystem3/To Workspace9'],...
		'hide name',0,...
		'mat-name','stop1',...
		'buffer','points_trace',...
		'position',[795,440,850,470])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace10'])
set_param([sys,'/','Subsystem3/To Workspace10'],...
		'hide name',0,...
		'mat-name','stop2',...
		'buffer','points_trace',...
		'position',[795,585,850,615])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace11'])
set_param([sys,'/','Subsystem3/To Workspace11'],...
		'hide name',0,...
		'mat-name','stop3',...
		'buffer','points_trace',...
		'position',[810,800,865,830])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant'])
set_param([sys,'/','Subsystem3/Constant'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Value','angle_quater_maxi',...
		'position',[430,524,570,546])

add_block('built-in/Note',[sys,'/',['Subsystem3/En effet, quand le gain vaut 1, la somme  ( sum1 ou sum2 ou sum3 ) est >=2, donc le test n''est plus pris en compte']])
set_param([sys,'/',['Subsystem3/En effet, quand le gain vaut 1, la somme  ( sum1 ou sum2 ou sum3 ) est >=2, donc le test n''est plus pris en compte']],...
		'position',[575,135,580,140])
add_line([sys,'/','Subsystem3'],[545,660;575,660])
add_line([sys,'/','Subsystem3'],[540,300;600,300])
add_line([sys,'/','Subsystem3'],[520,885;595,885])
add_line([sys,'/','Subsystem3'],[745,780;830,780;830,665;855,665])
add_line([sys,'/','Subsystem3'],[745,395;830,395;830,425;855,425])
add_line([sys,'/','Subsystem3'],[740,545;855,545])
add_line([sys,'/','Subsystem3'],[655,300;665,300;665,375;700,375])
add_line([sys,'/','Subsystem3'],[425,365;580,365;580,415;700,415])
add_line([sys,'/','Subsystem3'],[630,660;670,660;670,565;695,565])
add_line([sys,'/','Subsystem3'],[650,525;695,525])
add_line([sys,'/','Subsystem3'],[650,885;680,885;680,800;700,800])
add_line([sys,'/','Subsystem3'],[650,760;700,760])
add_line([sys,'/','Subsystem3'],[460,455;500,455])
add_line([sys,'/','Subsystem3'],[475,455;475,510;615,510])
add_line([sys,'/','Subsystem3'],[275,530;365,530;365,455;390,455])
add_line([sys,'/','Subsystem3'],[890,545;990,545])
add_line([sys,'/','Subsystem3'],[555,775;610,775])
add_line([sys,'/','Subsystem3'],[575,535;615,535])
add_line([sys,'/','Subsystem3'],[130,500;215,500])
add_line([sys,'/','Subsystem3'],[290,530;290,740;325,740])
add_line([sys,'/','Subsystem3'],[175,500;175,365;355,365])
add_line([sys,'/','Subsystem3'],[1045,685;1065,685])
add_line([sys,'/','Subsystem3'],[425,740;495,740])
add_line([sys,'/','Subsystem3'],[460,740;460,685;960,685])
add_line([sys,'/','Subsystem3'],[565,740;610,740])
add_line([sys,'/','Subsystem3'],[935,545;935,465;1005,465])
add_line([sys,'/','Subsystem3'],[765,395;765,455;790,455])
add_line([sys,'/','Subsystem3'],[780,545;790,600])
add_line([sys,'/','Subsystem3'],[780,780;780,815;805,815])
set_param([sys,'/','Subsystem3'],...
		'Mask Display','STOP MULTIPLE\nCHOIX POSSIBLES\nmis en sommeil')


%     Finished composite block 'Subsystem3'.

set_param([sys,'/','Subsystem3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'position',[860,376,1100,444])


%     Subsystem  'Subsystem11'.

new_system([sys,'/','Subsystem11'])
set_param([sys,'/','Subsystem11'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem11/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem11/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem11/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem11/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem11/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem11/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem11/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem11/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem11/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem11/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem11/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem11/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem11/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem11/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem11/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem11/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem11/d) Lancer la simulation'])
set_param([sys,'/','Subsystem11/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem11/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem11/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem11/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem11/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem11/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem11/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem11/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem11/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem11/b) '])
set_param([sys,'/','Subsystem11/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem11/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem11/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem11/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem11/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem11/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem11/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem11/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem11/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem11/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem11'],...
		'Mask Display','2 - Roue BCT\nH=0.005 mN\n 1665 tours/mn',...
		'Mask Dialogue','eval(''H=0.005'')')


%     Finished composite block 'Subsystem11'.

set_param([sys,'/','Subsystem11'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[540,740,755,831])


%     Subsystem  'Subsystem10'.

new_system([sys,'/','Subsystem10'])
set_param([sys,'/','Subsystem10'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem10/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem10/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem10/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem10/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem10/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem10/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem10/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem10/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem10/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem10/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem10/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem10/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem10/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem10/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem10/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem10/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem10/d) Lancer la simulation'])
set_param([sys,'/','Subsystem10/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem10/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem10/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem10/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem10/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem10/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem10/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem10/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem10/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem10/b) '])
set_param([sys,'/','Subsystem10/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem10/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem10/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem10/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem10/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem10/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem10/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem10/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem10/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem10/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem10'],...
		'Mask Display','2 - Roue BCT\nH=0.001 mN\n 333 tours/mn',...
		'Mask Dialogue','eval(''H=0.001'')')


%     Finished composite block 'Subsystem10'.

set_param([sys,'/','Subsystem10'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[535,640,750,731])


%     Subsystem  'Subsystem6'.

new_system([sys,'/','Subsystem6'])
set_param([sys,'/','Subsystem6'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem6/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem6/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem6/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem6/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem6/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem6/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem6/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem6/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem6/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem6/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem6/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem6/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem6/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem6/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem6/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem6/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem6/d) Lancer la simulation'])
set_param([sys,'/','Subsystem6/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem6/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem6/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem6/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem6/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem6/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem6/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem6/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem6/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem6/b) '])
set_param([sys,'/','Subsystem6/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem6/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem6/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem6/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem6/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem6/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem6/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem6'],...
		'Mask Display','2 - Roue AOCS\nH=0.00058 mNs\ntype A\nWheel  8000 tr/mn',...
		'Mask Dialogue','eval(''H=0.00058'')')


%     Finished composite block 'Subsystem6'.

set_param([sys,'/','Subsystem6'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[785,745,1000,836])


%     Subsystem  'Subsystem7'.

new_system([sys,'/','Subsystem7'])
set_param([sys,'/','Subsystem7'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem7/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem7/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem7/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem7/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem7/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem7/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem7/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem7/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem7/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem7/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem7/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem7/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem7/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem7/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem7/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem7/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem7/d) Lancer la simulation'])
set_param([sys,'/','Subsystem7/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem7/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem7/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem7/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem7/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem7/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem7/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem7/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem7/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem7/b) '])
set_param([sys,'/','Subsystem7/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem7/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem7/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem7/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem7/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem7/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem7/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem7'],...
		'Mask Display','2 - Roue BCT\nH=0.002 mN\n 666 tours/mn',...
		'Mask Dialogue','eval(''H=0.002'')')


%     Finished composite block 'Subsystem7'.

set_param([sys,'/','Subsystem7'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[785,640,1000,731])


%     Subsystem  'Subsystem9'.

new_system([sys,'/','Subsystem9'])
set_param([sys,'/','Subsystem9'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem9/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem9/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem9/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem9/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem9/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem9/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem9/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem9/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem9/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem9/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem9/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem9/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem9/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem9/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem9/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem9/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem9/d) Lancer la simulation'])
set_param([sys,'/','Subsystem9/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem9/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem9/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem9/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem9/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem9/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem9/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem9/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem9/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem9/b) '])
set_param([sys,'/','Subsystem9/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem9/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem9/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem9/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem9/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem9/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem9/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem9/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem9/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem9/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem9'],...
		'Mask Display','2- Roue\nH=0.003 mNs\nBCT Micro Reaction\nWheel  1000 tr/mn',...
		'Mask Dialogue','eval(''H=0.003'')')


%     Finished composite block 'Subsystem9'.

set_param([sys,'/','Subsystem9'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1025,745,1240,836])


%     Subsystem  'Subsystem8'.

new_system([sys,'/','Subsystem8'])
set_param([sys,'/','Subsystem8'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem8/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem8/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem8/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem8/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem8/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem8/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem8/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem8/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem8/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem8/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem8/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem8/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem8/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem8/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem8/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem8/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem8/d) Lancer la simulation'])
set_param([sys,'/','Subsystem8/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem8/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem8/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem8/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem8/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem8/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem8/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem8/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem8/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem8/b) '])
set_param([sys,'/','Subsystem8/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem8/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem8/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem8/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem8/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem8/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem8/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem8/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem8/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem8/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem8'],...
		'Mask Display','2 - Roue\nH=0.009 mNs\nBCT Micro Reaction\nWheel  3000 tr/mn',...
		'Mask Dialogue','eval(''H=0.009'')')


%     Finished composite block 'Subsystem8'.

set_param([sys,'/','Subsystem8'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1020,645,1235,736])


%     Subsystem  'Subsystem5'.

new_system([sys,'/','Subsystem5'])
set_param([sys,'/','Subsystem5'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem5/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem5/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem5/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem5/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem5/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem5/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem5/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem5/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem5/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem5/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem5/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem5/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem5/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem5/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem5/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem5/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem5/d) Lancer la simulation'])
set_param([sys,'/','Subsystem5/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem5/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem5/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem5/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem5/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem5/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem5/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem5/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem5/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem5/b) '])
set_param([sys,'/','Subsystem5/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem5/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem5/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem5/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem5/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem5/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem5/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem5'],...
		'Mask Display','2 - Roue\nH=0.018 mNs\nBCT Micro Reaction\nWheel  6000 tr/mn',...
		'Mask Dialogue','eval(''H=0.018'')')


%     Finished composite block 'Subsystem5'.

set_param([sys,'/','Subsystem5'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1015,545,1230,636])


%     Subsystem  'Subsystem4'.

new_system([sys,'/','Subsystem4'])
set_param([sys,'/','Subsystem4'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem4/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem4/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem4/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem4/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem4/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem4/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem4/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem4/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem4/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem4/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem4/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem4/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem4/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem4/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem4/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem4/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem4/d) Lancer la simulation'])
set_param([sys,'/','Subsystem4/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem4/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem4/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem4/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem4/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem4/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem4/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem4/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem4/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem4/b) '])
set_param([sys,'/','Subsystem4/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem4/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem4/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem4/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem4/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem4/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem4/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem4'],...
		'Mask Display','2 - La rouede mes\nsimulations\nH=3.2 e-3 mNs',...
		'Mask Dialogue','eval(''H=0.0032'')')


%     Finished composite block 'Subsystem4'.

set_param([sys,'/','Subsystem4'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[785,545,1000,631])

add_block('built-in/Note',[sys,'/',['L''initialisation lance nanodat0.m, puls par dfaut',13,'']])
set_param([sys,'/',['L''initialisation lance nanodat0.m, puls par dfaut',13,'']],...
		'BackGround',6,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[985,510,990,515])


%     Subsystem  'Extras1'.

new_system([sys,'/','Extras1'])
set_param([sys,'/','Extras1'],'Location',[4,42,1012,678])
set_param([sys,'/','Extras1'],...
		'Mask Display','1 - INITIALISATION\npar double clic souris\nCAS GENERAL',...
		'Mask Dialogue','eval(''nanodat0'')')


%     Finished composite block 'Extras1'.

set_param([sys,'/','Extras1'],...
		'ForeGround',1,...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[855,450,1100,511])


%     Subsystem  'Subsystem'.

new_system([sys,'/','Subsystem'])
set_param([sys,'/','Subsystem'],'Location',[4,112,1260,1004])

add_block('built-in/Note',[sys,'/',['Subsystem/SORTIES ET TRAITEMENT DE LA SORTIE',13,'POUR VERIFICATIONS DE FONCTIONNALITE']])
set_param([sys,'/',['Subsystem/SORTIES ET TRAITEMENT DE LA SORTIE',13,'POUR VERIFICATIONS DE FONCTIONNALITE']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[620,40,625,45])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux'])
set_param([sys,'/','Subsystem/Demux'],...
		'outputs','[3 4]',...
		'position',[470,339,520,506])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn1'])
set_param([sys,'/','Subsystem/MATLAB Fcn1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','angles(u)',...
		'Output Width','3',...
		'position',[700,448,760,482])

add_block('built-in/Note',[sys,'/','Subsystem/Energie'])
set_param([sys,'/','Subsystem/Energie'],...
		'position',[885,390,890,395])

add_block('built-in/Note',[sys,'/',['Subsystem/Rotation atellite par',13,'rapport aux axes orbitaux']])
set_param([sys,'/',['Subsystem/Rotation atellite par',13,'rapport aux axes orbitaux']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[905,670,910,675])

add_block('built-in/Note',[sys,'/','Subsystem/Quaternion Q'])
set_param([sys,'/','Subsystem/Quaternion Q'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[600,435,605,440])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn3'])
set_param([sys,'/','Subsystem/MATLAB Fcn3'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','ENERGIE(u)',...
		'Output Width','1',...
		'position',[745,361,825,399])

add_block('built-in/Note',[sys,'/',['Subsystem/Entre de',13,'p q r Q']])
set_param([sys,'/',['Subsystem/Entre de',13,'p q r Q']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[420,365,425,370])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux3'])
set_param([sys,'/','Subsystem/Demux3'],...
		'outputs','3',...
		'position',[610,158,655,262])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace15'])
set_param([sys,'/','Subsystem/To Workspace15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','p',...
		'buffer','points_trace',...
		'position',[715,165,785,185])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace16'])
set_param([sys,'/','Subsystem/To Workspace16'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','q',...
		'buffer','points_trace',...
		'position',[715,200,785,220])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace17'])
set_param([sys,'/','Subsystem/To Workspace17'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','r',...
		'buffer','points_trace',...
		'position',[715,235,785,255])

add_block('built-in/Note',[sys,'/',['Subsystem/Vitesses angulaires absolues',13,'autour des axes x y z']])
set_param([sys,'/',['Subsystem/Vitesses angulaires absolues',13,'autour des axes x y z']],...
		'position',[895,185,900,190])


%     Subsystem  ['Subsystem/Vitesses angulaires',13,'rd//s'].

new_system([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],'Location',[0,59,274,252])

add_block('built-in/Inport',[sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/x']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/x']],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfuny',...
		'parameters','ax, color,dt',...
		'position',[130,55,180,75])
add_line([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],[90,65;125,65])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter plotting ranges and line type.|Time range:|y-min:|y-max:|Line type (rgbw-:*). Seperate each plot by ''/'':')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Translate','color = @4; ax = [0, @1, @2, @3]; dt = -1;')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Scope block. Look at the m-file sfuny.m to see how it works. This block can take scalar or vector input signal.')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Entries','4000\/-0.2\/0.2\/''y-/g--/c-./w:/m*/ro/b+''\/')


%     Finished composite block ['Subsystem/Vitesses angulaires',13,'rd//s'].

set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'orientation',2,...
		'BackGround',2,...
		'position',[365,192,475,228])

add_block('built-in/Inport',[sys,'/','Subsystem/in_1'])
set_param([sys,'/','Subsystem/in_1'],...
		'position',[290,415,310,435])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace7'])
set_param([sys,'/','Subsystem/To Workspace7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','energie',...
		'buffer','points_trace',...
		'position',[945,370,1015,390])

add_block('built-in/Gain',[sys,'/',['Subsystem/Conversion',13,'rd // degrs']])
set_param([sys,'/',['Subsystem/Conversion',13,'rd // degrs']],...
		'Gain','180/pi',...
		'position',[795,452,820,478])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn2'])
set_param([sys,'/','Subsystem/MATLAB Fcn2'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','vibomega(u)',...
		'Output Width','1',...
		'position',[660,108,720,142])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace9'])
set_param([sys,'/','Subsystem/To Workspace9'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','GDomega',...
		'buffer','points_trace',...
		'position',[775,115,845,135])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace18'])
set_param([sys,'/','Subsystem/To Workspace18'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'mat-name','anglo',...
		'buffer','points_reduits',...
		'position',[950,310,1020,330])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace19'])
set_param([sys,'/','Subsystem/To Workspace19'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'mat-name','vitesso',...
		'buffer','points_reduits',...
		'position',[645,295,715,315])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace10'])
set_param([sys,'/','Subsystem/To Workspace10'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'mat-name','tempo',...
		'buffer','points_reduits',...
		'position',[575,665,645,685])

add_block('built-in/Clock',[sys,'/','Subsystem/Clock'])
set_param([sys,'/','Subsystem/Clock'],...
		'position',[510,615,530,635])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace8'])
set_param([sys,'/','Subsystem/To Workspace8'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','temps',...
		'buffer','points_trace+points_reduits',...
		'position',[570,615,640,635])


%     Subsystem  'Subsystem/Angles en degrs'.

new_system([sys,'/','Subsystem/Angles en degrs'])
set_param([sys,'/','Subsystem/Angles en degrs'],'Location',[0,59,274,252])

add_block('built-in/Inport',[sys,'/','Subsystem/Angles en degrs/x'])
set_param([sys,'/','Subsystem/Angles en degrs/x'],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['Subsystem/Angles en degrs/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Subsystem/Angles en degrs/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfuny',...
		'parameters','ax, color,dt',...
		'position',[130,55,180,75])
add_line([sys,'/','Subsystem/Angles en degrs'],[90,65;125,65])
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter plotting ranges and line type.|Time range:|y-min:|y-max:|Line type (rgbw-:*). Seperate each plot by ''/'':')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Translate','color = @4; ax = [0, @1, @2, @3]; dt = -1;')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Scope block. Look at the m-file sfuny.m to see how it works. This block can take scalar or vector input signal.')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Entries','5000\/-200\/200\/''y-/g--/c-./w:/m*/ro/b+''\/')


%     Finished composite block 'Subsystem/Angles en degrs'.

set_param([sys,'/','Subsystem/Angles en degrs'],...
		'BackGround',2,...
		'position',[930,522,1040,558])
add_line([sys,'/','Subsystem'],[525,380;740,380])
add_line([sys,'/','Subsystem'],[830,380;940,380])
add_line([sys,'/','Subsystem'],[315,425;465,425])
add_line([sys,'/','Subsystem'],[525,465;695,465])
add_line([sys,'/','Subsystem'],[535,625;565,625])
add_line([sys,'/','Subsystem'],[765,465;790,465])
add_line([sys,'/','Subsystem'],[550,380;550,210;605,210])
add_line([sys,'/','Subsystem'],[660,175;710,175])
add_line([sys,'/','Subsystem'],[660,210;710,210])
add_line([sys,'/','Subsystem'],[660,245;710,245])
add_line([sys,'/','Subsystem'],[550,210;480,210])
add_line([sys,'/','Subsystem'],[825,465;840,465;840,540;925,540])
add_line([sys,'/','Subsystem'],[550,210;550,125;655,125])
add_line([sys,'/','Subsystem'],[725,125;770,125])
add_line([sys,'/','Subsystem'],[545,625;545,675;570,675])
add_line([sys,'/','Subsystem'],[825,465;840,465;840,320;945,320])
add_line([sys,'/','Subsystem'],[580,210;580,305;640,305])
set_param([sys,'/','Subsystem'],...
		'Mask Display','SORTIE DES ANGLES\n\nEN DEGRES & ENERGIE')


%     Finished composite block 'Subsystem'.

set_param([sys,'/','Subsystem'],...
		'hide name',0,...
		'position',[845,259,1085,341])

add_block('built-in/Note',[sys,'/',['Vecteur d''tat de 7 composantes']])
set_param([sys,'/',['Vecteur d''tat de 7 composantes']],...
		'position',[365,560,370,565])


%     Subsystem  'UnNamed2'.

new_system([sys,'/','UnNamed2'])
set_param([sys,'/','UnNamed2'],'Location',[588,239,931,333])

add_block('built-in/Note',[sys,'/',['UnNamed2/Drive du quaternion d''attitude']])
set_param([sys,'/',['UnNamed2/Drive du quaternion d''attitude']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[185,5,190,10])

add_block('built-in/Outport',[sys,'/','UnNamed2/Outport'])
set_param([sys,'/','UnNamed2/Outport'],...
		'position',[260,45,280,65])

add_block('built-in/Inport',[sys,'/','UnNamed2/Inport'])
set_param([sys,'/','UnNamed2/Inport'],...
		'position',[80,45,100,65])

add_block('built-in/MATLAB Fcn',[sys,'/','UnNamed2/deriv_qt.m'])
set_param([sys,'/','UnNamed2/deriv_qt.m'],...
		'MATLAB Fcn','deriv_qt(u)',...
		'Output Width','4',...
		'position',[150,39,220,71])
add_line([sys,'/','UnNamed2'],[225,55;255,55])
add_line([sys,'/','UnNamed2'],[105,55;145,55])
set_param([sys,'/','UnNamed2'],...
		'Mask Display','F(X , t)\n\nPOUR LE QUATERNION')


%     Finished composite block 'UnNamed2'.

set_param([sys,'/','UnNamed2'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[120,443,335,547])

add_block('built-in/Note',[sys,'/','3'])
set_param([sys,'/','3'],...
		'position',[515,330,520,335])

add_block('built-in/Note',[sys,'/','4'])
set_param([sys,'/','4'],...
		'position',[510,470,515,475])


%     Subsystem  'F(X,t) pour la rotation'.

new_system([sys,'/','F(X,t) pour la rotation'])
set_param([sys,'/','F(X,t) pour la rotation'],'Location',[4,42,1268,968])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/pqr'])
set_param([sys,'/','F(X,t) pour la rotation/pqr'],...
		'position',[325,375,330,380])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Demux'])
set_param([sys,'/','F(X,t) pour la rotation/Demux'],...
		'outputs','[3 4]',...
		'position',[225,375,275,495])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/tau',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/tau',13,'']],...
		'position',[85,240,90,245])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Vecteur d''tat ',13,'7 composantes',13,'p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Vecteur d''tat ',13,'7 composantes',13,'p q r Q']],...
		'position',[105,335,110,340])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Inport'])
set_param([sys,'/','F(X,t) pour la rotation/Inport'],...
		'position',[75,300,95,320])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Suppression de',13,'toutes les',13,'perturbations']])
set_param([sys,'/',['F(X,t) pour la rotation/Suppression de',13,'toutes les',13,'perturbations']],...
		'position',[515,325,520,330])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']])
set_param([sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',22)
set_param([sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']],...
		'position',[630,40,635,45])


%     Subsystem  ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1'].

new_system([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],'Location',[-100,71,1167,677])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/B']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/B']],...
		'position',[510,425,515,430])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Vecteur d''tat',13,'7 composantes',13,'p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Vecteur d''tat',13,'7 composantes',13,'p q r Q']],...
		'position',[140,515,145,520])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain3']],...
		'hide name',0,...
		'Gain','-1',...
		'position',[800,382,835,408])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple magntique rsiduel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple magntique rsiduel']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[880,399,885,404])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple arodynamique']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple arodynamique']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[690,277,695,282])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel1']],...
		'MATLAB Fcn','norm(u,2)',...
		'Output Width','1',...
		'position',[770,438,830,472])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain']],...
		'Gain','mom_magn_res',...
		'position',[375,315,490,365])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace']],...
		'hide name',0,...
		'mat-name','moment_residuel',...
		'buffer','5000',...
		'position',[895,440,1015,470])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Moment magntique rsiduel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Moment magntique rsiduel']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[245,246,250,251])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel']],...
		'MATLAB Fcn','prodvect(u)',...
		'Output Width','3',...
		'position',[640,378,700,412])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux5']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux5']],...
		'inputs','[3 3]',...
		'position',[540,286,585,504])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux3']],...
		'ForeGround',1,...
		'BackGround',0,...
		'inputs','3',...
		'position',[320,288,350,392])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant12']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant12']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','alpha',...
		'position',[130,293,275,317])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant11']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant11']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','beta',...
		'position',[130,328,275,352])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant10']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant10']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','gamma',...
		'position',[130,363,275,387])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/M']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/M']],...
		'position',[525,315,530,320])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant6']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant6']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero3',...
		'position',[785,318,930,342])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant5']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant5']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero2',...
		'position',[785,283,930,307])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant4']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero1',...
		'position',[785,248,930,272])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux1']],...
		'ForeGround',1,...
		'BackGround',0,...
		'inputs','3',...
		'position',[975,243,1005,347])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Termes rsidus de gradient',13,'et forces inertie']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Termes rsidus de gradient',13,'et forces inertie']],...
		'ForeGround',4,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[895,195,900,200])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant3']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','Irt*vitesse_rotation^2',...
		'position',[795,158,940,182])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant2']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant2']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','3*Irl*vitesse_rotation^2',...
		'position',[795,123,940,147])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant1']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','-4*Itl*vitesse_rotation^2',...
		'position',[795,88,940,112])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux']],...
		'ForeGround',7,...
		'BackGround',0,...
		'inputs','3',...
		'position',[985,83,1015,187])


%     Subsystem  ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite'].

new_system([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],'Location',[195,288,1041,757])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']],...
		'hide name',0,...
		'MATLAB Fcn','champmgn(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL',...
		'position',[250,95,425,145])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Clock']],...
		'position',[180,104,215,136])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/BX BY BZ']],...
		'position',[340,70,345,75])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Vecteur d''tat ',13,'( 7 composantes)']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Vecteur d''tat ',13,'( 7 composantes)']],...
		'position',[70,235,75,240])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/in_1']],...
		'position',[50,190,70,210])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/out_1']],...
		'position',[705,150,725,170])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Sortie Bx By Bz']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Sortie Bx By Bz']],...
		'position',[725,200,730,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Equateur au temps 0 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Equateur au temps 0 s']],...
		'position',[150,75,155,80])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn1']],...
		'MATLAB Fcn','champsat(u)',...
		'Output Width','3',...
		'position',[560,143,620,177])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Mux1']],...
		'inputs','[3 7]',...
		'position',[485,80,530,240])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[220,120;245,120])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[430,120;480,120])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[625,160;700,160])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[75,200;480,200])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[535,160;555,160])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],...
		'Mask Display','CHAMP MAGNETIQUE EN\nAXES SATELLITE\nEntre=vecteur d''tat\nSortie Bx By Bz')


%     Finished composite block ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite'].

set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],...
		'position',[240,458,440,532])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'position',[385,95,390,100])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Inport']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Inport']],...
		'position',[20,485,40,505])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Sum']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Sum']],...
		'inputs','++++',...
		'position',[1090,254,1125,361])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Outport']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Outport']],...
		'position',[1200,300,1220,320])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'ForeGround',2,...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'position',[665,505,670,510])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Partie principale du gradient de gravit']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Partie principale du gradient de gravit']],...
		'position',[945,540,950,545])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'MATLAB Fcn','gradgrav(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']],...
		'Mask Display','GRADIENT PRINCIPAL\n\nDE GRAVITE',...
		'position',[630,554,805,606])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace1']],...
		'hide name',0,...
		'mat-name','perturb',...
		'buffer','points_trace',...
		'position',[1180,188,1235,212])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[45,495;235,495])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[840,395;1055,395;1055,320;1085,320])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[705,395;735,395;735,455;765,455])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[735,395;795,395])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[835,455;890,455])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[590,395;635,395])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[445,495;470,495;470,450;535,450])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[495,340;535,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[355,340;370,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,305;315,305])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,340;315,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,375;315,375])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,100;980,100])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,135;980,135])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,170;980,170])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,260;970,260])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,295;970,295])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1010,295;1085,295])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,330;970,330])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1020,135;1075,135;1085,270])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[75,495;75,580;625,580])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1130,310;1195,310])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[810,580;1070,580;1070,345;1085,345])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1155,310;1155,200;1175,200])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],...
		'Mask Display','Couples\nperturbateurs Mp')


%     Finished composite block ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1'].

set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],...
		'ForeGround',2,...
		'hide name',0,...
		'position',[285,282,415,338])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain3'])
set_param([sys,'/','F(X,t) pour la rotation/Gain3'],...
		'hide name',0,...
		'position',[885,248,925,282])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/out_1'],...
		'position',[1125,325,1145,345])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain1'])
set_param([sys,'/','F(X,t) pour la rotation/Gain1'],...
		'hide name',0,...
		'position',[1070,322,1095,348])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Sum1'])
set_param([sys,'/','F(X,t) pour la rotation/Sum1'],...
		'position',[970,194,1010,476])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Suppression de',13,'tous les couples']])
set_param([sys,'/',['F(X,t) pour la rotation/Suppression de',13,'tous les couples']],...
		'position',[900,205,905,210])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain'])
set_param([sys,'/','F(X,t) pour la rotation/Gain'],...
		'hide name',0,...
		'position',[460,293,500,327])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Sum2'])
set_param([sys,'/','F(X,t) pour la rotation/Sum2'],...
		'position',[680,128,720,372])


%     Subsystem  'F(X,t) pour la rotation/Subsystem5'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5'],'Location',[298,510,1135,815])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem5/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/out_1'],...
		'position',[635,140,655,160])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem5/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/in_1'],...
		'position',[65,100,85,120])


%     Subsystem  'F(X,t) pour la rotation/Subsystem5/Subsystem5'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],'Location',[580,281,938,501])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace3'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'mat-name','valeur_creneau',...
		'buffer','points_trace',...
		'position',[245,178,340,192])

add_block('built-in/Clock',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Clock'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Clock'],...
		'BackGround',7,...
		'position',[45,113,80,147])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/MATLAB Fcn'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/MATLAB Fcn'],...
		'MATLAB Fcn','creneau(u)',...
		'Output Width','1',...
		'position',[120,113,180,147])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/in_1'],...
		'position',[115,75,135,95])

add_block('built-in/Product',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Product'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Product'],...
		'hide name',0,...
		'position',[230,64,250,151])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/out_1'],...
		'position',[295,100,315,120])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/CRENEAU'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/CRENEAU'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[155,25,160,30])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace4'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace4'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'mat-name','temps_creneau',...
		'buffer','points_trace',...
		'position',[125,193,220,207])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[185,130;225,130])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[205,130;205,185;240,185])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[85,130;115,130])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[255,110;290,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[140,85;225,85])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[85,130;85,195;120,200])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],...
		'Mask Display','COUPURES\nMAGNETOMETRES\nDE MESURES')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem5/Subsystem5'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'position',[190,84,315,136])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem5/Gain2'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Gain2'],...
		'hide name',0,...
		'Gain','pulse',...
		'position',[340,90,475,130])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Subsystem5/Sum1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Sum1'],...
		'hide name',0,...
		'position',[525,69,545,231])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem5/Gain1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Gain1'],...
		'hide name',0,...
		'Gain','1-pulse',...
		'position',[230,170,365,210])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[90,110;125,110;125,190;225,190])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[550,150;630,150])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[320,110;335,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[370,190;520,190])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[480,110;520,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[90,110;185,110])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5'],...
		'Mask Display','PULSE\noui/non')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem5'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem5'],...
		'hide name',0,...
		'position',[560,166,645,214])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain2'])
set_param([sys,'/','F(X,t) pour la rotation/Gain2'],...
		'position',[485,175,530,205])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Q'])
set_param([sys,'/','F(X,t) pour la rotation/Q'],...
		'position',[280,445,285,450])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Couple roue unidimensionnel sur tangage'])
set_param([sys,'/','F(X,t) pour la rotation/Couple roue unidimensionnel sur tangage'],...
		'position',[705,435,710,440])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/out_2'])
set_param([sys,'/','F(X,t) pour la rotation/out_2'],...
		'Port','2',...
		'position',[1080,495,1100,515])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Sortie vitesse roue'])
set_param([sys,'/','F(X,t) pour la rotation/Sortie vitesse roue'],...
		'position',[985,515,990,520])

add_block('built-in/State-Space',[sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']])
set_param([sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'A','[]',...
		'B','[]',...
		'C','[]',...
		'D','inv(Asat)',...
		'Mask Display','Inv(Asat)',...
		'Mask Type','Matrix Gain')
set_param([sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'Mask Help','Multiplies input vector by entered matrix to produce output vector (y=Au).',...
		'position',[750,242,850,288])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/magneto.m1'])
set_param([sys,'/','F(X,t) pour la rotation/magneto.m1'],...
		'hide name',0,...
		'MATLAB Fcn','derivrot(u)',...
		'Output Width','3',...
		'Mask Display','DERIVROT(u)',...
		'position',[375,379,500,431])


%     Subsystem  'F(X,t) pour la rotation/Calcul des commandes, moments et roue'.

new_system([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],'Location',[181,42,1232,550])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Size',18)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'position',[595,10,600,15])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'ForeGround',2,...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'Font Size',14,...
		'position',[605,82,610,87])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple roue uniaxial'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple roue uniaxial'],...
		'position',[755,345,760,350])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple total magntique + roue',13,'de la rgulation de Laurens amliore']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple total magntique + roue',13,'de la rgulation de Laurens amliore']],...
		'position',[780,280,785,285])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],'Location',[4,42,1268,968])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','cpl_laurens2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']],...
		'buffer','points_trace',...
		'position',[1030,324,1150,346])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'ForeGround',1,...
		'BackGround',2)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'MATLAB Fcn','prodvect(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'position',[1065,418,1125,452])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],'Location',[442,254,1087,454])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/in_1']],...
		'hide name',0,...
		'position',[25,100,45,120])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_3']],...
		'hide name',0,...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_3']],...
		'position',[360,130,380,150])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_2']],...
		'hide name',0,...
		'position',[360,70,380,90])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_1']],...
		'hide name',0,...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_1']],...
		'position',[360,100,380,120])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],'Location',[76,147,1226,812])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/SIMULATION D''UN RETARD ORBITAL']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/SIMULATION D''UN RETARD ORBITAL']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/SIMULATION D''UN RETARD ORBITAL']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/SIMULATION D''UN RETARD ORBITAL']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/SIMULATION D''UN RETARD ORBITAL']],...
		'Font Size',20)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/SIMULATION D''UN RETARD ORBITAL']],...
		'position',[520,10,525,15])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/in_1']],...
		'position',[160,305,180,325])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite '].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],'Location',[382,182,1228,651])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn3']],...
		'MATLAB Fcn','champmgn(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn3']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn3']],...
		'position',[250,95,425,145])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Clock']],...
		'position',[180,104,215,136])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /BX BY BZ']],...
		'position',[340,70,345,75])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Vecteur d''tat ',13,'( 7 composantes)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Vecteur d''tat ',13,'( 7 composantes)']],...
		'position',[70,235,75,240])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /in_1']],...
		'position',[50,190,70,210])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /out_1']],...
		'position',[705,150,725,170])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Sortie Bx By Bz']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Sortie Bx By Bz']],...
		'position',[725,200,730,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Equateur au temps 0 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Equateur au temps 0 s']],...
		'position',[150,75,155,80])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn1']],...
		'MATLAB Fcn','champsat(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /MATLAB Fcn1']],...
		'position',[560,143,620,177])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Mux1']],...
		'inputs','[3 7]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite /Mux1']],...
		'position',[485,80,530,240])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],[220,120;245,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],[430,120;480,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],[625,160;700,160])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],[75,200;480,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],[535,160;555,160])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],...
		'Mask Display','CHAMP MAGNETIQUE EN\nAXES SATELLITE\nEntre=vecteur d''tat\nSortie Bx By Bz')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite '].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite ']],...
		'position',[240,278,440,352])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ rel']],...
		'position',[870,375,875,380])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ puls liss linairement ( mesure )']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ puls liss linairement ( mesure )']],...
		'position',[950,330,955,335])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Drive pulse crneau(mesure )']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Drive pulse crneau(mesure )']],...
		'position',[930,290,935,295])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],'Location',[4,386,1102,910])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_1']],...
		'position',[115,190,135,210])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ rel satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ rel satellite']],...
		'position',[690,55,695,60])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ  liss']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ  liss']],...
		'position',[775,115,780,120])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive pulse non lisse']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive pulse non lisse']],...
		'position',[790,425,795,430])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_3']],...
		'position',[800,390,820,410])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_1']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_1']],...
		'position',[780,45,800,65])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_2']],...
		'position',[790,145,810,165])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Sum']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Sum']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Sum']],...
		'position',[640,134,665,321])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],'Location',[4,50,853,441])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_2']],...
		'position',[95,250,115,270])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Temps courant',13,'de la simulation']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Temps courant',13,'de la simulation']],...
		'position',[70,295,75,300])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],'Location',[125,1114771,765,1115016])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'position',[175,70,180,75])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']],...
		'position',[475,54,495,176])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']],...
		'MATLAB Fcn','fix(u/T)*T')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']],...
		'Output Width','1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']],...
		'position',[365,128,425,162])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']],...
		'position',[295,107,320,183])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']],...
		'Value','instant_injection')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']],...
		'position',[165,154,270,176])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Calcul t-nT1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Calcul t-nT1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Calcul t-nT1']],...
		'position',[460,206,465,211])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/out_1']],...
		'position',[525,105,545,125])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/in_1']],...
		'position',[245,115,265,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[325,145;360,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[345,145;345,85;470,85])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[275,165;290,165])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[430,145;470,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[500,115;520,115])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[270,125;290,125])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],...
		'Mask Display','Mise en place\ndes priodes')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],...
		'position',[155,236,295,284])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'mat-name','der_seconde')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'buffer','20000')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'position',[705,568,825,602])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau']],...
		'position',[785,211,790,216])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/B(n)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/B(n)']],...
		'position',[620,196,625,201])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie de t-tinj-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie de t-tinj-nT']],...
		'position',[395,310,400,315])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/out_1']],...
		'position',[715,215,735,235])

add_block('built-in/To File',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']],...
		'File name','derbchap.mat')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']],...
		'Matrix name','derb_chapeau')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']],...
		'position',[635,134,730,166])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'MATLAB Fcn','storderB(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'position',[515,208,575,242])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_1']],...
		'position',[105,135,125,155])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t-tinj -nT<=Ta']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t-tinj -nT<=Ta']],...
		'position',[390,290,395,295])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'mat-name','derb_chapeau')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'buffer','20000')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'position',[630,268,750,302])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']],...
		'Value','Ta/2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']],...
		'position',[325,189,360,211])

add_block('built-in/Relational Operator',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Relational',13,'Operator']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Relational',13,'Operator']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Relational',13,'Operator']],...
		'position',[375,168,400,292])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t--tinj -nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t--tinj -nT']],...
		'position',[335,235,340,240])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Mux1']],...
		'inputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Mux1']],...
		'position',[455,206,485,239])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie B_prime']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie B_prime']],...
		'position',[550,175,555,180])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Entre de dB//dt //xyz ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Entre de dB//dt //xyz ']],...
		'position',[355,115,360,120])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'position',[425,0,430,5])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'Font Size',12)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'position',[405,75,410,80])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'Font Size',12)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'position',[420,45,425,50])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[120,260;150,260])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[580,225;710,225])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[595,225;595,150;630,150])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[595,225;595,285;625,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[490,225;510,225])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[405,230;450,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[365,200;370,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[130,145;430,145;430,215;450,215])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[300,260;370,260])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],...
		'Mask Display','DERIVEE DE B\n\nMESUREE\n\nPERIODE n')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],...
		'position',[310,260,445,340])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Calcul t-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Calcul t-nT']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Calcul t-nT']],...
		'position',[410,385,415,390])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Product']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Product']],...
		'position',[535,227,560,323])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],'Location',[198,215,1097,749])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_1']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_1']],...
		'position',[615,220,635,240])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_3']],...
		'position',[615,100,635,120])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/in_1']],...
		'position',[65,210,85,230])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre de B(t) //xyz ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre de B(t) //xyz ']],...
		'position',[115,180,120,185])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Calcul t-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Calcul t-nT']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Calcul t-nT']],...
		'position',[265,430,270,435])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie champ B chapeau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie champ B chapeau']],...
		'position',[620,180,625,185])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t - tinj- nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t - tinj- nT']],...
		'position',[335,350,340,355])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre du champ en axes satellite ',13,' l''instant t de la simulation',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre du champ en axes satellite ',13,' l''instant t de la simulation',13,'']],...
		'position',[215,200,220,205])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Mux1']],...
		'inputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Mux1']],...
		'position',[445,211,475,244])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'MATLAB Fcn','storcham(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'position',[505,213,565,247])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_2']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_2']],...
		'position',[615,420,635,440])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']],...
		'mat-name','champ1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']],...
		'buffer','20000')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']],...
		'position',[610,272,660,288])

add_block('built-in/Relational Operator',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Relational',13,'Operator']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Relational',13,'Operator']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Relational',13,'Operator']],...
		'position',[375,258,400,382])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']],...
		'Value','Ta/2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']],...
		'position',[305,279,340,301])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie de to = t -tinj - nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie de to = t -tinj - nT']],...
		'position',[585,385,590,390])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Temps courant',13,'de la simulation']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Temps courant',13,'de la simulation']],...
		'position',[75,280,80,285])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],'Location',[163,376,803,621])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/out_1']],...
		'position',[555,105,575,125])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'Font Size',12)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'position',[315,0,320,5])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'position',[135,100,140,105])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']],...
		'position',[475,54,495,176])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']],...
		'MATLAB Fcn','fix(u/T)*T')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']],...
		'Output Width','1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']],...
		'position',[365,128,425,162])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']],...
		'position',[295,107,320,183])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']],...
		'Value','instant_injection')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']],...
		'position',[165,154,270,176])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Calcul t-nT1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Calcul t-nT1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Calcul t-nT1']],...
		'position',[460,206,465,211])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/in_1']],...
		'position',[245,115,265,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[325,145;360,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[345,145;345,85;470,85])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[275,165;290,165])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[430,145;470,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[500,115;550,115])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[270,125;290,125])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],...
		'Mask Display','Mise en place\ndes priodes')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],...
		'position',[115,326,255,374])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Clock2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Clock2']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Clock2']],...
		'position',[45,330,85,370])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t-nT']],...
		'position',[930,355,935,360])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'position',[360,5,365,10])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[90,220;440,220])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[390,220;390,110;610,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[90,350;110,350])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[345,290;370,290])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[570,230;610,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[590,230;590,280;605,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[405,320;430,320;440,235])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[480,230;500,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[260,350;285,350;285,430;610,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[285,350;370,350])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],...
		'Mask Display','CHAMP MESURE\n\n periode n')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],...
		'position',[300,165,435,235])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive B']],...
		'position',[495,270,500,275])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/derb_chapeau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/derb_chapeau']],...
		'position',[535,335,540,340])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive de',13,'B relle']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive de',13,'B relle']],...
		'position',[230,262,285,298])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/t -tinj - nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/t -tinj - nT']],...
		'position',[500,225,505,230])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/B(n)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/B(n)']],...
		'position',[535,175,540,180])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ magntique',13,'terrestre',13,'(repre satellite)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ magntique',13,'terrestre',13,'(repre satellite)']],...
		'position',[130,120,135,125])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Temps']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Temps']],...
		'position',[130,285,135,290])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_2']],...
		'position',[110,310,130,330])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux']],...
		'position',[505,94,545,126])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux1']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux1']],...
		'position',[680,349,720,381])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'mat-name','b_reel_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'position',[580,84,660,116])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'mat-name','derb_pulse_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'position',[755,339,870,371])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'mat-name','champ_pulse')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'position',[745,213,865,247])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux2']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux2']],...
		'position',[750,279,790,311])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux3']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux3']],...
		'position',[375,419,415,451])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'mat-name','derb_reel_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'position',[460,409,540,441])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'mat-name','b_pulse_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'position',[825,266,925,304])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[670,230;740,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[685,230;685,155;785,155])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[450,300;530,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[475,300;475,400;795,400])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[440,175;465,175;465,55;775,55])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[135,320;305,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[290,280;305,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[565,275;635,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[440,225;455,225;455,250;530,250])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[440,200;580,200;580,180;635,180])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[140,200;295,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[275,200;190,200;190,280;225,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[465,110;500,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[615,400;615,365;675,365])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[550,100;575,100])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[725,355;750,355])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[795,285;820,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[710,230;710,295;745,295])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[420,425;455,425])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[290,280;290,435;370,435])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'Mask Display','Champ rel\n&\nchamp mesur')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'Mask Help','Entre 1= champ satellite, entre 2=temps, sortie 1= champ satellite, sortie2= champ liss, sortie 3 = drive non lisse')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'position',[595,287,730,403])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_1']],...
		'position',[760,295,780,315])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_2']],...
		'position',[760,335,780,355])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_3']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_3']],...
		'position',[760,375,780,395])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']],...
		'position',[475,130,480,135])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']],...
		'position',[500,65,505,70])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Clock']],...
		'position',[330,393,370,427])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[735,305;755,305])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[735,345;755,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[735,385;755,385])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[185,315;235,315])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[445,315;590,315])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[375,410;420,410;435,375;590,375])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],...
		'Mask Display','CHAMP ET DERIVEE\n\nPULSES + CHAMP REEL')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],...
		'position',[85,64,285,156])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Drive pulse crneau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Drive pulse crneau']],...
		'position',[460,65,465,70])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ puls linaire']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ puls linaire']],...
		'position',[455,95,460,100])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ rel']],...
		'position',[425,125,430,130])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[290,80;355,80])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[290,110;355,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[290,140;355,140])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[50,110;80,110])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],...
		'Mask Display','CHAMP ET DERIVEE\n\nPULSES')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],...
		'position',[80,263,220,337])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc']],...
		'position',[485,340,490,345])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B ']],...
		'position',[505,295,510,300])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B']],...
		'position',[260,345,265,350])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative']],...
		'position',[455,399,500,421])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''']],...
		'position',[445,250,450,255])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/in_1']],...
		'position',[10,289,25,311])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Entre vecteur d''tat ',13,'7 composantes p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Entre vecteur d''tat ',13,'7 composantes p q r Q']],...
		'position',[140,370,145,375])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','Bc_prime')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']],...
		'buffer','points_trace',...
		'position',[425,439,490,461])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']],...
		'hide name',0,...
		'mat-name','Bc',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']],...
		'position',[430,469,495,491])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pulse crneau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pulse crneau']],...
		'position',[280,250,285,255])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux2']],...
		'hide name',0,...
		'inputs','3',...
		'position',[1055,660,1080,730])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant']],...
		'hide name',0,...
		'Value','0',...
		'position',[1000,660,1020,680])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant1']],...
		'hide name',0,...
		'Value','0',...
		'position',[1000,710,1020,730])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']],...
		'hide name',0,...
		'mat-name','M4',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']],...
		'position',[1150,364,1185,386])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux4']],...
		'hide name',0,...
		'inputs','[3 3]',...
		'position',[1015,386,1040,479])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc '].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],'Location',[4,42,1268,968])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /SIMULATION D''UN RETARD ORBITAL']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /SIMULATION D''UN RETARD ORBITAL']],...
		'ForeGround',3,...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /SIMULATION D''UN RETARD ORBITAL']],...
		'Font Weight','bold',...
		'Font Size',20)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /SIMULATION D''UN RETARD ORBITAL']],...
		'position',[570,25,575,30])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Le champ rel actif est celui existant  la position du satellite, donc au temps t',13,'Le champ mesur n''est pas affect par le retard']],...
		'position',[600,80,605,85])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Seule la valeur du champ de consigne Bc est affecte par le retard',13,'donc Bc doit tre calcul avec le champ au temps t+Dt.']],...
		'position',[530,150,535,155])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Sum6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Sum6']],...
		'hide name',0,...
		'position',[340,292,390,388])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Clock']],...
		'position',[185,349,220,381])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Constant']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Constant']],...
		'hide name',0,...
		'Value','deltaT',...
		'position',[185,304,305,326])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /BX BY BZ']],...
		'position',[620,280,625,285])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'hide name',0,...
		'MATLAB Fcn','champmgn(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'position',[535,315,710,365])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /out_1']],...
		'position',[790,330,810,350])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[395,340;530,340])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[715,340;785,340])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[310,315;335,315])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[225,365;335,365])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],...
		'Mask Display','CALCUL\nBc')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc '].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],...
		'position',[335,367,395,403])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_2']],...
		'Port','2',...
		'position',[1125,760,1145,780])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'dimension 3',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'dimension 3',13,'']],...
		'position',[1165,665,1170,670])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Puls',13,'linaire']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Puls',13,'linaire']],...
		'position',[390,280,395,285])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple',13,'de commande',13,' raliser']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple',13,'de commande',13,' raliser']],...
		'position',[1235,525,1240,530])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum3']],...
		'hide name',0,...
		'position',[1155,395,1185,555])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_3']],...
		'position',[1225,465,1245,485])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux3']],...
		'inputs','[3 7]',...
		'position',[270,354,305,436])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B+pqr+Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B+pqr+Q']],...
		'position',[120,415,125,420])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace6']],...
		'hide name',0,...
		'mat-name','B',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace6']],...
		'position',[295,309,360,331])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ rel']],...
		'position',[910,500,915,505])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'outputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']],...
		'position',[915,392,955,468])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pas de roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pas de roue']],...
		'position',[730,485,735,490])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']],...
		'hide name',0,...
		'mat-name','M1',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']],...
		'position',[885,359,920,381])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''c']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''c']],...
		'position',[510,375,515,380])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux']],...
		'hide name',0,...
		'inputs','[3 3 3 3]',...
		'position',[520,251,555,434])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'ForeGround',1)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'BackGround',2)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'MATLAB Fcn','mom_prop(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'Output Width','4')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'position',[850,413,910,447])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux1']],...
		'hide name',0,...
		'inputs','[3 1]',...
		'position',[790,345,825,520])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant2']],...
		'hide name',0,...
		'Value','0',...
		'position',[720,465,740,485])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Possibilit de',13,'supprimer M1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Possibilit de',13,'supprimer M1']],...
		'position',[780,295,785,300])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'MATLAB Fcn','laur0(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'position',[565,328,625,362])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain2']],...
		'hide name',0,...
		'position',[740,332,765,358])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],'Location',[662,349,1152,679])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Mux5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Mux5']],...
		'inputs','3',...
		'position',[345,94,380,276])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Product2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Product2']],...
		'position',[285,233,315,257])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Product1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Product1']],...
		'position',[280,178,310,202])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Demux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Demux1']],...
		'outputs','3',...
		'position',[200,104,235,266])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Product']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Product']],...
		'position',[285,113,315,137])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/out_1']],...
		'position',[410,175,430,195])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/in_1']],...
		'position',[150,175,170,195])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']],'Location',[10,444,485,774])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/SURVOL STATION',13,'ARRT REGULATION DURANT 400 secondes']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/SURVOL STATION',13,'ARRT REGULATION DURANT 400 secondes']],...
		'position',[255,10,260,15])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/Rgulation',13,'alterne']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/Rgulation',13,'alterne']],...
		'MATLAB Fcn','regulalt(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/Rgulation',13,'alterne']],...
		'Output Width','1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/Rgulation',13,'alterne']],...
		'position',[195,98,255,132])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/Clock']],...
		'position',[125,98,165,132])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1/out_3']],...
		'position',[285,105,305,125])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']],[170,115;190,115])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']],[260,115;280,115])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']],...
		'Mask Display','Rgime\naltern')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']],...
		'Font Name','Arial',...
		'Font Size',8)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/Subsystem1']],...
		'position',[155,53,220,87])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/To Workspace13']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/To Workspace13']],...
		'hide name',0,...
		'mat-name','coupures')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2/To Workspace13']],...
		'buffer','points_reduits',...
		'position',[330,59,400,81])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[315,190;320,190;320,185;340,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[320,245;340,245])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[320,125;340,125])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[225,70;260,70;260,250;280,250])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[260,195;275,195])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[240,185;275,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[240,240;280,240])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[260,120;280,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[240,130;280,130])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[385,185;405,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[175,185;195,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],[260,70;325,70])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],...
		'Mask Display','REGULATION\nALTERNEE')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',8)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Subsystem2']],...
		'position',[650,323,725,367])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[505,410;495,450])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[400,385;430,385;430,365;515,365])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[400,385;400,480;425,480])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[225,300;485,300;485,320;515,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[250,300;250,375;265,375])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[430,385;430,410;450,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[505,410;515,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[560,345;560,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[225,275;515,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[30,300;75,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[30,300;30,415;265,415])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[915,430;910,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[960,410;1010,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1045,435;1060,435])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1130,435;1150,435])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1190,475;1220,475])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1025,670;1050,670])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1025,720;1050,720])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1085,695;1115,695;1115,515;1150,515])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1190,475;1190,335;1155,335])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[960,450;967,450;967,695;1050,695])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1130,435;1130,375;1145,375])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[970,695;970,770;1120,770])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[745,475;785,475])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[225,325;235,325;235,530;990,530;990,455;1010,455])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[250,320;290,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[830,435;845,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[835,430;835,370;880,370])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[630,345;645,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[730,345;735,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[770,345;775,345;785,390])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],...
		'Mask Display','CALCUL DES MOMENTS\n\nMAGNETIQUES\n\nET COUPLE ROUE')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],...
		'position',[310,274,525,376])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Vecteur d''tat']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Vecteur d''tat']],...
		'position',[175,310,180,315])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/in_1'],...
		'hide name',0,...
		'position',[230,315,250,335])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/pqrQ'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/pqrQ'],...
		'position',[245,280,250,285])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_5'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_5'],...
		'hide name',0,...
		'position',[605,290,625,310])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_7'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_7'],...
		'hide name',0,...
		'Port','2',...
		'position',[605,340,625,360])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[255,325;305,325])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[530,300;600,300])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[530,350;600,350])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],...
		'Mask Display','REGULATION BOUSSOLE\n\nTECHNIQUE LAURENS\n\nPULSEE TYPE 2')


%     Finished composite block 'F(X,t) pour la rotation/Calcul des commandes, moments et roue'.

set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],...
		'position',[205,171,435,249])


%     Subsystem  'F(X,t) pour la rotation/Subsystem'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],'Location',[4,50,970,451])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Subsystem/Inversion',13,'signe',13,'(action//roue)',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Inversion',13,'signe',13,'(action//roue)',13,'']],...
		'ForeGround',2,...
		'Gain','-1',...
		'position',[115,122,145,148])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem/Gain3'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/Gain3'],...
		'ForeGround',2,...
		'Gain','30/pi',...
		'position',[445,118,470,152])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace5'],...
		'ForeGround',2,...
		'BackGround',7,...
		'hide name',0,...
		'Font Size',14,...
		'mat-name','vit_roue',...
		'buffer','points_trace',...
		'position',[515,120,595,150])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue ',13,'en rd//s']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue ',13,'en rd//s']],...
		'ForeGround',2,...
		'position',[520,210,525,215])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/in_1'],...
		'ForeGround',2,...
		'position',[20,125,40,145])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue',13,'en tours//mn']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue',13,'en tours//mn']],...
		'ForeGround',2,...
		'position',[455,70,460,75])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Subsystem/Inertie',13,'de la ',13,'roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Inertie',13,'de la ',13,'roue']],...
		'ForeGround',2,...
		'BackGround',7,...
		'Gain','1/J_ROUE',...
		'position',[195,115,230,155])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Subsystem/SUIVI DE LA ROUE'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/SUIVI DE LA ROUE'],...
		'ForeGround',2,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[390,10,395,15])

add_block('built-in/Integrator',[sys,'/','F(X,t) pour la rotation/Subsystem/Integrator2'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/Integrator2'],...
		'ForeGround',2,...
		'BackGround',7,...
		'Initial','rotation_initiale',...
		'position',[280,113,325,157])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'],...
		'ForeGround',2,...
		'BackGround',7,...
		'hide name',0,...
		'Font Size',14,...
		'mat-name','vitesseroue',...
		'buffer','points_trace')
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'],...
		'position',[445,179,590,211])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/out_1'],...
		'hide name',0,...
		'position',[610,330,630,350])

add_block('built-in/To File',[sys,'/','F(X,t) pour la rotation/Subsystem/To File'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To File'],...
		'ForeGround',2,...
		'BackGround',7,...
		'File name','vit_roue.mat',...
		'Matrix name','vit_roue_rd_s',...
		'position',[455,247,570,273])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Sortie de la vitesse ',13,'de la roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Sortie de la vitesse ',13,'de la roue']],...
		'position',[485,320,490,325])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[475,135;510,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[150,135;190,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[45,135;110,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[235,135;275,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[330,135;440,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[395,135;395,195;440,195])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[395,195;395,345;605,340])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[420,195;420,260;450,260])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],...
		'Mask Display','EVOLUTION DE LA\nVITESSE DE LA ROUE')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'position',[710,474,855,536])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/To Workspace4'])
set_param([sys,'/','F(X,t) pour la rotation/To Workspace4'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','quater',...
		'buffer','points_trace',...
		'position',[365,450,430,480])
add_line([sys,'/','F(X,t) pour la rotation'],[420,310;455,310])
add_line([sys,'/','F(X,t) pour la rotation'],[280,405;370,405])
add_line([sys,'/','F(X,t) pour la rotation'],[280,465;360,465])
add_line([sys,'/','F(X,t) pour la rotation'],[1015,335;1065,335])
add_line([sys,'/','F(X,t) pour la rotation'],[1100,335;1120,335])
add_line([sys,'/','F(X,t) pour la rotation'],[100,310;280,310])
add_line([sys,'/','F(X,t) pour la rotation'],[190,310;190,435;220,435])
add_line([sys,'/','F(X,t) pour la rotation'],[190,310;200,210])
add_line([sys,'/','F(X,t) pour la rotation'],[505,310;675,310])
add_line([sys,'/','F(X,t) pour la rotation'],[725,250;745,265])
add_line([sys,'/','F(X,t) pour la rotation'],[855,265;880,265])
add_line([sys,'/','F(X,t) pour la rotation'],[930,265;965,265])
add_line([sys,'/','F(X,t) pour la rotation'],[505,405;965,405])
add_line([sys,'/','F(X,t) pour la rotation'],[535,190;555,190])
add_line([sys,'/','F(X,t) pour la rotation'],[650,190;675,190])
add_line([sys,'/','F(X,t) pour la rotation'],[440,230;580,230;580,505;705,505])
add_line([sys,'/','F(X,t) pour la rotation'],[860,505;1075,505])
add_line([sys,'/','F(X,t) pour la rotation'],[440,190;480,190])
set_param([sys,'/','F(X,t) pour la rotation'],...
		'Mask Display','F(X , t ) \n\nPOUR LA ROTATION')


%     Finished composite block 'F(X,t) pour la rotation'.

set_param([sys,'/','F(X,t) pour la rotation'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[125,278,350,362])

add_block('built-in/Mux',[sys,'/','Mux'])
set_param([sys,'/','Mux'],...
		'hide name',0,...
		'inputs','[3 4]',...
		'position',[500,233,545,582])

add_block('built-in/Note',[sys,'/',['Entres',13,'drive rotation (3)']])
set_param([sys,'/',['Entres',13,'drive rotation (3)']],...
		'position',[435,235,440,240])

add_block('built-in/Note',[sys,'/','Drive quaternion (4)'])
set_param([sys,'/','Drive quaternion (4)'],...
		'position',[425,495,430,500])

add_block('built-in/Sum',[sys,'/','Sum'])
set_param([sys,'/','Sum'],...
		'position',[420,280,445,355])

add_block('built-in/Integrator',[sys,'/','Integrator'])
set_param([sys,'/','Integrator'],...
		'hide name',0,...
		'Initial','etat_init',...
		'position',[620,384,720,436])

add_block('built-in/Note',[sys,'/',['p'' q'' r'' Q''']])
set_param([sys,'/',['p'' q'' r'' Q''']],...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[785,315,790,320])

add_block('built-in/Note',[sys,'/','p q r Q'])
set_param([sys,'/','p q r Q'],...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[785,250,790,255])

add_block('built-in/Note',[sys,'/','1  3 = rotation inertielle'])
set_param([sys,'/','1  3 = rotation inertielle'],...
		'position',[180,575,185,580])

add_block('built-in/Note',[sys,'/',['4  7 = quaternion d''attitude dans le repre orbital']])
set_param([sys,'/',['4  7 = quaternion d''attitude dans le repre orbital']],...
		'position',[585,580,590,585])


%     Subsystem  'Couplage roue-satellite'.

new_system([sys,'/','Couplage roue-satellite'])
set_param([sys,'/','Couplage roue-satellite'],'Location',[4,42,1195,546])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/CALCUL DU COUPLE DE COUPLAGE ROUE SATELLITE'])
set_param([sys,'/','Couplage roue-satellite/CALCUL DU COUPLE DE COUPLAGE ROUE SATELLITE'],...
		'ForeGround',3,...
		'BackGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[530,50,535,55])

add_block('built-in/Gain',[sys,'/','Couplage roue-satellite/Gain1'])
set_param([sys,'/','Couplage roue-satellite/Gain1'],...
		'hide name',0,...
		'position',[475,172,500,198])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/Fonction jw_rotot.m'])
set_param([sys,'/','Couplage roue-satellite/Fonction jw_rotot.m'],...
		'position',[375,220,380,225])

add_block('built-in/Inport',[sys,'/','Couplage roue-satellite/in_3'])
set_param([sys,'/','Couplage roue-satellite/in_3'],...
		'position',[95,140,115,160])

add_block('built-in/Inport',[sys,'/','Couplage roue-satellite/in_4'])
set_param([sys,'/','Couplage roue-satellite/in_4'],...
		'Port','2',...
		'position',[100,209,125,231])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/pqrQ'])
set_param([sys,'/','Couplage roue-satellite/pqrQ'],...
		'position',[80,205,85,210])

add_block('built-in/MATLAB Fcn',[sys,'/','Couplage roue-satellite/magneto.m2'])
set_param([sys,'/','Couplage roue-satellite/magneto.m2'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'MATLAB Fcn','jwrotot0(u)',...
		'Output Width','3',...
		'Mask Display','Participation\nde la roue\nau second membre')
set_param([sys,'/','Couplage roue-satellite/magneto.m2'],...
		'position',[300,161,440,209])

add_block('built-in/Note',[sys,'/',['Couplage roue-satellite/Sortie du couplage',13,'roue satellite']])
set_param([sys,'/',['Couplage roue-satellite/Sortie du couplage',13,'roue satellite']],...
		'position',[600,220,605,225])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/Vitesseroue'])
set_param([sys,'/','Couplage roue-satellite/Vitesseroue'],...
		'position',[100,110,105,115])

add_block('built-in/Outport',[sys,'/','Couplage roue-satellite/out_1'])
set_param([sys,'/','Couplage roue-satellite/out_1'],...
		'position',[590,175,610,195])

add_block('built-in/Mux',[sys,'/','Couplage roue-satellite/Mux'])
set_param([sys,'/','Couplage roue-satellite/Mux'],...
		'inputs','[1 7]',...
		'position',[195,113,250,257])

add_block('built-in/To Workspace',[sys,'/','Couplage roue-satellite/To Workspace7'])
set_param([sys,'/','Couplage roue-satellite/To Workspace7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','couplage_roue_sat',...
		'buffer','points_trace',...
		'position',[685,294,845,316])

add_block('built-in/State-Space',[sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']])
set_param([sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'A','[]',...
		'B','[]',...
		'C','[]',...
		'D','Asat',...
		'Mask Display','Asat',...
		'Mask Type','Matrix Gain')
set_param([sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'Mask Help','Multiplies input vector by entered matrix to produce output vector (y=Au).',...
		'position',[555,282,655,328])
add_line([sys,'/','Couplage roue-satellite'],[255,185;295,185])
add_line([sys,'/','Couplage roue-satellite'],[120,150;190,150])
add_line([sys,'/','Couplage roue-satellite'],[130,220;190,220])
add_line([sys,'/','Couplage roue-satellite'],[445,185;470,185])
add_line([sys,'/','Couplage roue-satellite'],[505,185;585,185])
add_line([sys,'/','Couplage roue-satellite'],[660,305;680,305])
add_line([sys,'/','Couplage roue-satellite'],[530,185;530,305;550,305])
set_param([sys,'/','Couplage roue-satellite'],...
		'Mask Display','COUPLAGE ROUE-SATELLITE')


%     Finished composite block 'Couplage roue-satellite'.

set_param([sys,'/','Couplage roue-satellite'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'position',[115,380,330,435])


%     Subsystem  'Subsystem12'.

new_system([sys,'/','Subsystem12'])
set_param([sys,'/','Subsystem12'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem12/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem12/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem12/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem12/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem12/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem12/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem12/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem12/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem12/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem12/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem12/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem12/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem12/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem12/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem12/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem12/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem12/d) Lancer la simulation'])
set_param([sys,'/','Subsystem12/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem12/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem12/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem12/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem12/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem12/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem12/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem12/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem12/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem12/b) '])
set_param([sys,'/','Subsystem12/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem12/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem12/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem12/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem12/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem12/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem12/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem12/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem12/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem12/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem12'],...
		'Mask Display','3\n\ndeltaT=5 s',...
		'Mask Dialogue','eval(''deltaT=5'')')


%     Finished composite block 'Subsystem12'.

set_param([sys,'/','Subsystem12'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1115,265,1240,336])


%     Subsystem  'Subsystem13'.

new_system([sys,'/','Subsystem13'])
set_param([sys,'/','Subsystem13'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem13/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem13/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem13/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem13/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem13/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem13/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem13/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem13/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem13/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem13/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem13/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem13/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem13/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem13/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem13/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem13/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem13/d) Lancer la simulation'])
set_param([sys,'/','Subsystem13/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem13/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem13/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem13/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem13/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem13/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem13/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem13/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem13/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem13/b) '])
set_param([sys,'/','Subsystem13/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem13/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem13/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem13/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem13/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem13/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem13/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem13/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem13/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem13/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem13'],...
		'Mask Display','3\n\ndeltaT=10 s',...
		'Mask Dialogue','eval(''deltaT=5'')')


%     Finished composite block 'Subsystem13'.

set_param([sys,'/','Subsystem13'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1120,350,1245,421])


%     Subsystem  'Courbes'.

new_system([sys,'/','Courbes'])
set_param([sys,'/','Courbes'],'Location',[4,74,1268,1000])
open_system([sys,'/','Courbes'])


%     Subsystem  'Courbes/Extras8'.

new_system([sys,'/','Courbes/Extras8'])
set_param([sys,'/','Courbes/Extras8'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras8'],...
		'Mask Display','Voir courbe\ncouple de\ncommande (Nm)',...
		'Mask Dialogue','eval(''plot(temps,cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras8'.

set_param([sys,'/','Courbes/Extras8'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,155,750,221])


%     Subsystem  'Courbes/Extras9'.

new_system([sys,'/','Courbes/Extras9'])
set_param([sys,'/','Courbes/Extras9'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras9'],...
		'Mask Display','Voir les courbes\nmoments magntiques\nmx,my,mz',...
		'Mask Dialogue','eval(''plot(temps,mx,temps,my,temps,mz);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras9'.

set_param([sys,'/','Courbes/Extras9'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[330,75,555,136])


%     Subsystem  'Courbes/Extras17'.

new_system([sys,'/','Courbes/Extras17'])
set_param([sys,'/','Courbes/Extras17'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras17'],...
		'Mask Display','Champs magntiques\nrel roulis\npuls linaire roulis',...
		'Mask Dialogue','eval(''plot(temps,b_reel_roulis,temps,b_pulse_roulis);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras17'.

set_param([sys,'/','Courbes/Extras17'],...
		'BackGround',7,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[975,70,1175,136])


%     Subsystem  'Courbes/Extras16'.

new_system([sys,'/','Courbes/Extras16'])
set_param([sys,'/','Courbes/Extras16'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras16'],...
		'Mask Display','Voir la commande \navant le "hachoir "',...
		'Mask Dialogue','eval(''plot(temps,com_avant);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras16'.

set_param([sys,'/','Courbes/Extras16'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[760,75,960,141])


%     Subsystem  'Courbes/Extras5'.

new_system([sys,'/','Courbes/Extras5'])
set_param([sys,'/','Courbes/Extras5'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras5'],...
		'Mask Display','Voir les courbes\ndes angles (en degrs)',...
		'Mask Dialogue','eval(''plot(temps,roulis,temps,tangage,temps,lacet);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras5'.

set_param([sys,'/','Courbes/Extras5'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,75,295,141])


%     Subsystem  'Courbes/Extras7'.

new_system([sys,'/','Courbes/Extras7'])
set_param([sys,'/','Courbes/Extras7'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras7'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,p,temps,q,temps,r);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras7'.

set_param([sys,'/','Courbes/Extras7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,160,300,221])


%     Subsystem  'Courbes/Extras20'.

new_system([sys,'/','Courbes/Extras20'])
set_param([sys,'/','Courbes/Extras20'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras20'],...
		'Mask Display','Voir la commande \napres le "hachoir "',...
		'Mask Dialogue','eval(''plot(temps,com_avant-com_apres);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras20'.

set_param([sys,'/','Courbes/Extras20'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[765,240,965,306])


%     Subsystem  'Courbes/Extras18'.

new_system([sys,'/','Courbes/Extras18'])
set_param([sys,'/','Courbes/Extras18'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras18'],...
		'Mask Display','Voir la commande \napres le "hachoir "',...
		'Mask Dialogue','eval(''plot(temps,com_apres);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras18'.

set_param([sys,'/','Courbes/Extras18'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[760,155,960,221])


%     Subsystem  'Courbes/Extras6'.

new_system([sys,'/','Courbes/Extras6'])
set_param([sys,'/','Courbes/Extras6'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras6'],...
		'Mask Display','Voir la courbe\n\nenergie absolue (J)',...
		'Mask Dialogue','eval(''plot(temps,energie);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras6'.

set_param([sys,'/','Courbes/Extras6'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,75,750,131])

add_block('built-in/Note',[sys,'/','Courbes/********************************************************************************************'])
set_param([sys,'/','Courbes/********************************************************************************************'],...
		'Font Name','Arial',...
		'Font Size',20,...
		'position',[670,345,675,350])

add_block('built-in/Note',[sys,'/','Courbes/COURBES PRINCIPALES'])
set_param([sys,'/','Courbes/COURBES PRINCIPALES'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[595,15,600,20])


%     Subsystem  'Courbes/Extras14'.

new_system([sys,'/','Courbes/Extras14'])
set_param([sys,'/','Courbes/Extras14'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras14'],...
		'Mask Display','Voir courbe\ndiffrence Bc - B ',...
		'Mask Dialogue','eval(''plot(temps,Bc-B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras14'.

set_param([sys,'/','Courbes/Extras14'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,450,275,516])


%     Subsystem  'Courbes/Extras10'.

new_system([sys,'/','Courbes/Extras10'])
set_param([sys,'/','Courbes/Extras10'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras10'],...
		'Mask Display','Couleurs des courbes\nroulis=jaune\ntangage=bleu\nlacet=magenta')


%     Finished composite block 'Courbes/Extras10'.

set_param([sys,'/','Courbes/Extras10'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,230,755,311])


%     Subsystem  'Courbes/Extras26'.

new_system([sys,'/','Courbes/Extras26'])
set_param([sys,'/','Courbes/Extras26'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras26'],...
		'Mask Display','Voir la vitesse angulaire\nde lacet seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,r,''''c.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras26'.

set_param([sys,'/','Courbes/Extras26'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,165,540,226])


%     Subsystem  'Courbes/Extras25'.

new_system([sys,'/','Courbes/Extras25'])
set_param([sys,'/','Courbes/Extras25'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras25'],...
		'Mask Display','Voir la vitesse angulaire\nde roulis seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,p,''''y.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras25'.

set_param([sys,'/','Courbes/Extras25'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,240,540,301])


%     Subsystem  'Courbes/Extras11'.

new_system([sys,'/','Courbes/Extras11'])
set_param([sys,'/','Courbes/Extras11'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras11'],...
		'Mask Display','Voir la vitesse angulaire\nde tangage seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,q,''''m.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras11'.

set_param([sys,'/','Courbes/Extras11'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[90,235,295,296])


%     Subsystem  'Courbes/Extras32'.

new_system([sys,'/','Courbes/Extras32'])
set_param([sys,'/','Courbes/Extras32'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras32'],...
		'Mask Display','Voir courbe\nquaternion(2:4)',...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),2:4));grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras32'.

set_param([sys,'/','Courbes/Extras32'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[830,450,1005,516])


%     Subsystem  'Courbes/Extras33'.

new_system([sys,'/','Courbes/Extras33'])
set_param([sys,'/','Courbes/Extras33'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras33'],...
		'Mask Display','Voir courbe\nquaternion(1)',...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),1));grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras33'.

set_param([sys,'/','Courbes/Extras33'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[830,535,1005,601])


%     Subsystem  'Courbes/Extras22'.

new_system([sys,'/','Courbes/Extras22'])
set_param([sys,'/','Courbes/Extras22'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras22'],...
		'Mask Display','Voir l''angle\nde tangage seul ()',...
		'Mask Dialogue','eval(''plot(temps,tangage,''''m.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras22'.

set_param([sys,'/','Courbes/Extras22'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,320,300,381])


%     Subsystem  'Courbes/Extras38'.

new_system([sys,'/','Courbes/Extras38'])
set_param([sys,'/','Courbes/Extras38'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras38'],...
		'Mask Display','Voir l''angle\nde roulis seul ()',...
		'Mask Dialogue','eval(''plot(temps,roulis,''''y.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras38'.

set_param([sys,'/','Courbes/Extras38'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,315,540,376])


%     Subsystem  'Courbes/Extras12'.

new_system([sys,'/','Courbes/Extras12'])
set_param([sys,'/','Courbes/Extras12'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras12'],...
		'Mask Display','Voir courbe\ncouple commande\n roue (Nm)',...
		'Mask Dialogue','eval(''plot(temps,cpl_roue);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras12'.

set_param([sys,'/','Courbes/Extras12'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1025,450,1200,516])


%     Subsystem  'Courbes/Extras27'.

new_system([sys,'/','Courbes/Extras27'])
set_param([sys,'/','Courbes/Extras27'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras27'],...
		'Mask Display','Voir courbe\ndiffrence PB - B ',...
		'Mask Dialogue','eval(''plot(temps,PB_moins_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras27'.

set_param([sys,'/','Courbes/Extras27'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,610,275,676])


%     Subsystem  'Courbes/Extras31'.

new_system([sys,'/','Courbes/Extras31'])
set_param([sys,'/','Courbes/Extras31'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras31'],...
		'Mask Display','Voir courbe\nsomme PB+B ',...
		'Mask Dialogue','eval(''plot(temps,PB_plus_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras31'.

set_param([sys,'/','Courbes/Extras31'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,690,275,756])


%     Subsystem  'Courbes/Extras13'.

new_system([sys,'/','Courbes/Extras13'])
set_param([sys,'/','Courbes/Extras13'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras13'],...
		'Mask Display','Voir courbe\nvitesse de la\n roue (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,vit_roue);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras13'.

set_param([sys,'/','Courbes/Extras13'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,535,1215,591])

add_block('built-in/Note',[sys,'/','Courbes/COURBES ANNEXES ET MISE AU POINT'])
set_param([sys,'/','Courbes/COURBES ANNEXES ET MISE AU POINT'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[595,385,600,390])


%     Subsystem  'Courbes/Extras39'.

new_system([sys,'/','Courbes/Extras39'])
set_param([sys,'/','Courbes/Extras39'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras39'],...
		'Mask Display','Voir l''angle\nde lacet seul ()',...
		'Mask Dialogue','eval(''plot(temps,lacet,''''c.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras39'.

set_param([sys,'/','Courbes/Extras39'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,320,760,381])


%     Subsystem  'Courbes/Extras30'.

new_system([sys,'/','Courbes/Extras30'])
set_param([sys,'/','Courbes/Extras30'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras30'],...
		'Mask Display','Voir courbe\npulsation\nmoyenne de p',...
		'Mask Dialogue','eval(''plot(temps,GDomega);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras30'.

set_param([sys,'/','Courbes/Extras30'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[775,320,965,376])


%     Subsystem  'Courbes/Extras43'.

new_system([sys,'/','Courbes/Extras43'])
set_param([sys,'/','Courbes/Extras43'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras43'],...
		'Mask Display','Voir courbe\nstop multiple ',...
		'Mask Dialogue','eval(''plot(temps,stop_multiple);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras43'.

set_param([sys,'/','Courbes/Extras43'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1000,315,1175,381])


%     Subsystem  'Courbes/Extras40'.

new_system([sys,'/','Courbes/Extras40'])
set_param([sys,'/','Courbes/Extras40'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras40'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (degrs/s)',...
		'Mask Dialogue','eval(''plot(temps,p*180/pi,temps,q*180/pi,temps,r*180/pi);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras40'.

set_param([sys,'/','Courbes/Extras40'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[990,225,1175,291])


%     Subsystem  'Courbes/Extras19'.

new_system([sys,'/','Courbes/Extras19'])
set_param([sys,'/','Courbes/Extras19'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras19'],...
		'Mask Display','Champs magn drivs\nrel roulis\npuls constant roulis',...
		'Mask Dialogue','eval(''plot(temps,derb_reel_roulis,temps,derb_pulse_roulis);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras19'.

set_param([sys,'/','Courbes/Extras19'],...
		'BackGround',7,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[985,150,1185,216])


%     Subsystem  'Courbes/Extras41'.

new_system([sys,'/','Courbes/Extras41'])
set_param([sys,'/','Courbes/Extras41'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras41'],...
		'Mask Display','Voir courbe du\ncouplage roue-satellite\nsur le roulis',...
		'Mask Dialogue','eval(''plot(temps,couplage_roue_sat(1:length(temps),1),''''y'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras41'.

set_param([sys,'/','Courbes/Extras41'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1030,610,1220,676])


%     Subsystem  'Courbes/Extras42'.

new_system([sys,'/','Courbes/Extras42'])
set_param([sys,'/','Courbes/Extras42'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras42'],...
		'Mask Display','Voir courbe du\ncouplage roue-satellite\nsur le lacet',...
		'Mask Dialogue','eval(''plot(temps,couplage_roue_sat(1:length(temps),3),''''c'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras42'.

set_param([sys,'/','Courbes/Extras42'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,685,1220,751])

add_block('built-in/Note',[sys,'/',['Courbes/Au cas o le satellite serait  l''envers']])
set_param([sys,'/',['Courbes/Au cas o le satellite serait  l''envers']],...
		'BackGround',0,...
		'Font Name','Arial',...
		'Font Size',20,...
		'position',[290,855,295,860])


%     Subsystem  'Courbes/Extras37'.

new_system([sys,'/','Courbes/Extras37'])
set_param([sys,'/','Courbes/Extras37'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras37'],...
		'Mask Display','Voir courbe\nunitaire(Q)+ unitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitQ+unitB);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras37'.

set_param([sys,'/','Courbes/Extras37'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,775,540,836])


%     Subsystem  'Courbes/Extras36'.

new_system([sys,'/','Courbes/Extras36'])
set_param([sys,'/','Courbes/Extras36'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras36'],...
		'Mask Display','Voir courbe\nunitaire(Q)',...
		'Mask Dialogue','eval(''plot(temps,unitQ);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras36'.

set_param([sys,'/','Courbes/Extras36'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[115,770,270,831])


%     Subsystem  'Courbes/Extras34'.

new_system([sys,'/','Courbes/Extras34'])
set_param([sys,'/','Courbes/Extras34'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras34'],...
		'Mask Display','Voir courbe\nunitaire(Q)- unitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitQ-unitB);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras34'.

set_param([sys,'/','Courbes/Extras34'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,700,545,761])


%     Subsystem  'Courbes/Extras44'.

new_system([sys,'/','Courbes/Extras44'])
set_param([sys,'/','Courbes/Extras44'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras44'],...
		'Mask Display','Voir courbes\ncouple perturbateur ',...
		'Mask Dialogue','eval(''plot(temps,perturb);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras44'.

set_param([sys,'/','Courbes/Extras44'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[825,630,1000,696])


%     Subsystem  'Courbes/Extras45'.

new_system([sys,'/','Courbes/Extras45'])
set_param([sys,'/','Courbes/Extras45'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras45'],...
		'Mask Display','Voir courbes\ndes 3 stops',...
		'Mask Dialogue','eval(''plot(temps,stop1,''''y'''',temps,stop2,''''m'''',temps,stop3,''''c'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras45'.

set_param([sys,'/','Courbes/Extras45'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[825,705,1000,756])


%     Subsystem  'Courbes/Extras15'.

new_system([sys,'/','Courbes/Extras15'])
set_param([sys,'/','Courbes/Extras15'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras15'],...
		'Mask Display','Voir courbe\ndiffrence B''c - B'' ',...
		'Mask Dialogue','eval(''plot(temps,Bc_prime-B_prime);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras15'.

set_param([sys,'/','Courbes/Extras15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,535,270,601])


%     Subsystem  'Courbes/Extras28'.

new_system([sys,'/','Courbes/Extras28'])
set_param([sys,'/','Courbes/Extras28'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras28'],...
		'Mask Display','Voir courbe\nnorme de la\ndiffrence PB - B ',...
		'Mask Dialogue','eval(''plot(temps,norm_PB_moins_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras28'.

set_param([sys,'/','Courbes/Extras28'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[595,685,770,751])


%     Subsystem  'Courbes/Extras24'.

new_system([sys,'/','Courbes/Extras24'])
set_param([sys,'/','Courbes/Extras24'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras24'],...
		'Mask Display','Voir courbe\ndcalage R / R0\nen degrs',...
		'Mask Dialogue','eval(''plot(temps,angle_rot_R0_to_R);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras24'.

set_param([sys,'/','Courbes/Extras24'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,530,760,596])


%     Subsystem  'Courbes/Extras23'.

new_system([sys,'/','Courbes/Extras23'])
set_param([sys,'/','Courbes/Extras23'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras23'],...
		'Mask Display','Voir courbe\ncouple total 1',...
		'Mask Dialogue','eval(''plot(temps,cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras23'.

set_param([sys,'/','Courbes/Extras23'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,450,760,516])


%     Subsystem  'Courbes/Extras46'.

new_system([sys,'/','Courbes/Extras46'])
set_param([sys,'/','Courbes/Extras46'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras46'],...
		'Mask Display','Voir courbe\nmoment magntique\n de rappel Mr',...
		'Mask Dialogue','eval(''plot(temps,Bc*KPD_laurens);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras46'.

set_param([sys,'/','Courbes/Extras46'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[320,625,540,691])


%     Subsystem  'Courbes/Extras29'.

new_system([sys,'/','Courbes/Extras29'])
set_param([sys,'/','Courbes/Extras29'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras29'],...
		'Mask Display','Voir courbe\nmoment magntique\n amortisseur Ma',...
		'Mask Dialogue','eval(''plot(temps,(Bc_prime-B_prime)*FPD_laurens);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras29'.

set_param([sys,'/','Courbes/Extras29'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[325,555,540,616])


%     Subsystem  'Courbes/Extras21'.

new_system([sys,'/','Courbes/Extras21'])
set_param([sys,'/','Courbes/Extras21'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras21'],...
		'Mask Display','Voir courbe\nmoment magntique\nde Laurens M1=Mr+Ma\npartie utile',...
		'Mask Dialogue','eval(''plot(temps,M1);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras21'.

set_param([sys,'/','Courbes/Extras21'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[330,450,535,541])


%     Subsystem  'Courbes/Extras35'.

new_system([sys,'/','Courbes/Extras35'])
set_param([sys,'/','Courbes/Extras35'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras35'],...
		'Mask Display','Voir courbe\nunitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitB);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras35'.

set_param([sys,'/','Courbes/Extras35'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[595,605,750,661])


%     Subsystem  'Courbes/Extras49'.

new_system([sys,'/','Courbes/Extras49'])
set_param([sys,'/','Courbes/Extras49'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras49'],...
		'Mask Display','Voir les courbes\ndes angles (en degrs)',...
		'Mask Dialogue','eval(''plot(tempo,anglo);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras49'.

set_param([sys,'/','Courbes/Extras49'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,770,785,836])


%     Subsystem  'Courbes/Extras48'.

new_system([sys,'/','Courbes/Extras48'])
set_param([sys,'/','Courbes/Extras48'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras48'],...
		'Mask Display','Voir courbe\nnorme moment magntique',...
		'Mask Dialogue','eval(''plot(temps,norm_moment_magn);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras48'.

set_param([sys,'/','Courbes/Extras48'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[975,835,1240,896])


%     Subsystem  'Courbes/Extras47'.

new_system([sys,'/','Courbes/Extras47'])
set_param([sys,'/','Courbes/Extras47'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras47'],...
		'Mask Display','Voir courbe\nnorme moment de commande',...
		'Mask Dialogue','eval(''plot(temps,norm_cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras47'.

set_param([sys,'/','Courbes/Extras47'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[975,770,1245,831])


%     Subsystem  'Courbes/Extras50'.

new_system([sys,'/','Courbes/Extras50'])
set_param([sys,'/','Courbes/Extras50'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras50'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (rd/s)',...
		'Mask Dialogue','eval(''plot(tempo,vitesso);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras50'.

set_param([sys,'/','Courbes/Extras50'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,840,790,901])
set_param([sys,'/','Courbes'],...
		'Mask Display','SORTIES GRAPHIQUES')


%     Finished composite block 'Courbes'.

set_param([sys,'/','Courbes'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[120,600,395,676])


%     Subsystem  'Subsystem15'.

new_system([sys,'/','Subsystem15'])
set_param([sys,'/','Subsystem15'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem15/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem15/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem15/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem15/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem15/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem15/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem15/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem15/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem15/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem15/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem15/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem15/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem15/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem15/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem15/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem15/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem15/d) Lancer la simulation'])
set_param([sys,'/','Subsystem15/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem15/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem15/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem15/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem15/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem15/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem15/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem15/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem15/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem15/b) '])
set_param([sys,'/','Subsystem15/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem15/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem15/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem15/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem15/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem15/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem15/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem15/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem15/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem15/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem15'],...
		'Mask Display','3\n\ndeltaT=100 s',...
		'Mask Dialogue','eval(''deltaT=100'')')


%     Finished composite block 'Subsystem15'.

set_param([sys,'/','Subsystem15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1120,435,1245,506])


%     Subsystem  'Subsystem1'.

new_system([sys,'/','Subsystem1'])
set_param([sys,'/','Subsystem1'],'Location',[196,101,1144,611])

add_block('built-in/Note',[sys,'/','Subsystem1/QUE FAIT LA SIMULATION'])
set_param([sys,'/','Subsystem1/QUE FAIT LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem1/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem1/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem1'],...
		'Mask Display','LIRE LES EXPLICATIONS\nDANS L''ESPACE MATLAB',...
		'Mask Dialogue','eval(''nanoexp0'')')


%     Finished composite block 'Subsystem1'.

set_param([sys,'/','Subsystem1'],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[85,815,435,876])


%     Subsystem  'Subsystem2'.

new_system([sys,'/','Subsystem2'])
set_param([sys,'/','Subsystem2'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem2/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem2/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem2/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem2/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem2/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem2/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem2/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem2/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem2/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem2/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem2/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem2/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem2/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem2/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem2/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem2/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem2/d) Lancer la simulation'])
set_param([sys,'/','Subsystem2/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem2/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem2/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem2/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem2/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem2/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem2/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem2/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem2/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem2/b) '])
set_param([sys,'/','Subsystem2/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem2/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem2/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem2/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem2/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem2/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem2/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem2'],...
		'Mask Display','RECUPERER LES RESULTATS\n\nDANS L''ESPACE MATLAB',...
		'Mask Dialogue','eval(''display'')')


%     Finished composite block 'Subsystem2'.

set_param([sys,'/','Subsystem2'],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[105,740,405,801])

add_block('built-in/Note',[sys,'/',['Ci-dessous cliquer',13,'pour obtenir les tracs de courbes']])
set_param([sys,'/',['Ci-dessous cliquer',13,'pour obtenir les tracs de courbes']],...
		'Font Name','Arial',...
		'Font Angle','italic',...
		'Font Size',16,...
		'position',[260,675,265,680])


%     Subsystem  'Subsystem14'.

new_system([sys,'/','Subsystem14'])
set_param([sys,'/','Subsystem14'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem14/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem14/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem14/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem14/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem14/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem14/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem14/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem14/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem14/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem14/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem14/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem14/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem14/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem14/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem14/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem14/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem14/d) Lancer la simulation'])
set_param([sys,'/','Subsystem14/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem14/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem14/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem14/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem14/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem14/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem14/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem14/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem14/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem14/b) '])
set_param([sys,'/','Subsystem14/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem14/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem14/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem14/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem14/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem14/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem14/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem14/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem14/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem14/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem14'],...
		'Mask Display','3\ndeltaT=0 s\nvaleur\npar dfaut',...
		'Mask Dialogue','eval(''deltaT=0'')')


%     Finished composite block 'Subsystem14'.

set_param([sys,'/','Subsystem14'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1105,175,1250,256])
add_line(sys,[725,410;745,410;745,580;35,580;35,495;115,495])
add_line(sys,[550,410;615,410])
add_line(sys,[340,495;495,495])
add_line(sys,[745,410;745,300;840,300])
add_line(sys,[595,410;595,320;830,320])
add_line(sys,[35,495;35,320;120,320])
add_line(sys,[355,300;415,300])
add_line(sys,[335,410;400,410;400,335;415,335])
add_line(sys,[450,320;495,320])
add_line(sys,[35,420;110,420])
add_line(sys,[355,340;365,340;365,370;95,370;95,395;110,395])
add_line(sys,[745,410;855,410])

drawnow

% Return any arguments.
if (nargin | nargout)
	% Must use feval here to access system in memory
	if (nargin > 3)
		if (flag == 0)
			eval(['[ret,x0,str,ts,xts]=',sys,'(t,x,u,flag);'])
		else
			eval(['ret =', sys,'(t,x,u,flag);'])
		end
	else
		[ret,x0,str,ts,xts] = feval(sys);
	end
else
	drawnow % Flash up the model and execute load callback
end
