function [ret,x0,str,ts,xts]=regboux7(t,x,u,flag);
%REGBOUX7	is the M-file description of the SIMULINK system named REGBOUX7.
%	The block-diagram can be displayed by typing: REGBOUX7.
%
%	SYS=REGBOUX7(T,X,U,FLAG) returns depending on FLAG certain
%	system values given time point, T, current state vector, X,
%	and input vector, U.
%	FLAG is used to indicate the type of output to be returned in SYS.
%
%	Setting FLAG=1 causes REGBOUX7 to return state derivatives, FLAG=2
%	discrete states, FLAG=3 system outputs and FLAG=4 next sample
%	time. For more information and other options see SFUNC.
%
%	Calling REGBOUX7 with a FLAG of zero:
%	[SIZES]=REGBOUX7([],[],[],0),  returns a vector, SIZES, which
%	contains the sizes of the state vector and other parameters.
%		SIZES(1) number of states
%		SIZES(2) number of discrete states
%		SIZES(3) number of outputs
%		SIZES(4) number of inputs
%		SIZES(5) number of roots (currently unsupported)
%		SIZES(6) direct feedthrough flag
%		SIZES(7) number of sample times
%
%	For the definition of other parameters in SIZES, see SFUNC.
%	See also, TRIM, LINMOD, LINSIM, EULER, RK23, RK45, ADAMS, GEAR.

% Note: This M-file is only used for saving graphical information;
%       after the model is loaded into memory an internal model
%       representation is used.

% the system will take on the name of this mfile:
sys = mfilename;
new_system(sys)
simver(1.3)
if (0 == (nargin + nargout))
     set_param(sys,'Location',[71,149,1335,1075])
     open_system(sys)
end;
set_param(sys,'algorithm',     'RK-45')
set_param(sys,'Start time',    'instant_injection')
set_param(sys,'Stop time',     '10000')
set_param(sys,'Min step size', '0.02')
set_param(sys,'Max step size', '0.1')
set_param(sys,'Relative error','1e-3')
set_param(sys,'Return vars',   '')

add_block('built-in/Note',[sys,'/',['DETUMBLING AVEC QUATERNION D''ATTITUDE',13,'PHASE FINALE D''ACQUISITION DES ANGLES ',13,'',13,'',13,'']])
set_param([sys,'/',['DETUMBLING AVEC QUATERNION D''ATTITUDE',13,'PHASE FINALE D''ACQUISITION DES ANGLES ',13,'',13,'',13,'']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',26)
set_param([sys,'/',['DETUMBLING AVEC QUATERNION D''ATTITUDE',13,'PHASE FINALE D''ACQUISITION DES ANGLES ',13,'',13,'',13,'']],...
		'position',[640,0,645,5])

add_block('built-in/Note',[sys,'/',['L''initialisation',13,' Lance laurdat0.m, puls par dfaut',13,'']])
set_param([sys,'/',['L''initialisation',13,' Lance laurdat0.m, puls par dfaut',13,'']],...
		'BackGround',6,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[990,580,995,585])


%     Subsystem  'Couplage roue-satellite'.

new_system([sys,'/','Couplage roue-satellite'])
set_param([sys,'/','Couplage roue-satellite'],'Location',[90,50,975,548])
open_system([sys,'/','Couplage roue-satellite'])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/CALCUL DU COUPLE DE COUPLAGE ROUE SATELLITE'])
set_param([sys,'/','Couplage roue-satellite/CALCUL DU COUPLE DE COUPLAGE ROUE SATELLITE'],...
		'ForeGround',3,...
		'BackGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[530,50,535,55])

add_block('built-in/Gain',[sys,'/','Couplage roue-satellite/Gain1'])
set_param([sys,'/','Couplage roue-satellite/Gain1'],...
		'hide name',0,...
		'position',[475,172,500,198])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/Fonction jw_rotot.m'])
set_param([sys,'/','Couplage roue-satellite/Fonction jw_rotot.m'],...
		'position',[375,220,380,225])

add_block('built-in/Inport',[sys,'/','Couplage roue-satellite/in_3'])
set_param([sys,'/','Couplage roue-satellite/in_3'],...
		'position',[95,140,115,160])

add_block('built-in/Inport',[sys,'/','Couplage roue-satellite/in_4'])
set_param([sys,'/','Couplage roue-satellite/in_4'],...
		'Port','2',...
		'position',[100,209,125,231])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/pqrQ'])
set_param([sys,'/','Couplage roue-satellite/pqrQ'],...
		'position',[80,205,85,210])

add_block('built-in/Note',[sys,'/',['Couplage roue-satellite/Sortie du couplage',13,'roue satellite']])
set_param([sys,'/',['Couplage roue-satellite/Sortie du couplage',13,'roue satellite']],...
		'position',[600,220,605,225])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/Vitesseroue'])
set_param([sys,'/','Couplage roue-satellite/Vitesseroue'],...
		'position',[100,110,105,115])

add_block('built-in/Outport',[sys,'/','Couplage roue-satellite/out_1'])
set_param([sys,'/','Couplage roue-satellite/out_1'],...
		'position',[590,175,610,195])

add_block('built-in/Mux',[sys,'/','Couplage roue-satellite/Mux'])
set_param([sys,'/','Couplage roue-satellite/Mux'],...
		'inputs','[1 7]',...
		'position',[195,113,250,257])

add_block('built-in/To Workspace',[sys,'/','Couplage roue-satellite/To Workspace7'])
set_param([sys,'/','Couplage roue-satellite/To Workspace7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','couplage_roue_sat',...
		'buffer','points_trace',...
		'position',[685,294,845,316])

add_block('built-in/State-Space',[sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']])
set_param([sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'A','[]',...
		'B','[]',...
		'C','[]',...
		'D','Asat',...
		'Mask Display','Asat',...
		'Mask Type','Matrix Gain')
set_param([sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'Mask Help','Multiplies input vector by entered matrix to produce output vector (y=Au).',...
		'position',[555,282,655,328])

add_block('built-in/MATLAB Fcn',[sys,'/','Couplage roue-satellite/magneto.m2'])
set_param([sys,'/','Couplage roue-satellite/magneto.m2'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'MATLAB Fcn','jw_rotot(u)',...
		'Output Width','3',...
		'Mask Display','Participation\nde la roue\nau second membre')
set_param([sys,'/','Couplage roue-satellite/magneto.m2'],...
		'position',[295,161,435,209])
add_line([sys,'/','Couplage roue-satellite'],[255,185;290,185])
add_line([sys,'/','Couplage roue-satellite'],[120,150;190,150])
add_line([sys,'/','Couplage roue-satellite'],[130,220;190,220])
add_line([sys,'/','Couplage roue-satellite'],[440,185;470,185])
add_line([sys,'/','Couplage roue-satellite'],[505,185;585,185])
add_line([sys,'/','Couplage roue-satellite'],[660,305;680,305])
add_line([sys,'/','Couplage roue-satellite'],[530,185;530,305;550,305])
set_param([sys,'/','Couplage roue-satellite'],...
		'Mask Display','COUPLAGE ROUE-SATELLITE')


%     Finished composite block 'Couplage roue-satellite'.

set_param([sys,'/','Couplage roue-satellite'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'position',[115,430,330,485])

add_block('built-in/Note',[sys,'/',['4  7 = quaternion d''attitude dans le repre orbital']])
set_param([sys,'/',['4  7 = quaternion d''attitude dans le repre orbital']],...
		'position',[585,630,590,635])

add_block('built-in/Note',[sys,'/','1  3 = rotation inertielle'])
set_param([sys,'/','1  3 = rotation inertielle'],...
		'position',[180,625,185,630])

add_block('built-in/Note',[sys,'/','p q r Q'])
set_param([sys,'/','p q r Q'],...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[785,300,790,305])

add_block('built-in/Note',[sys,'/',['p'' q'' r'' Q''']])
set_param([sys,'/',['p'' q'' r'' Q''']],...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[785,365,790,370])

add_block('built-in/Integrator',[sys,'/','Integrator'])
set_param([sys,'/','Integrator'],...
		'hide name',0,...
		'Initial','etat_init',...
		'position',[620,434,720,486])

add_block('built-in/Sum',[sys,'/','Sum'])
set_param([sys,'/','Sum'],...
		'position',[420,330,445,405])

add_block('built-in/Note',[sys,'/','Drive quaternion (4)'])
set_param([sys,'/','Drive quaternion (4)'],...
		'position',[425,545,430,550])

add_block('built-in/Note',[sys,'/',['Entres',13,'drive rotation (3)']])
set_param([sys,'/',['Entres',13,'drive rotation (3)']],...
		'position',[435,285,440,290])

add_block('built-in/Mux',[sys,'/','Mux'])
set_param([sys,'/','Mux'],...
		'hide name',0,...
		'inputs','[3 4]',...
		'position',[500,283,545,632])


%     Subsystem  'F(X,t) pour la rotation'.

new_system([sys,'/','F(X,t) pour la rotation'])
set_param([sys,'/','F(X,t) pour la rotation'],'Location',[4,89,1258,946])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/pqr'])
set_param([sys,'/','F(X,t) pour la rotation/pqr'],...
		'position',[325,375,330,380])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Demux'])
set_param([sys,'/','F(X,t) pour la rotation/Demux'],...
		'outputs','[3 4]',...
		'position',[225,375,275,495])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/tau',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/tau',13,'']],...
		'position',[85,240,90,245])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Vecteur d''tat ',13,'7 composantes',13,'p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Vecteur d''tat ',13,'7 composantes',13,'p q r Q']],...
		'position',[105,335,110,340])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Inport'])
set_param([sys,'/','F(X,t) pour la rotation/Inport'],...
		'position',[75,300,95,320])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Suppression de',13,'toutes les',13,'perturbations']])
set_param([sys,'/',['F(X,t) pour la rotation/Suppression de',13,'toutes les',13,'perturbations']],...
		'position',[515,325,520,330])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']])
set_param([sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',22)
set_param([sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']],...
		'position',[630,40,635,45])


%     Subsystem  ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1'].

new_system([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],'Location',[4,176,1271,782])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/B']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/B']],...
		'position',[510,425,515,430])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Vecteur d''tat',13,'7 composantes',13,'p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Vecteur d''tat',13,'7 composantes',13,'p q r Q']],...
		'position',[140,515,145,520])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain3']],...
		'hide name',0,...
		'Gain','-1',...
		'position',[800,382,835,408])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple magntique rsiduel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple magntique rsiduel']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[880,399,885,404])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple arodynamique']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple arodynamique']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[690,277,695,282])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel1']],...
		'MATLAB Fcn','norm(u,2)',...
		'Output Width','1',...
		'position',[770,438,830,472])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain']],...
		'Gain','mom_magn_res',...
		'position',[375,315,490,365])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace']],...
		'hide name',0,...
		'mat-name','moment_residuel',...
		'buffer','5000',...
		'position',[895,440,1015,470])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Moment magntique rsiduel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Moment magntique rsiduel']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[245,246,250,251])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel']],...
		'MATLAB Fcn','prodvect(u)',...
		'Output Width','3',...
		'position',[640,378,700,412])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux5']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux5']],...
		'inputs','[3 3]',...
		'position',[540,286,585,504])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux3']],...
		'ForeGround',1,...
		'BackGround',0,...
		'inputs','3',...
		'position',[320,288,350,392])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant12']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant12']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','alpha',...
		'position',[130,293,275,317])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant11']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant11']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','beta',...
		'position',[130,328,275,352])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant10']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant10']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','gamma',...
		'position',[130,363,275,387])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/M']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/M']],...
		'position',[525,315,530,320])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant6']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant6']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero3',...
		'position',[785,318,930,342])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant5']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant5']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero2',...
		'position',[785,283,930,307])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant4']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero1',...
		'position',[785,248,930,272])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux1']],...
		'ForeGround',1,...
		'BackGround',0,...
		'inputs','3',...
		'position',[975,243,1005,347])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Termes rsidus de gradient',13,'et forces inertie']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Termes rsidus de gradient',13,'et forces inertie']],...
		'ForeGround',4,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[895,195,900,200])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant3']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','Irt*vitesse_rotation^2',...
		'position',[795,158,940,182])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant2']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant2']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','3*Irl*vitesse_rotation^2',...
		'position',[795,123,940,147])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant1']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','-4*Itl*vitesse_rotation^2',...
		'position',[795,88,940,112])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux']],...
		'ForeGround',7,...
		'BackGround',0,...
		'inputs','3',...
		'position',[985,83,1015,187])


%     Subsystem  ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite'].

new_system([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],'Location',[195,288,1041,757])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']],...
		'hide name',0,...
		'MATLAB Fcn','champmgn(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL',...
		'position',[250,95,425,145])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Clock']],...
		'position',[180,104,215,136])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/BX BY BZ']],...
		'position',[340,70,345,75])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Vecteur d''tat ',13,'( 7 composantes)']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Vecteur d''tat ',13,'( 7 composantes)']],...
		'position',[70,235,75,240])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/in_1']],...
		'position',[50,190,70,210])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/out_1']],...
		'position',[705,150,725,170])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Sortie Bx By Bz']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Sortie Bx By Bz']],...
		'position',[725,200,730,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Equateur au temps 0 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Equateur au temps 0 s']],...
		'position',[150,75,155,80])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn1']],...
		'MATLAB Fcn','champsat(u)',...
		'Output Width','3',...
		'position',[560,143,620,177])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Mux1']],...
		'inputs','[3 7]',...
		'position',[485,80,530,240])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[220,120;245,120])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[430,120;480,120])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[625,160;700,160])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[75,200;480,200])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[535,160;555,160])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],...
		'Mask Display','CHAMP MAGNETIQUE EN\nAXES SATELLITE\nEntre=vecteur d''tat\nSortie Bx By Bz')


%     Finished composite block ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite'].

set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],...
		'position',[240,458,440,532])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'position',[385,95,390,100])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Inport']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Inport']],...
		'position',[20,485,40,505])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Sum']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Sum']],...
		'inputs','++++',...
		'position',[1090,254,1125,361])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Outport']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Outport']],...
		'position',[1200,300,1220,320])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'ForeGround',2,...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'position',[665,505,670,510])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Partie principale du gradient de gravit']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Partie principale du gradient de gravit']],...
		'position',[945,540,950,545])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'MATLAB Fcn','gradgrav(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']],...
		'Mask Display','GRADIENT PRINCIPAL\n\nDE GRAVITE',...
		'position',[630,554,805,606])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[45,495;235,495])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[840,395;1055,395;1055,320;1085,320])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[705,395;735,395;735,455;765,455])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[735,395;795,395])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[835,455;890,455])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[590,395;635,395])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[445,495;470,495;470,450;535,450])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[495,340;535,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[355,340;370,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,305;315,305])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,340;315,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,375;315,375])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,100;980,100])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,135;980,135])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,170;980,170])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,260;970,260])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,295;970,295])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1010,295;1085,295])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,330;970,330])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1020,135;1075,135;1085,270])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[75,495;75,580;625,580])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1130,310;1195,310])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[810,580;1070,580;1070,345;1085,345])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],...
		'Mask Display','Couples\nperturbateurs Mp')


%     Finished composite block ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1'].

set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],...
		'ForeGround',2,...
		'hide name',0,...
		'position',[285,282,415,338])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain3'])
set_param([sys,'/','F(X,t) pour la rotation/Gain3'],...
		'hide name',0,...
		'position',[885,248,925,282])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/out_1'],...
		'position',[1125,325,1145,345])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain1'])
set_param([sys,'/','F(X,t) pour la rotation/Gain1'],...
		'hide name',0,...
		'position',[1070,322,1095,348])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Sum1'])
set_param([sys,'/','F(X,t) pour la rotation/Sum1'],...
		'position',[970,194,1010,476])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Suppression de',13,'tous les couples']])
set_param([sys,'/',['F(X,t) pour la rotation/Suppression de',13,'tous les couples']],...
		'position',[900,205,905,210])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain'])
set_param([sys,'/','F(X,t) pour la rotation/Gain'],...
		'hide name',0,...
		'position',[460,293,500,327])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Sum2'])
set_param([sys,'/','F(X,t) pour la rotation/Sum2'],...
		'position',[680,128,720,372])


%     Subsystem  'F(X,t) pour la rotation/Subsystem5'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5'],'Location',[298,510,1135,815])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem5/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/out_1'],...
		'position',[635,140,655,160])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem5/To Workspace2'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/To Workspace2'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'mat-name','com_avant',...
		'buffer','points_trace',...
		'position',[205,38,295,52])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Subsystem5/Demux'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Demux'],...
		'outputs','3',...
		'position',[140,39,180,71])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem5/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/in_1'],...
		'position',[65,100,85,120])


%     Subsystem  'F(X,t) pour la rotation/Subsystem5/Subsystem5'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],'Location',[580,281,938,501])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace3'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'mat-name','valeur_creneau',...
		'buffer','points_trace',...
		'position',[245,178,340,192])

add_block('built-in/Clock',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Clock'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Clock'],...
		'BackGround',7,...
		'position',[45,113,80,147])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/MATLAB Fcn'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/MATLAB Fcn'],...
		'MATLAB Fcn','creneau(u)',...
		'Output Width','1',...
		'position',[120,113,180,147])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/in_1'],...
		'position',[115,75,135,95])

add_block('built-in/Product',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Product'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/Product'],...
		'hide name',0,...
		'position',[230,64,250,151])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/out_1'],...
		'position',[295,100,315,120])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/CRENEAU'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/CRENEAU'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[155,25,160,30])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace4'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5/To Workspace4'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'mat-name','temps_creneau',...
		'buffer','points_trace',...
		'position',[125,193,220,207])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[185,130;225,130])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[205,130;205,185;240,185])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[85,130;115,130])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[255,110;290,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[140,85;225,85])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],[85,130;85,195;120,200])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],...
		'Mask Display','COUPURES\nMAGNETOMETRES\nDE MESURES')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem5/Subsystem5'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Subsystem5'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'position',[190,84,315,136])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem5/Gain2'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Gain2'],...
		'hide name',0,...
		'Gain','pulse',...
		'position',[340,90,475,130])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Subsystem5/Sum1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Sum1'],...
		'hide name',0,...
		'position',[525,69,545,231])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem5/Gain1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Gain1'],...
		'hide name',0,...
		'Gain','1-pulse',...
		'position',[230,170,365,210])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem5/To Workspace3'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/To Workspace3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'mat-name','com_apres',...
		'buffer','points_trace',...
		'position',[580,23,675,37])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Subsystem5/Demux1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5/Demux1'],...
		'outputs','3',...
		'position',[505,24,545,56])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[90,110;125,110;125,190;225,190])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[105,110;105,55;135,55])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[185,45;200,45])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[550,150;630,150])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[320,110;335,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[370,190;520,190])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[480,110;520,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[90,110;185,110])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[550,30;575,30])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem5'],[480,110;480,40;500,40])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem5'],...
		'Mask Display','PULSE\noui/non')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem5'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem5'],...
		'hide name',0,...
		'position',[560,166,645,214])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain2'])
set_param([sys,'/','F(X,t) pour la rotation/Gain2'],...
		'position',[485,175,530,205])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Q'])
set_param([sys,'/','F(X,t) pour la rotation/Q'],...
		'position',[280,445,285,450])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/To Workspace4'])
set_param([sys,'/','F(X,t) pour la rotation/To Workspace4'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','quater',...
		'buffer','points_trace',...
		'position',[470,450,535,480])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Couple roue unidimensionnel sur tangage'])
set_param([sys,'/','F(X,t) pour la rotation/Couple roue unidimensionnel sur tangage'],...
		'position',[705,435,710,440])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/out_2'])
set_param([sys,'/','F(X,t) pour la rotation/out_2'],...
		'Port','2',...
		'position',[1080,495,1100,515])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Sortie vitesse roue'])
set_param([sys,'/','F(X,t) pour la rotation/Sortie vitesse roue'],...
		'position',[985,515,990,520])

add_block('built-in/State-Space',[sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']])
set_param([sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'A','[]',...
		'B','[]',...
		'C','[]',...
		'D','inv(Asat)',...
		'Mask Display','Inv(Asat)',...
		'Mask Type','Matrix Gain')
set_param([sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'Mask Help','Multiplies input vector by entered matrix to produce output vector (y=Au).',...
		'position',[750,242,850,288])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/magneto.m1'])
set_param([sys,'/','F(X,t) pour la rotation/magneto.m1'],...
		'hide name',0,...
		'MATLAB Fcn','derivrot(u)',...
		'Output Width','3',...
		'Mask Display','DERIVROT(u)',...
		'position',[375,379,500,431])


%     Subsystem  'F(X,t) pour la rotation/Calcul des commandes, moments et roue'.

new_system([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],'Location',[4,107,1267,1004])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Size',18)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'position',[595,10,600,15])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'ForeGround',2,...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'Font Size',14,...
		'position',[605,82,610,87])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/pqrQ'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/pqrQ'],...
		'position',[445,205,450,210])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/in_1'],...
		'hide name',0,...
		'position',[430,240,450,260])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],'Location',[11,113,1274,1010])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','cpl_laurens2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']],...
		'buffer','points_trace',...
		'position',[1030,324,1150,346])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'ForeGround',1)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'BackGround',2)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'MATLAB Fcn','mom_prop(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'Output Width','4')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'position',[825,413,885,447])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue']],...
		'position',[755,570,760,575])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace12']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace12']],...
		'hide name',0,...
		'mat-name','mz',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace12']],...
		'position',[1110,279,1145,301])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace11']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace11']],...
		'hide name',0,...
		'mat-name','my',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace11']],...
		'position',[1110,254,1145,276])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace10']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace10']],...
		'hide name',0,...
		'mat-name','mx',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace10']],...
		'position',[1110,229,1145,251])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'ForeGround',1,...
		'BackGround',2)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'MATLAB Fcn','prodvect(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'position',[1065,418,1125,452])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],'Location',[442,254,1087,454])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/in_1']],...
		'hide name',0,...
		'position',[25,100,45,120])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_3']],...
		'hide name',0,...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_3']],...
		'position',[360,130,380,150])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_2']],...
		'hide name',0,...
		'position',[360,70,380,90])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_1']],...
		'hide name',0,...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/out_1']],...
		'position',[360,100,380,120])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],'Location',[4,192,1154,487])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_3']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_3']],...
		'position',[760,185,780,205])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_2']],...
		'position',[760,145,780,165])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/out_1']],...
		'position',[760,105,780,125])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],'Location',[4,386,1102,910])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_1']],...
		'position',[115,190,135,210])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ rel satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ rel satellite']],...
		'position',[690,55,695,60])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ  liss']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ  liss']],...
		'position',[775,115,780,120])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive pulse non lisse']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive pulse non lisse']],...
		'position',[790,425,795,430])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_3']],...
		'position',[800,390,820,410])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_1']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_1']],...
		'position',[780,45,800,65])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/out_2']],...
		'position',[790,145,810,165])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Sum']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Sum']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Sum']],...
		'position',[640,134,665,321])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],'Location',[4,50,853,441])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_2']],...
		'position',[95,250,115,270])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Temps courant',13,'de la simulation']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Temps courant',13,'de la simulation']],...
		'position',[70,295,75,300])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],'Location',[125,1114771,765,1115016])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'position',[175,70,180,75])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum1']],...
		'position',[475,54,495,176])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']],...
		'MATLAB Fcn','fix(u/T)*T')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']],...
		'Output Width','1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T2']],...
		'position',[365,128,425,162])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Sum4']],...
		'position',[295,107,320,183])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']],...
		'Value','instant_injection')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Constant4']],...
		'position',[165,154,270,176])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Calcul t-nT1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Calcul t-nT1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/Calcul t-nT1']],...
		'position',[460,206,465,211])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/out_1']],...
		'position',[525,105,545,125])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem/in_1']],...
		'position',[245,115,265,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[325,145;360,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[345,145;345,85;470,85])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[275,165;290,165])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[430,145;470,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[500,115;520,115])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],[270,125;290,125])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],...
		'Mask Display','Mise en place\ndes priodes')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Subsystem']],...
		'position',[155,236,295,284])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'mat-name','der_seconde')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'buffer','20000')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace7']],...
		'position',[705,568,825,602])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau']],...
		'position',[785,211,790,216])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/B(n)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/B(n)']],...
		'position',[620,196,625,201])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie de t-tinj-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie de t-tinj-nT']],...
		'position',[395,310,400,315])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/out_1']],...
		'position',[715,215,735,235])

add_block('built-in/To File',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']],...
		'File name','derbchap.mat')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']],...
		'Matrix name','derb_chapeau')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To File']],...
		'position',[635,134,730,166])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'MATLAB Fcn','storderB(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Partie',13,'entire n1']],...
		'position',[515,208,575,242])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/in_1']],...
		'position',[105,135,125,155])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t-tinj -nT<=Ta']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t-tinj -nT<=Ta']],...
		'position',[390,290,395,295])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'mat-name','derb_chapeau')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'buffer','20000')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/To Workspace5']],...
		'position',[630,268,750,302])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']],...
		'Value','Ta/2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Constant1']],...
		'position',[325,189,360,211])

add_block('built-in/Relational Operator',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Relational',13,'Operator']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Relational',13,'Operator']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Relational',13,'Operator']],...
		'position',[375,168,400,292])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t--tinj -nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/t--tinj -nT']],...
		'position',[335,235,340,240])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Mux1']],...
		'inputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Mux1']],...
		'position',[455,206,485,239])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie B_prime']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Sortie B_prime']],...
		'position',[550,175,555,180])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Entre de dB//dt //xyz ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/Entre de dB//dt //xyz ']],...
		'position',[355,115,360,120])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/MESURE ECHANTILLONEE DE LA DERIVEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta']],...
		'position',[425,0,430,5])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'Font Size',12)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_lin est la valeur linarise conserve aprs la mesure']],...
		'position',[405,75,410,80])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'Font Size',12)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE/derb_chapeau est la valeur constante conserve aprs la mesure',13,'']],...
		'position',[420,45,425,50])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[120,260;150,260])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[580,225;710,225])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[595,225;595,150;630,150])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[595,225;595,285;625,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[490,225;510,225])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[405,230;450,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[365,200;370,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[130,145;430,145;430,215;450,215])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],[300,260;370,260])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],...
		'Mask Display','DERIVEE DE B\n\nMESUREE\n\nPERIODE n')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/DERIVEE MESUREE']],...
		'position',[310,260,445,340])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Calcul t-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Calcul t-nT']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Calcul t-nT']],...
		'position',[410,385,415,390])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Product']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Product']],...
		'position',[535,227,560,323])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],'Location',[198,215,1097,749])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_1']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_1']],...
		'position',[615,220,635,240])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_3']],...
		'position',[615,100,635,120])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/in_1']],...
		'position',[65,210,85,230])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre de B(t) //xyz ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre de B(t) //xyz ']],...
		'position',[115,180,120,185])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Calcul t-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Calcul t-nT']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Calcul t-nT']],...
		'position',[265,430,270,435])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie champ B chapeau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie champ B chapeau']],...
		'position',[620,180,625,185])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t - tinj- nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t - tinj- nT']],...
		'position',[335,350,340,355])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre du champ en axes satellite ',13,' l''instant t de la simulation',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Entre du champ en axes satellite ',13,' l''instant t de la simulation',13,'']],...
		'position',[215,200,220,205])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Mux1']],...
		'inputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Mux1']],...
		'position',[445,211,475,244])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'MATLAB Fcn','storcham(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Partie',13,'entire n1']],...
		'position',[505,213,565,247])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_2']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/out_2']],...
		'position',[615,420,635,440])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']],...
		'mat-name','champ1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']],...
		'buffer','20000')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/To Workspace3']],...
		'position',[610,272,660,288])

add_block('built-in/Relational Operator',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Relational',13,'Operator']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Relational',13,'Operator']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Relational',13,'Operator']],...
		'position',[375,258,400,382])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']],...
		'Value','Ta/2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Constant1']],...
		'position',[305,279,340,301])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie de to = t -tinj - nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Sortie de to = t -tinj - nT']],...
		'position',[585,385,590,390])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Temps courant',13,'de la simulation']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Temps courant',13,'de la simulation']],...
		'position',[75,280,80,285])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],'Location',[163,376,803,621])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/out_1']],...
		'position',[555,105,575,125])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'Font Size',12)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/CALCUL DE LA PARTIE FRACTIONNAIRE',13,'DE LA PERIODE ENTRE nT ET (n+1)T']],...
		'position',[315,0,320,5])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Le temps courant de la simulation',13,'commence  l''instant d''injection ']],...
		'position',[135,100,140,105])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum1']],...
		'position',[475,54,495,176])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']],...
		'MATLAB Fcn','fix(u/T)*T')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']],...
		'Output Width','1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Partie entire ',13,'de (t-tinj)T',13,' X par T']],...
		'position',[365,128,425,162])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']],...
		'inputs','+-')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Sum4']],...
		'position',[295,107,320,183])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']],...
		'Value','instant_injection')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Constant4']],...
		'position',[165,154,270,176])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Calcul t-nT1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Calcul t-nT1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/Calcul t-nT1']],...
		'position',[460,206,465,211])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem/in_1']],...
		'position',[245,115,265,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[325,145;360,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[345,145;345,85;470,85])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[275,165;290,165])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[430,145;470,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[500,115;550,115])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],[270,125;290,125])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],...
		'Mask Display','Mise en place\ndes priodes')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Subsystem']],...
		'position',[115,326,255,374])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Clock2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Clock2']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/Clock2']],...
		'position',[45,330,85,370])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t-nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/t-nT']],...
		'position',[930,355,935,360])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'ForeGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE/MESURE ECHANTILLONEE DU CHAMP',13,'SUR LA PERIODE N, AU TEMPS to+Ta//2']],...
		'position',[360,5,365,10])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[90,220;440,220])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[390,220;390,110;610,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[90,350;110,350])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[345,290;370,290])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[570,230;610,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[590,230;590,280;605,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[405,320;430,320;440,235])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[480,230;500,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[260,350;285,350;285,430;610,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],[285,350;370,350])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],...
		'Mask Display','CHAMP MESURE\n\n periode n')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/CHAMP MESURE']],...
		'position',[300,165,435,235])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive B']],...
		'position',[495,270,500,275])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/derb_chapeau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/derb_chapeau']],...
		'position',[535,335,540,340])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive de',13,'B relle']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Drive de',13,'B relle']],...
		'position',[230,262,285,298])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/t -tinj - nT']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/t -tinj - nT']],...
		'position',[500,225,505,230])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/B(n)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/B(n)']],...
		'position',[535,175,540,180])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ magntique',13,'terrestre',13,'(repre satellite)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Champ magntique',13,'terrestre',13,'(repre satellite)']],...
		'position',[130,120,135,125])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Temps']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Temps']],...
		'position',[130,285,135,290])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/in_2']],...
		'position',[110,310,130,330])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux']],...
		'position',[505,94,545,126])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux1']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux1']],...
		'position',[680,349,720,381])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'mat-name','b_reel_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace4']],...
		'position',[580,84,660,116])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'mat-name','derb_pulse_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace6']],...
		'position',[755,339,870,371])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'mat-name','champ_pulse')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace5']],...
		'position',[745,213,865,247])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux2']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux2']],...
		'position',[750,279,790,311])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux3']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/Demux3']],...
		'position',[375,419,415,451])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'mat-name','derb_reel_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace8']],...
		'position',[460,409,540,441])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'mat-name','b_pulse_roulis')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3/To Workspace7']],...
		'position',[825,266,925,304])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[670,230;740,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[685,230;685,155;785,155])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[450,300;530,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[475,300;475,400;795,400])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[440,175;465,175;465,55;775,55])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[135,320;305,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[290,280;305,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[565,275;635,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[440,225;455,225;455,250;530,250])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[440,200;580,200;580,180;635,180])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[140,200;295,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[275,200;190,200;190,280;225,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[465,110;500,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[615,400;615,365;675,365])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[550,100;575,100])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[725,355;750,355])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[795,285;820,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[710,230;710,295;745,295])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[420,425;455,425])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],[290,280;290,435;370,435])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'Mask Display','Champ rel\n&\nchamp mesur')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'Mask Help','Entre 1= champ satellite, entre 2=temps, sortie 1= champ satellite, sortie2= champ liss, sortie 3 = drive non lisse')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Subsystem3']],...
		'position',[595,97,730,213])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Drive pulse crneau(mesure )']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Drive pulse crneau(mesure )']],...
		'position',[930,100,935,105])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ puls liss linairement ( mesure )']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ puls liss linairement ( mesure )']],...
		'position',[950,140,955,145])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ rel']],...
		'position',[870,185,875,190])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],'Location',[382,182,1228,651])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'MATLAB Fcn','champmgn(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'position',[250,95,425,145])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Clock']],...
		'position',[180,104,215,136])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/BX BY BZ']],...
		'position',[340,70,345,75])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Vecteur d''tat ',13,'( 7 composantes)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Vecteur d''tat ',13,'( 7 composantes)']],...
		'position',[70,235,75,240])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/in_1']],...
		'position',[50,190,70,210])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/out_1']],...
		'position',[705,150,725,170])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Sortie Bx By Bz']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Sortie Bx By Bz']],...
		'position',[725,200,730,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Equateur au temps 0 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Equateur au temps 0 s']],...
		'position',[150,75,155,80])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']],...
		'MATLAB Fcn','champsat(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']],...
		'position',[560,143,620,177])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Mux1']],...
		'inputs','[3 7]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_/Mux1']],...
		'position',[485,80,530,240])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],[220,120;245,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],[430,120;480,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],[625,160;700,160])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],[75,200;480,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],[535,160;555,160])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],...
		'Mask Display','CHAMP MAGNETIQUE EN\nAXES SATELLITE\nEntre=vecteur d''tat\nSortie Bx By Bz')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Champ magntique',13,'en axes satellite_']],...
		'position',[240,88,440,162])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/in_1']],...
		'position',[160,115,180,135])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem/Clock']],...
		'position',[150,168,190,202])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[195,185;590,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[735,115;755,115])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[735,155;755,155])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[735,195;755,195])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[185,125;235,125])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],[445,125;590,125])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],...
		'Mask Display','CHAMP ET DERIVEE\n\nPULSES + CHAMP REEL')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Subsystem']],...
		'position',[85,64,285,156])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Drive pulse crneau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Drive pulse crneau']],...
		'position',[460,65,465,70])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ puls linaire']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ puls linaire']],...
		'position',[455,95,460,100])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES/Champ rel']],...
		'position',[425,125,430,130])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[290,80;355,80])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[290,110;355,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[290,140;355,140])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],[50,110;80,110])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],...
		'Mask Display','CHAMP ET DERIVEE\n\nPULSES')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/CHAMP ET DERIVEE',13,' PULSES']],...
		'position',[80,263,220,337])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc']],...
		'position',[485,340,490,345])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B ']],...
		'position',[505,295,510,300])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B']],...
		'position',[260,345,265,350])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''c']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''c']],...
		'position',[515,380,520,385])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative']],...
		'position',[455,399,500,421])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux']],...
		'hide name',0,...
		'inputs','[3 3 3 3]',...
		'position',[530,251,565,434])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''']],...
		'position',[445,250,450,255])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux3']],...
		'inputs','[3 7]',...
		'position',[275,354,310,436])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/in_1']],...
		'position',[10,289,25,311])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Entre vecteur d''tat ',13,'7 composantes p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Entre vecteur d''tat ',13,'7 composantes p q r Q']],...
		'position',[140,370,145,375])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','Bc_prime')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']],...
		'buffer','points_trace',...
		'position',[425,439,490,461])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']],...
		'hide name',0,...
		'mat-name','Bc',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']],...
		'position',[430,469,495,491])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux1']],...
		'hide name',0,...
		'inputs','[3 1]',...
		'position',[760,338,795,522])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux4']],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux4']],...
		'position',[1005,227,1045,303])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace15']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace15']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','B_prime')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace15']],...
		'buffer','points_trace',...
		'position',[345,224,410,246])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ rel']],...
		'position',[860,590,865,595])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pulse crneau']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pulse crneau']],...
		'position',[280,250,285,255])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace14']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace14']],...
		'hide name',0,...
		'mat-name','cpl_roue_brut')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace14']],...
		'buffer','points_trace',...
		'position',[790,559,900,581])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux2']],...
		'hide name',0,...
		'inputs','3',...
		'position',[1055,660,1080,730])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant']],...
		'hide name',0,...
		'Value','0',...
		'position',[1000,660,1020,680])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant1']],...
		'hide name',0,...
		'Value','0',...
		'position',[1000,710,1020,730])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']],...
		'hide name',0,...
		'mat-name','M4',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']],...
		'position',[1150,364,1185,386])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux4']],...
		'hide name',0,...
		'inputs','[3 3]',...
		'position',[1015,386,1040,479])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc '].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],'Location',[4,566,324,746])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'hide name',0,...
		'MATLAB Fcn','champmgn(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'position',[105,75,280,125])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Clock']],...
		'position',[35,84,70,116])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /BX BY BZ']],...
		'position',[195,50,200,55])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /out_1']],...
		'position',[310,90,330,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[75,100;100,100])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[285,100;305,100])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],...
		'Mask Display','CALCUL\nBc')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc '].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],...
		'position',[335,367,395,403])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'outputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']],...
		'position',[905,392,945,468])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B+pqr+Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B+pqr+Q']],...
		'position',[285,555,290,560])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain1']],...
		'position',[585,557,610,583])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_2']],...
		'Port','2',...
		'position',[1125,760,1145,780])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace13']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace13']],...
		'hide name',0,...
		'mat-name','cpl_roue',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace13']],...
		'position',[1060,799,1130,821])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'uniaxial']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'uniaxial']],...
		'position',[1100,825,1105,830])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'dimension 3',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'dimension 3',13,'']],...
		'position',[1165,665,1170,670])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain2']],...
		'hide name',0,...
		'position',[660,332,685,358])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'MATLAB Fcn','laur0(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'position',[580,328,640,362])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum2']],...
		'hide name',0,...
		'position',[710,305,740,465])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','M1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']],...
		'buffer','points_trace',...
		'position',[615,289,650,311])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Puls',13,'linaire']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Puls',13,'linaire']],...
		'position',[390,280,395,285])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum4']],...
		'hide name',0,...
		'inputs','+-',...
		'position',[520,139,550,206])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],'Location',[67,304,1152,889])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/out_1']],...
		'position',[970,380,990,400])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],'Location',[259,494,1089,769])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Derivative1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Derivative1']],...
		'position',[170,162,220,188])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain6']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain6']],...
		'Gain','KFG')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain6']],...
		'position',[240,163,310,187])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Sum5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Sum5']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Sum5']],...
		'position',[330,55,360,215])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain3']],...
		'Gain','KRG')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain3']],...
		'position',[240,83,310,107])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/in_1']],...
		'position',[95,85,115,105])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain7']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain7']],...
		'Gain','-1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain7']],...
		'position',[390,123,445,147])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/out_1']],...
		'position',[700,125,720,145])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain  d''essai']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain  d''essai']],...
		'Gain','gain_pour_essai')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion/Gain  d''essai']],...
		'position',[535,120,585,150])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[365,135;385,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[120,95;235,95])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[215,95;150,95;150,175;165,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[225,175;235,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[315,175;325,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[315,95;325,95])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[590,135;695,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],[450,135;530,135])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],...
		'Mask Display','PD\nQUATERNION')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/PD quaternion']],...
		'position',[285,211,390,259])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Cx']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Cx']],...
		'position',[585,220,590,225])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Sortie du',13,'couple PD']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Sortie du',13,'couple PD']],...
		'position',[570,166,575,171])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/C']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/C']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/C']],...
		'position',[635,191,640,196])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/My']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/My']],...
		'position',[700,366,710,381])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Couple roue dim 1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Couple roue dim 1']],...
		'position',[715,236,720,241])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/in_1']],...
		'position',[35,275,55,295])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],'Location',[215,148,1123,800])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Champ B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Champ B']],...
		'position',[625,240,630,245])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Angle de rotation',13,'du quaternion ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Angle de rotation',13,'du quaternion ']],...
		'position',[560,145,565,150])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_1']],...
		'position',[635,190,655,210])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace18']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace18']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace18']],...
		'mat-name','B')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace18']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace18']],...
		'position',[505,219,570,241])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/PREPARATION DU COUPLE PD SUR LE QUATERNION',13,'COUPLE D''AXE UNITAIREQUATERNION']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/PREPARATION DU COUPLE PD SUR LE QUATERNION',13,'COUPLE D''AXE UNITAIREQUATERNION']],...
		'position',[420,65,425,70])

add_block('built-in/Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Fcn1']],...
		'Expr','acos(2*u^2-1)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Fcn1']],...
		'position',[185,190,225,210])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Q(1)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Q(1)']],...
		'position',[150,360,155,365])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/in_1']],...
		'position',[15,340,35,360])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn1']],...
		'MATLAB Fcn','VectUnit(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn1']],...
		'position',[155,248,215,282])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace19']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace19']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace19']],...
		'mat-name','unitB')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace19']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace19']],...
		'position',[275,254,340,276])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn2']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn2']],...
		'MATLAB Fcn','VectUnit(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn2']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/MATLAB Fcn2']],...
		'position',[165,413,225,447])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace20']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace20']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace20']],...
		'mat-name','unitQ')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace20']],...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/To Workspace20']],...
		'position',[275,415,330,445])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Demux6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Demux6']],...
		'outputs','[3 3 1 3]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/Demux6']],...
		'position',[55,236,105,459])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_3']],...
		'Port','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_3']],...
		'position',[635,265,655,285])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/out_2']],...
		'position',[635,360,655,380])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/unitQ',13,'unitaire de Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/unitQ',13,'unitaire de Q']],...
		'position',[640,400,645,405])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/unitaire',13,'de la rotation Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/unitaire',13,'de la rotation Q']],...
		'position',[305,448,310,453])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/pqr inutiliss']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ/pqr inutiliss']],...
		'position',[60,305,65,310])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[110,265;150,265])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[125,265;125,230;500,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[465,230;465,275;630,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[230,430;270,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[250,430;250,370;630,370])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[230,200;630,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[40,350;50,350])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[110,375;135,375;135,200;180,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[110,430;160,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],[220,265;270,265])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],...
		'Mask Display','CALCUL unitB-unitQ \nSTOP en 2\nCALCUL ANGLE\nQUATERNION EN 1 ')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitB-unitQ']],...
		'position',[85,212,245,358])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Couple',13,'roue dim 1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Couple',13,'roue dim 1']],...
		'position',[785,496,790,501])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Cy']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Cy']],...
		'position',[585,255,590,260])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Cz']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Cz']],...
		'position',[585,290,590,295])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Champ B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Champ B']],...
		'position',[315,311,320,316])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/out_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/out_2']],...
		'position',[855,485,875,505])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Demux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Demux']],...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Demux']],...
		'position',[455,234,490,336])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Moment',13,'magntique']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Moment',13,'magntique']],...
		'position',[770,441,775,446])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/M']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/M']],...
		'position',[775,366,780,371])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mx']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mx']],...
		'position',[730,331,735,336])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mz']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mz']],...
		'position',[730,401,735,406])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mux6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mux6']],...
		'hide name',0,...
		'inputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mux6']],...
		'position',[765,339,795,441])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Constant']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Constant']],...
		'hide name',0,...
		'Value','0')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Constant']],...
		'position',[725,380,745,400])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Gain']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Gain']],...
		'Gain','-1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Gain']],...
		'position',[635,410,685,440])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],'Location',[33,479,253,634])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/in_1']],...
		'position',[15,25,35,45])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/in_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/in_2']],...
		'Port','2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/in_2']],...
		'position',[15,90,35,110])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/out_1']],...
		'position',[185,55,205,75])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/Sum']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/Sum']],...
		'inputs','+')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/Sum']],...
		'position',[125,55,145,75])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/Product']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product/Product']],...
		'position',[65,55,90,75])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],[150,65;180,65])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],[95,65;120,65])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],[40,35;60,60])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],[40,100;60,70])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],...
		'Mask Display','.\n')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],...
		'Mask Type','Dot Product')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],...
		'Mask Dialogue','Inner (dot) product.\ny=sum(u1.*u2)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],...
		'Mask Help','Computes the inner (dot) product of the inputs: sum(u1.*u2).  The inputs are vectors of the same length. Output is a scalar.')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Inner',13,'Product']],...
		'position',[680,271,705,309])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitQ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/unitQ']],...
		'position',[470,351,475,356])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product2']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product2']],...
		'position',[540,264,565,291])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product1']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product1']],...
		'position',[540,229,565,256])

add_block('built-in/Product',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Product3']],...
		'position',[540,299,565,326])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mux5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mux5']],...
		'hide name',0,...
		'inputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/Mux5']],...
		'position',[620,229,650,331])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/REGULATION SUR L''ANGLE DE ROTATION DU QUATERNION']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/REGULATION SUR L''ANGLE DE ROTATION DU QUATERNION']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/REGULATION SUR L''ANGLE DE ROTATION DU QUATERNION']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION/REGULATION SUR L''ANGLE DE ROTATION DU QUATERNION']],...
		'position',[505,45,510,50])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[710,290;715,290;715,495;850,495])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[570,245;615,245])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[605,245;605,425;630,425])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[690,425;760,425])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[750,390;760,390])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[570,315;615,315])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[585,315;585,355;760,355])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[655,280;675,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[250,235;280,235])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[250,285;450,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[495,250;535,250])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[495,285;535,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[570,280;615,280])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[395,235;410,235;410,220;500,220;500,305;535,305])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[500,271;535,270])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[495,320;535,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[500,236;535,235])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[60,285;80,285])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[250,335;405,335;405,385;665,385;675,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],[800,390;965,390])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],...
		'Mask Display','REGULATION ANGLE\nQUATERNION')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],...
		'ForeGround',1,...
		'BackGround',3)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/REGULATION PD',13,'SUR ANGLE QUATERNION']],...
		'position',[335,527,485,583])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace18']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace18']],...
		'hide name',0,...
		'mat-name','M3',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace18']],...
		'position',[670,459,705,481])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain']],...
		'hide name',0,...
		'position',[600,412,625,438])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple',13,'de commande',13,' raliser']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple',13,'de commande',13,' raliser']],...
		'position',[1235,525,1240,530])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum3']],...
		'hide name',0,...
		'position',[1155,395,1185,555])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Arrt possible']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Arrt possible']],...
		'position',[595,590,600,595])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_3']],...
		'position',[1225,465,1245,485])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Possibilit de',13,'supprimer M1 ou M3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Possibilit de',13,'supprimer M1 ou M3']],...
		'position',[635,205,640,210])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Somme des',13,'moments magntiques',13,'M1+M2+M3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Somme des',13,'moments magntiques',13,'M1+M2+M3']],...
		'position',[735,235,740,240])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/MATLAB Fcn1']],...
		'hide name',0,...
		'MATLAB Fcn','VectUnit(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/MATLAB Fcn1']],...
		'position',[295,643,355,677])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace8']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace8']],...
		'hide name',0,...
		'mat-name','unitQ',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace8']],...
		'position',[390,645,445,675])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/unitaire',13,'de la rotation Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/unitaire',13,'de la rotation Q']],...
		'position',[410,679,415,684])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum5']],...
		'hide name',0,...
		'position',[520,59,550,126])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace16']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace16']],...
		'hide name',0,...
		'mat-name','norm_PB_moins_B')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace16']],...
		'buffer','points_trace',...
		'position',[725,161,845,189])

add_block('built-in/Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Fcn']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Fcn']],...
		'Expr','(u(1)^2+u(2)^2+u(3)^2)^0.5',...
		'position',[655,165,695,185])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace19']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace19']],...
		'hide name',0,...
		'mat-name','norm_PB_plus_B')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace19']],...
		'buffer','points_trace',...
		'position',[715,81,835,109])

add_block('built-in/Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Fcn1']],...
		'Expr','(u(1)^2+u(2)^2+u(3)^2)^0.5',...
		'position',[645,85,685,105])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace20']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace20']],...
		'hide name',0,...
		'mat-name','PB_plus_B')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace20']],...
		'buffer','points_trace',...
		'position',[665,46,785,74])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace21']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace21']],...
		'hide name',0,...
		'mat-name','PB_moins_B')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace21']],...
		'buffer','points_trace',...
		'position',[665,126,785,154])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[505,410;495,450])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[400,385;430,385;430,365;525,365])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[400,385;400,480;425,480])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[225,300;485,300;485,320;525,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[250,300;250,375;270,375])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[430,385;430,410;450,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[505,410;525,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[570,345;575,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[225,275;525,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[30,300;75,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[30,300;30,415;270,415])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[490,570;580,570])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[745,385;755,385])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[800,430;820,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[890,430;900,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[225,325;235,325;235,615;990,615;990,455;1010,455])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[950,410;1010,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1045,435;1060,435])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1130,435;1150,435])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1190,475;1220,475])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1025,670;1050,670])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1025,720;1050,720])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1085,695;1115,695;1115,515;1150,515])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1190,475;1190,335;1155,335])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[970,410;970,265;1000,265])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1050,240;1105,240])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1050,265;1105,265])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1050,290;1105,290])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[615,570;785,570])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[950,450;967,450;967,695;1050,695])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[725,570;725,475;755,475])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[315,395;315,555;330,555])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[425,275;415,235])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[490,540;580,540;580,425;595,425])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[630,425;705,425])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1130,435;1130,375;1145,375])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[970,695;970,770;1120,770])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1030,770;1030,810;1055,810])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[645,345;655,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[690,345;705,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[690,345;690,300;655,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[465,365;465,155;515,155])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[480,300;480,190;515,190])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[555,175;650,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[700,175;720,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[630,425;630,470;665,470])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[360,660;385,660])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[235,615;235,660;290,660])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[555,95;640,95])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[690,95;710,95])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[470,155;465,155;465,75;515,75])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[480,190;480,110;515,110])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[590,95;590,60;660,60])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[590,175;590,140;660,140])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],...
		'Mask Display','CALCUL DES MOMENTS\n\nMAGNETIQUES\n\nET COUPLE ROUE')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],...
		'position',[505,199,720,301])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple total magntique + roue',13,'de la rgulation de Laurens amliore']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple total magntique + roue',13,'de la rgulation de Laurens amliore']],...
		'position',[900,200,905,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Vecteur d''tat']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Vecteur d''tat']],...
		'position',[375,235,380,240])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_7'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_7'],...
		'hide name',0,...
		'Port','2',...
		'position',[750,265,770,285])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_5'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_5'],...
		'hide name',0,...
		'position',[750,215,770,235])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple roue uniaxial'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple roue uniaxial'],...
		'position',[850,260,855,265])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[455,250;500,250])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[725,225;745,225])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[725,275;745,275])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],...
		'Mask Display','REGULATION BOUSSOLE\n\nTECHNIQUE LAURENS\n\nPULSEE TYPE 2')


%     Finished composite block 'F(X,t) pour la rotation/Calcul des commandes, moments et roue'.

set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],...
		'position',[205,171,435,249])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Demux1'])
set_param([sys,'/','F(X,t) pour la rotation/Demux1'],...
		'outputs','[1 3]',...
		'position',[355,536,395,569])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/unitaire',13,'de la rotation Q']])
set_param([sys,'/',['F(X,t) pour la rotation/unitaire',13,'de la rotation Q']],...
		'position',[550,580,555,585])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/To Workspace7'])
set_param([sys,'/','F(X,t) pour la rotation/To Workspace7'],...
		'hide name',0,...
		'mat-name','unitQ',...
		'buffer','points_trace',...
		'position',[530,545,585,575])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/MATLAB Fcn1'])
set_param([sys,'/','F(X,t) pour la rotation/MATLAB Fcn1'],...
		'hide name',0,...
		'MATLAB Fcn','VectUnit(u)',...
		'Output Width','3',...
		'position',[435,543,495,577])


%     Subsystem  'F(X,t) pour la rotation/Subsystem'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],'Location',[4,50,970,451])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Subsystem/Inversion',13,'signe',13,'(action//roue)',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Inversion',13,'signe',13,'(action//roue)',13,'']],...
		'ForeGround',2,...
		'Gain','-1',...
		'position',[115,122,145,148])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem/Gain3'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/Gain3'],...
		'ForeGround',2,...
		'Gain','30/pi',...
		'position',[445,118,470,152])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace5'],...
		'ForeGround',2,...
		'BackGround',7,...
		'hide name',0,...
		'Font Size',14,...
		'mat-name','vit_roue',...
		'buffer','points_trace',...
		'position',[515,120,595,150])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue ',13,'en rd//s']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue ',13,'en rd//s']],...
		'ForeGround',2,...
		'position',[520,210,525,215])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/in_1'],...
		'ForeGround',2,...
		'position',[20,125,40,145])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue',13,'en tours//mn']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue',13,'en tours//mn']],...
		'ForeGround',2,...
		'position',[455,70,460,75])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Subsystem/Inertie',13,'de la ',13,'roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Inertie',13,'de la ',13,'roue']],...
		'ForeGround',2,...
		'BackGround',7,...
		'Gain','1/J_ROUE',...
		'position',[195,115,230,155])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Subsystem/SUIVI DE LA ROUE'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/SUIVI DE LA ROUE'],...
		'ForeGround',2,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[390,10,395,15])

add_block('built-in/Integrator',[sys,'/','F(X,t) pour la rotation/Subsystem/Integrator2'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/Integrator2'],...
		'ForeGround',2,...
		'BackGround',7,...
		'Initial','rotation_initiale',...
		'position',[280,113,325,157])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'],...
		'ForeGround',2,...
		'BackGround',7,...
		'hide name',0,...
		'Font Size',14,...
		'mat-name','vitesseroue',...
		'buffer','points_trace')
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'],...
		'position',[445,179,590,211])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/out_1'],...
		'hide name',0,...
		'position',[610,330,630,350])

add_block('built-in/To File',[sys,'/','F(X,t) pour la rotation/Subsystem/To File'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To File'],...
		'ForeGround',2,...
		'BackGround',7,...
		'File name','vit_roue.mat',...
		'Matrix name','vit_roue_rd_s',...
		'position',[455,247,570,273])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Sortie de la vitesse ',13,'de la roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Sortie de la vitesse ',13,'de la roue']],...
		'position',[485,320,490,325])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[475,135;510,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[150,135;190,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[45,135;110,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[235,135;275,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[330,135;440,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[395,135;395,195;440,195])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[395,195;395,345;605,340])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[420,195;420,260;450,260])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],...
		'Mask Display','EVOLUTION DE LA\nVITESSE DE LA ROUE')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'position',[710,474,855,536])
add_line([sys,'/','F(X,t) pour la rotation'],[420,310;455,310])
add_line([sys,'/','F(X,t) pour la rotation'],[280,405;370,405])
add_line([sys,'/','F(X,t) pour la rotation'],[280,465;465,465])
add_line([sys,'/','F(X,t) pour la rotation'],[1015,335;1065,335])
add_line([sys,'/','F(X,t) pour la rotation'],[1100,335;1120,335])
add_line([sys,'/','F(X,t) pour la rotation'],[100,310;280,310])
add_line([sys,'/','F(X,t) pour la rotation'],[190,310;190,435;220,435])
add_line([sys,'/','F(X,t) pour la rotation'],[190,310;200,210])
add_line([sys,'/','F(X,t) pour la rotation'],[505,310;675,310])
add_line([sys,'/','F(X,t) pour la rotation'],[725,250;745,265])
add_line([sys,'/','F(X,t) pour la rotation'],[855,265;880,265])
add_line([sys,'/','F(X,t) pour la rotation'],[930,265;965,265])
add_line([sys,'/','F(X,t) pour la rotation'],[505,405;965,405])
add_line([sys,'/','F(X,t) pour la rotation'],[535,190;555,190])
add_line([sys,'/','F(X,t) pour la rotation'],[650,190;675,190])
add_line([sys,'/','F(X,t) pour la rotation'],[440,230;580,230;580,505;705,505])
add_line([sys,'/','F(X,t) pour la rotation'],[860,505;1075,505])
add_line([sys,'/','F(X,t) pour la rotation'],[440,190;480,190])
add_line([sys,'/','F(X,t) pour la rotation'],[295,465;295,555;350,555])
add_line([sys,'/','F(X,t) pour la rotation'],[400,560;430,560])
add_line([sys,'/','F(X,t) pour la rotation'],[500,560;525,560])
set_param([sys,'/','F(X,t) pour la rotation'],...
		'Mask Display','F(X , t ) \n\nPOUR LA ROTATION')


%     Finished composite block 'F(X,t) pour la rotation'.

set_param([sys,'/','F(X,t) pour la rotation'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[125,328,350,412])

add_block('built-in/Note',[sys,'/','4'])
set_param([sys,'/','4'],...
		'position',[510,520,515,525])

add_block('built-in/Note',[sys,'/','3'])
set_param([sys,'/','3'],...
		'position',[515,380,520,385])


%     Subsystem  'UnNamed2'.

new_system([sys,'/','UnNamed2'])
set_param([sys,'/','UnNamed2'],'Location',[588,239,931,333])

add_block('built-in/Note',[sys,'/',['UnNamed2/Drive du quaternion d''attitude']])
set_param([sys,'/',['UnNamed2/Drive du quaternion d''attitude']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[185,5,190,10])

add_block('built-in/Outport',[sys,'/','UnNamed2/Outport'])
set_param([sys,'/','UnNamed2/Outport'],...
		'position',[260,45,280,65])

add_block('built-in/Inport',[sys,'/','UnNamed2/Inport'])
set_param([sys,'/','UnNamed2/Inport'],...
		'position',[80,45,100,65])

add_block('built-in/MATLAB Fcn',[sys,'/','UnNamed2/deriv_qt.m'])
set_param([sys,'/','UnNamed2/deriv_qt.m'],...
		'MATLAB Fcn','deriv_qt(u)',...
		'Output Width','4',...
		'position',[150,39,220,71])
add_line([sys,'/','UnNamed2'],[225,55;255,55])
add_line([sys,'/','UnNamed2'],[105,55;145,55])
set_param([sys,'/','UnNamed2'],...
		'Mask Display','F(X , t)\n\nPOUR LE QUATERNION')


%     Finished composite block 'UnNamed2'.

set_param([sys,'/','UnNamed2'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[120,493,335,597])


%     Subsystem  'Extras1'.

new_system([sys,'/','Extras1'])
set_param([sys,'/','Extras1'],'Location',[4,42,1012,678])
set_param([sys,'/','Extras1'],...
		'Mask Display','INITIALISATION\npar double clic souris\nCAS GENERAL',...
		'Mask Dialogue','eval(''laurdat0'')')


%     Finished composite block 'Extras1'.

set_param([sys,'/','Extras1'],...
		'ForeGround',1,...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[855,510,1100,571])

add_block('built-in/Note',[sys,'/','ACQUISQITION DES ANGLES PAR REGULATION PD SUR LA ROTATION QUATERNION'])
set_param([sys,'/','ACQUISQITION DES ANGLES PAR REGULATION PD SUR LA ROTATION QUATERNION'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[595,190,600,195])

add_block('built-in/Note',[sys,'/','TECHNIQUE DE M LAURENS'])
set_param([sys,'/','TECHNIQUE DE M LAURENS'],...
		'position',[600,230,605,235])

add_block('built-in/Note',[sys,'/',['Ci-dessous cliquer',13,'pour obtenir les tracs de courbes']])
set_param([sys,'/',['Ci-dessous cliquer',13,'pour obtenir les tracs de courbes']],...
		'Font Name','Arial',...
		'Font Angle','italic',...
		'Font Size',16,...
		'position',[575,685,580,690])


%     Subsystem  'Courbes'.

new_system([sys,'/','Courbes'])
set_param([sys,'/','Courbes'],'Location',[30,105,1260,936])
open_system([sys,'/','Courbes'])


%     Subsystem  'Courbes/Extras8'.

new_system([sys,'/','Courbes/Extras8'])
set_param([sys,'/','Courbes/Extras8'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras8'],...
		'Mask Display','Voir courbe\ncouple de\ncommande (Nm)',...
		'Mask Dialogue','eval(''plot(temps,cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras8'.

set_param([sys,'/','Courbes/Extras8'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,155,750,221])


%     Subsystem  'Courbes/Extras9'.

new_system([sys,'/','Courbes/Extras9'])
set_param([sys,'/','Courbes/Extras9'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras9'],...
		'Mask Display','Voir les courbes\nmoments magntiques\nmx,my,mz',...
		'Mask Dialogue','eval(''plot(temps,mx,temps,my,temps,mz);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras9'.

set_param([sys,'/','Courbes/Extras9'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[330,75,555,136])


%     Subsystem  'Courbes/Extras19'.

new_system([sys,'/','Courbes/Extras19'])
set_param([sys,'/','Courbes/Extras19'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras19'],...
		'Mask Display','Champs magn drivs\nrel roulis\npuls constant roulis',...
		'Mask Dialogue','eval(''plot(temps,derb_reel_roulis,temps,derb_pulse_roulis);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras19'.

set_param([sys,'/','Courbes/Extras19'],...
		'BackGround',7,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[975,145,1175,211])


%     Subsystem  'Courbes/Extras17'.

new_system([sys,'/','Courbes/Extras17'])
set_param([sys,'/','Courbes/Extras17'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras17'],...
		'Mask Display','Champs magntiques\nrel roulis\npuls linaire roulis',...
		'Mask Dialogue','eval(''plot(temps,b_reel_roulis,temps,b_pulse_roulis);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras17'.

set_param([sys,'/','Courbes/Extras17'],...
		'BackGround',7,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[975,70,1175,136])


%     Subsystem  'Courbes/Extras16'.

new_system([sys,'/','Courbes/Extras16'])
set_param([sys,'/','Courbes/Extras16'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras16'],...
		'Mask Display','Voir la commande \navant le "hachoir "',...
		'Mask Dialogue','eval(''plot(temps,com_avant);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras16'.

set_param([sys,'/','Courbes/Extras16'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[760,75,960,141])


%     Subsystem  'Courbes/Extras5'.

new_system([sys,'/','Courbes/Extras5'])
set_param([sys,'/','Courbes/Extras5'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras5'],...
		'Mask Display','Voir les courbes\ndes angles (en degrs)',...
		'Mask Dialogue','eval(''plot(temps,roulis,temps,tangage,temps,lacet);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras5'.

set_param([sys,'/','Courbes/Extras5'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,75,295,141])


%     Subsystem  'Courbes/Extras7'.

new_system([sys,'/','Courbes/Extras7'])
set_param([sys,'/','Courbes/Extras7'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras7'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,p,temps,q,temps,r);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras7'.

set_param([sys,'/','Courbes/Extras7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,160,300,221])


%     Subsystem  'Courbes/Extras20'.

new_system([sys,'/','Courbes/Extras20'])
set_param([sys,'/','Courbes/Extras20'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras20'],...
		'Mask Display','Voir la commande \napres le "hachoir "',...
		'Mask Dialogue','eval(''plot(temps,com_avant-com_apres);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras20'.

set_param([sys,'/','Courbes/Extras20'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[765,240,965,306])


%     Subsystem  'Courbes/Extras18'.

new_system([sys,'/','Courbes/Extras18'])
set_param([sys,'/','Courbes/Extras18'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras18'],...
		'Mask Display','Voir la commande \napres le "hachoir "',...
		'Mask Dialogue','eval(''plot(temps,com_apres);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras18'.

set_param([sys,'/','Courbes/Extras18'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[760,155,960,221])


%     Subsystem  'Courbes/Extras6'.

new_system([sys,'/','Courbes/Extras6'])
set_param([sys,'/','Courbes/Extras6'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras6'],...
		'Mask Display','Voir la courbe\n\nenergie absolue (J)',...
		'Mask Dialogue','eval(''plot(temps,energie);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras6'.

set_param([sys,'/','Courbes/Extras6'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,75,750,131])

add_block('built-in/Note',[sys,'/','Courbes/********************************************************************************************'])
set_param([sys,'/','Courbes/********************************************************************************************'],...
		'Font Name','Arial',...
		'Font Size',20,...
		'position',[670,345,675,350])

add_block('built-in/Note',[sys,'/','Courbes/COURBES PRINCIPALES'])
set_param([sys,'/','Courbes/COURBES PRINCIPALES'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[595,15,600,20])


%     Subsystem  'Courbes/Extras14'.

new_system([sys,'/','Courbes/Extras14'])
set_param([sys,'/','Courbes/Extras14'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras14'],...
		'Mask Display','Voir courbe\ndiffrence Bc - B ',...
		'Mask Dialogue','eval(''plot(temps,Bc-B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras14'.

set_param([sys,'/','Courbes/Extras14'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,450,275,516])


%     Subsystem  'Courbes/Extras15'.

new_system([sys,'/','Courbes/Extras15'])
set_param([sys,'/','Courbes/Extras15'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras15'],...
		'Mask Display','Voir courbe\ndiffrence B''c - B'' ',...
		'Mask Dialogue','eval(''plot(temps,Bc_prime-B_prime);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras15'.

set_param([sys,'/','Courbes/Extras15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,535,275,601])


%     Subsystem  'Courbes/Extras21'.

new_system([sys,'/','Courbes/Extras21'])
set_param([sys,'/','Courbes/Extras21'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras21'],...
		'Mask Display','Voir courbe\nmoment magntique\nde Laurens M1',...
		'Mask Dialogue','eval(''plot(temps,M1);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras21'.

set_param([sys,'/','Courbes/Extras21'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[340,450,530,516])


%     Subsystem  'Courbes/Extras10'.

new_system([sys,'/','Courbes/Extras10'])
set_param([sys,'/','Courbes/Extras10'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras10'],...
		'Mask Display','Couleurs des courbes\nroulis=jaune\ntangage=bleu\nlacet=magenta')


%     Finished composite block 'Courbes/Extras10'.

set_param([sys,'/','Courbes/Extras10'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,230,755,311])


%     Subsystem  'Courbes/Extras26'.

new_system([sys,'/','Courbes/Extras26'])
set_param([sys,'/','Courbes/Extras26'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras26'],...
		'Mask Display','Voir la vitesse angulaire\nde lacet seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,r,''''c.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras26'.

set_param([sys,'/','Courbes/Extras26'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,165,540,226])


%     Subsystem  'Courbes/Extras25'.

new_system([sys,'/','Courbes/Extras25'])
set_param([sys,'/','Courbes/Extras25'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras25'],...
		'Mask Display','Voir la vitesse angulaire\nde roulis seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,p,''''y.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras25'.

set_param([sys,'/','Courbes/Extras25'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,240,540,301])


%     Subsystem  'Courbes/Extras11'.

new_system([sys,'/','Courbes/Extras11'])
set_param([sys,'/','Courbes/Extras11'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras11'],...
		'Mask Display','Voir la vitesse angulaire\nde tangage seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,q,''''m.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras11'.

set_param([sys,'/','Courbes/Extras11'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[90,235,295,296])


%     Subsystem  'Courbes/Extras32'.

new_system([sys,'/','Courbes/Extras32'])
set_param([sys,'/','Courbes/Extras32'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras32'],...
		'Mask Display','Voir courbe\nquaternion(2:4)',...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),2:4));grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras32'.

set_param([sys,'/','Courbes/Extras32'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[830,450,1005,516])


%     Subsystem  'Courbes/Extras33'.

new_system([sys,'/','Courbes/Extras33'])
set_param([sys,'/','Courbes/Extras33'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras33'],...
		'Mask Display','Voir courbe\nquaternion(1)',...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),1));grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras33'.

set_param([sys,'/','Courbes/Extras33'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[830,535,1005,601])


%     Subsystem  'Courbes/Extras29'.

new_system([sys,'/','Courbes/Extras29'])
set_param([sys,'/','Courbes/Extras29'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras29'],...
		'Mask Display','Voir courbe\nmoment magntique\n annexe M3',...
		'Mask Dialogue','eval(''plot(temps,M3);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras29'.

set_param([sys,'/','Courbes/Extras29'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[340,535,530,601])


%     Subsystem  'Courbes/Extras22'.

new_system([sys,'/','Courbes/Extras22'])
set_param([sys,'/','Courbes/Extras22'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras22'],...
		'Mask Display','Voir l''angle\nde tangage seul ()',...
		'Mask Dialogue','eval(''plot(temps,tangage,''''m.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras22'.

set_param([sys,'/','Courbes/Extras22'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,320,300,381])


%     Subsystem  'Courbes/Extras38'.

new_system([sys,'/','Courbes/Extras38'])
set_param([sys,'/','Courbes/Extras38'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras38'],...
		'Mask Display','Voir l''angle\nde roulis seul ()',...
		'Mask Dialogue','eval(''plot(temps,roulis,''''y.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras38'.

set_param([sys,'/','Courbes/Extras38'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,315,540,376])


%     Subsystem  'Courbes/Extras39'.

new_system([sys,'/','Courbes/Extras39'])
set_param([sys,'/','Courbes/Extras39'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras39'],...
		'Mask Display','Voir l''angle\nde lacet seul ()',...
		'Mask Dialogue','eval(''plot(temps,lacet,''''c.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras39'.

set_param([sys,'/','Courbes/Extras39'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,320,780,381])


%     Subsystem  'Courbes/Extras40'.

new_system([sys,'/','Courbes/Extras40'])
set_param([sys,'/','Courbes/Extras40'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras40'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (degrs/s)',...
		'Mask Dialogue','eval(''plot(temps,p*180/pi,temps,q*180/pi,temps,r*180/pi);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras40'.

set_param([sys,'/','Courbes/Extras40'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[995,240,1200,301])


%     Subsystem  'Courbes/Extras35'.

new_system([sys,'/','Courbes/Extras35'])
set_param([sys,'/','Courbes/Extras35'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras35'],...
		'Mask Display','Voir courbe\nunitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitB);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras35'.

set_param([sys,'/','Courbes/Extras35'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[840,700,995,756])


%     Subsystem  'Courbes/Extras36'.

new_system([sys,'/','Courbes/Extras36'])
set_param([sys,'/','Courbes/Extras36'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras36'],...
		'Mask Display','Voir courbe\nunitaire(Q)',...
		'Mask Dialogue','eval(''plot(temps,unitQ);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras36'.

set_param([sys,'/','Courbes/Extras36'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[840,630,995,691])


%     Subsystem  'Courbes/Extras12'.

new_system([sys,'/','Courbes/Extras12'])
set_param([sys,'/','Courbes/Extras12'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras12'],...
		'Mask Display','Voir courbe\ncouple commande\n roue (Nm)',...
		'Mask Dialogue','eval(''plot(temps,cpl_roue);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras12'.

set_param([sys,'/','Courbes/Extras12'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1025,450,1200,516])


%     Subsystem  'Courbes/Extras27'.

new_system([sys,'/','Courbes/Extras27'])
set_param([sys,'/','Courbes/Extras27'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras27'],...
		'Mask Display','Voir courbe\ndiffrence PB - B ',...
		'Mask Dialogue','eval(''plot(temps,PB_moins_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras27'.

set_param([sys,'/','Courbes/Extras27'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,610,275,676])


%     Subsystem  'Courbes/Extras31'.

new_system([sys,'/','Courbes/Extras31'])
set_param([sys,'/','Courbes/Extras31'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras31'],...
		'Mask Display','Voir courbe\nsomme PB+B ',...
		'Mask Dialogue','eval(''plot(temps,PB_plus_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras31'.

set_param([sys,'/','Courbes/Extras31'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,690,275,756])

add_block('built-in/Note',[sys,'/',['Courbes/Au cas o le satellite serait  l''envers']])
set_param([sys,'/',['Courbes/Au cas o le satellite serait  l''envers']],...
		'BackGround',0,...
		'Font Name','Arial',...
		'Font Size',20,...
		'position',[310,760,315,765])


%     Subsystem  'Courbes/Extras28'.

new_system([sys,'/','Courbes/Extras28'])
set_param([sys,'/','Courbes/Extras28'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras28'],...
		'Mask Display','Voir courbe\nnorme de la\ndiffrence PB - B ',...
		'Mask Dialogue','eval(''plot(temps,norm_PB_moins_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras28'.

set_param([sys,'/','Courbes/Extras28'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[350,610,525,676])


%     Subsystem  'Courbes/Extras30'.

new_system([sys,'/','Courbes/Extras30'])
set_param([sys,'/','Courbes/Extras30'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras30'],...
		'Mask Display','Voir courbe\nnorme de la\nsomme PB + B ',...
		'Mask Dialogue','eval(''plot(temps,norm_PB_plus_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras30'.

set_param([sys,'/','Courbes/Extras30'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[350,695,525,761])


%     Subsystem  'Courbes/Extras13'.

new_system([sys,'/','Courbes/Extras13'])
set_param([sys,'/','Courbes/Extras13'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras13'],...
		'Mask Display','Voir courbe\nvitesse de la\n roue (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,vit_roue);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras13'.

set_param([sys,'/','Courbes/Extras13'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,535,1200,596])


%     Subsystem  'Courbes/Extras41'.

new_system([sys,'/','Courbes/Extras41'])
set_param([sys,'/','Courbes/Extras41'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras41'],...
		'Mask Display','Voir courbe du\ncouplage roue-satellite\nsur le roulis',...
		'Mask Dialogue','eval(''plot(temps,couplage_roue_sat(1:length(temps),1),''''y'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras41'.

set_param([sys,'/','Courbes/Extras41'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1030,630,1220,686])

add_block('built-in/Note',[sys,'/','Courbes/COURBES ANNEXES ET MISE AU POINT'])
set_param([sys,'/','Courbes/COURBES ANNEXES ET MISE AU POINT'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[595,385,600,390])


%     Subsystem  'Courbes/Extras24'.

new_system([sys,'/','Courbes/Extras24'])
set_param([sys,'/','Courbes/Extras24'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras24'],...
		'Mask Display','Voir courbe\ndcalage R / R0\nen degrs',...
		'Mask Dialogue','eval(''plot(temps,angle_rot_R0_to_R);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras24'.

set_param([sys,'/','Courbes/Extras24'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,520,760,586])


%     Subsystem  'Courbes/Extras23'.

new_system([sys,'/','Courbes/Extras23'])
set_param([sys,'/','Courbes/Extras23'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras23'],...
		'Mask Display','Voir courbe\ncouple total 1',...
		'Mask Dialogue','eval(''plot(temps,cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras23'.

set_param([sys,'/','Courbes/Extras23'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,435,760,501])


%     Subsystem  'Courbes/Extras37'.

new_system([sys,'/','Courbes/Extras37'])
set_param([sys,'/','Courbes/Extras37'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras37'],...
		'Mask Display','Voir courbe\nunitaire(Q)+ unitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitQ+unitB);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras37'.

set_param([sys,'/','Courbes/Extras37'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,630,775,691])


%     Subsystem  'Courbes/Extras34'.

new_system([sys,'/','Courbes/Extras34'])
set_param([sys,'/','Courbes/Extras34'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras34'],...
		'Mask Display','Voir courbe\nunitaire(Q)- unitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitQ-unitB);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras34'.

set_param([sys,'/','Courbes/Extras34'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[565,700,775,761])


%     Subsystem  'Courbes/Extras42'.

new_system([sys,'/','Courbes/Extras42'])
set_param([sys,'/','Courbes/Extras42'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras42'],...
		'Mask Display','Voir courbe du\ncouplage roue-satellite\nsur le lacet',...
		'Mask Dialogue','eval(''plot(temps,couplage_roue_sat(1:length(temps),3),''''c'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras42'.

set_param([sys,'/','Courbes/Extras42'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,700,1225,756])
set_param([sys,'/','Courbes'],...
		'Mask Display','SORTIES GRAPHIQUES')


%     Finished composite block 'Courbes'.

set_param([sys,'/','Courbes'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[430,745,705,821])


%     Subsystem  'Subsystem'.

new_system([sys,'/','Subsystem'])
set_param([sys,'/','Subsystem'],'Location',[4,112,1260,1004])

add_block('built-in/Note',[sys,'/',['Subsystem/SORTIES ET TRAITEMENT DE LA SORTIE',13,'POUR VERIFICATIONS DE FONCTIONNALITE']])
set_param([sys,'/',['Subsystem/SORTIES ET TRAITEMENT DE LA SORTIE',13,'POUR VERIFICATIONS DE FONCTIONNALITE']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[620,40,625,45])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux'])
set_param([sys,'/','Subsystem/Demux'],...
		'outputs','[3 4]',...
		'position',[470,339,520,506])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn1'])
set_param([sys,'/','Subsystem/MATLAB Fcn1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','angles(u)',...
		'Output Width','3',...
		'position',[700,448,760,482])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace14'])
set_param([sys,'/','Subsystem/To Workspace14'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','lacet',...
		'buffer','points_trace',...
		'position',[965,490,1035,510])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace12'])
set_param([sys,'/','Subsystem/To Workspace12'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','roulis',...
		'buffer','points_trace',...
		'position',[965,420,1035,440])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux2'])
set_param([sys,'/','Subsystem/Demux2'],...
		'outputs','3',...
		'position',[860,413,905,517])

add_block('built-in/Note',[sys,'/','Subsystem/Energie'])
set_param([sys,'/','Subsystem/Energie'],...
		'position',[885,390,890,395])

add_block('built-in/Note',[sys,'/','Subsystem/Drive de Q'])
set_param([sys,'/','Subsystem/Drive de Q'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[635,665,640,670])

add_block('built-in/Note',[sys,'/',['Subsystem/Rotation atellite par',13,'rapport aux axes orbitaux']])
set_param([sys,'/',['Subsystem/Rotation atellite par',13,'rapport aux axes orbitaux']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[905,670,910,675])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn4'])
set_param([sys,'/','Subsystem/MATLAB Fcn4'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','omegaorb(u)',...
		'Output Width','3',...
		'position',[815,621,895,659])

add_block('built-in/Note',[sys,'/','Subsystem/Quaternion Q'])
set_param([sys,'/','Subsystem/Quaternion Q'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[600,435,605,440])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace5'])
set_param([sys,'/','Subsystem/To Workspace5'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','accel_angulaire',...
		'buffer','points_trace',...
		'position',[565,562,670,598])

add_block('built-in/Mux',[sys,'/','Subsystem/Mux'])
set_param([sys,'/','Subsystem/Mux'],...
		'inputs','[4 4]',...
		'position',[735,591,770,689])

add_block('built-in/Note',[sys,'/',['Subsystem/Q''']])
set_param([sys,'/',['Subsystem/Q''']],...
		'position',[710,670,715,675])

add_block('built-in/Note',[sys,'/','Subsystem/Q'])
set_param([sys,'/','Subsystem/Q'],...
		'position',[705,590,710,595])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace11'])
set_param([sys,'/','Subsystem/To Workspace11'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','omega_quat',...
		'buffer','points_trace',...
		'position',[915,624,1000,656])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn3'])
set_param([sys,'/','Subsystem/MATLAB Fcn3'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','ENERGIE(u)',...
		'Output Width','1',...
		'position',[745,361,825,399])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux1'])
set_param([sys,'/','Subsystem/Demux1'],...
		'outputs','[3 4]',...
		'position',[470,539,520,706])

add_block('built-in/Note',[sys,'/',['Subsystem/Entre de',13,'p q r Q']])
set_param([sys,'/',['Subsystem/Entre de',13,'p q r Q']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[420,365,425,370])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace13'])
set_param([sys,'/','Subsystem/To Workspace13'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','tangage',...
		'buffer','points_trace',...
		'position',[965,455,1035,475])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace8'])
set_param([sys,'/','Subsystem/To Workspace8'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','temps',...
		'buffer','points_trace',...
		'position',[965,525,1035,545])

add_block('built-in/Clock',[sys,'/','Subsystem/Clock'])
set_param([sys,'/','Subsystem/Clock'],...
		'position',[905,525,925,545])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux3'])
set_param([sys,'/','Subsystem/Demux3'],...
		'outputs','3',...
		'position',[610,158,655,262])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace15'])
set_param([sys,'/','Subsystem/To Workspace15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','p',...
		'buffer','points_trace',...
		'position',[715,165,785,185])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace16'])
set_param([sys,'/','Subsystem/To Workspace16'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','q',...
		'buffer','points_trace',...
		'position',[715,200,785,220])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace17'])
set_param([sys,'/','Subsystem/To Workspace17'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','r',...
		'buffer','points_trace',...
		'position',[715,235,785,255])

add_block('built-in/Note',[sys,'/',['Subsystem/Vitesses angulaires absolues',13,'autour des axes x y z']])
set_param([sys,'/',['Subsystem/Vitesses angulaires absolues',13,'autour des axes x y z']],...
		'position',[895,185,900,190])

add_block('built-in/Inport',[sys,'/','Subsystem/in_2'])
set_param([sys,'/','Subsystem/in_2'],...
		'Port','2',...
		'position',[390,615,410,635])


%     Subsystem  ['Subsystem/Vitesses angulaires',13,'rd//s'].

new_system([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],'Location',[0,59,274,252])

add_block('built-in/Inport',[sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/x']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/x']],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfuny',...
		'parameters','ax, color,dt',...
		'position',[130,55,180,75])
add_line([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],[90,65;125,65])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter plotting ranges and line type.|Time range:|y-min:|y-max:|Line type (rgbw-:*). Seperate each plot by ''/'':')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Translate','color = @4; ax = [0, @1, @2, @3]; dt = -1;')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Scope block. Look at the m-file sfuny.m to see how it works. This block can take scalar or vector input signal.')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Entries','4000\/-0.2\/0.2\/''y-/g--/c-./w:/m*/ro/b+''\/')


%     Finished composite block ['Subsystem/Vitesses angulaires',13,'rd//s'].

set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'orientation',2,...
		'BackGround',2,...
		'position',[365,192,475,228])


%     Subsystem  'Subsystem/Angles en degrs'.

new_system([sys,'/','Subsystem/Angles en degrs'])
set_param([sys,'/','Subsystem/Angles en degrs'],'Location',[0,59,274,252])

add_block('built-in/Inport',[sys,'/','Subsystem/Angles en degrs/x'])
set_param([sys,'/','Subsystem/Angles en degrs/x'],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['Subsystem/Angles en degrs/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Subsystem/Angles en degrs/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfuny',...
		'parameters','ax, color,dt',...
		'position',[130,55,180,75])
add_line([sys,'/','Subsystem/Angles en degrs'],[90,65;125,65])
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter plotting ranges and line type.|Time range:|y-min:|y-max:|Line type (rgbw-:*). Seperate each plot by ''/'':')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Translate','color = @4; ax = [0, @1, @2, @3]; dt = -1;')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Scope block. Look at the m-file sfuny.m to see how it works. This block can take scalar or vector input signal.')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Entries','5000\/-200\/200\/''y-/g--/c-./w:/m*/ro/b+''\/')


%     Finished composite block 'Subsystem/Angles en degrs'.

set_param([sys,'/','Subsystem/Angles en degrs'],...
		'orientation',2,...
		'BackGround',2,...
		'position',[705,522,815,558])

add_block('built-in/Inport',[sys,'/','Subsystem/in_1'])
set_param([sys,'/','Subsystem/in_1'],...
		'position',[290,415,310,435])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace7'])
set_param([sys,'/','Subsystem/To Workspace7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','energie',...
		'buffer','points_trace',...
		'position',[945,370,1015,390])

add_block('built-in/Gain',[sys,'/',['Subsystem/Conversion',13,'rd // degrs']])
set_param([sys,'/',['Subsystem/Conversion',13,'rd // degrs']],...
		'Gain','180/pi',...
		'position',[795,452,820,478])
add_line([sys,'/','Subsystem'],[525,380;740,380])
add_line([sys,'/','Subsystem'],[830,380;940,380])
add_line([sys,'/','Subsystem'],[315,425;465,425])
add_line([sys,'/','Subsystem'],[525,465;695,465])
add_line([sys,'/','Subsystem'],[930,535;960,535])
add_line([sys,'/','Subsystem'],[415,625;465,625])
add_line([sys,'/','Subsystem'],[525,580;560,580])
add_line([sys,'/','Subsystem'],[525,665;730,665])
add_line([sys,'/','Subsystem'],[675,465;675,615;730,615])
add_line([sys,'/','Subsystem'],[775,640;810,640])
add_line([sys,'/','Subsystem'],[900,640;910,640])
add_line([sys,'/','Subsystem'],[765,465;790,465])
add_line([sys,'/','Subsystem'],[825,465;855,465])
add_line([sys,'/','Subsystem'],[910,430;960,430])
add_line([sys,'/','Subsystem'],[910,465;960,465])
add_line([sys,'/','Subsystem'],[910,500;960,500])
add_line([sys,'/','Subsystem'],[550,380;550,210;605,210])
add_line([sys,'/','Subsystem'],[660,175;710,175])
add_line([sys,'/','Subsystem'],[660,210;710,210])
add_line([sys,'/','Subsystem'],[660,245;710,245])
add_line([sys,'/','Subsystem'],[550,210;480,210])
add_line([sys,'/','Subsystem'],[840,465;840,540;820,540])
set_param([sys,'/','Subsystem'],...
		'Mask Display','SORTIE DES ANGLES\n\nEN DEGRES & ENERGIE')


%     Finished composite block 'Subsystem'.

set_param([sys,'/','Subsystem'],...
		'hide name',0,...
		'position',[845,309,1085,391])


%     Subsystem  'Subsystem3'.

new_system([sys,'/','Subsystem3'])
set_param([sys,'/','Subsystem3'],'Location',[4,44,1244,1004])
open_system([sys,'/','Subsystem3'])

add_block('built-in/Note',[sys,'/',['Subsystem3/Suppression du test',13,'sur les vitesses']])
set_param([sys,'/',['Subsystem3/Suppression du test',13,'sur les vitesses']],...
		'Font Name','Arial',...
		'position',[625,240,630,245])


%     Subsystem  'Subsystem3/Subsystem2'.

new_system([sys,'/','Subsystem3/Subsystem2'])
set_param([sys,'/','Subsystem3/Subsystem2'],'Location',[559,482,1161,775])

add_block('built-in/Note',[sys,'/','Subsystem3/Subsystem2/Pour mettre le stop hors course, fixer le gain  0'])
set_param([sys,'/','Subsystem3/Subsystem2/Pour mettre le stop hors course, fixer le gain  0'],...
		'ForeGround',7,...
		'Font Name','Arial',...
		'Font Size',18,...
		'position',[345,140,350,145])

add_block('built-in/Gain',[sys,'/','Subsystem3/Subsystem2/Gain'])
set_param([sys,'/','Subsystem3/Subsystem2/Gain'],...
		'BackGround',7,...
		'position',[185,63,265,97])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/Subsystem2/To Workspace'])
set_param([sys,'/','Subsystem3/Subsystem2/To Workspace'],...
		'mat-name','stop',...
		'buffer','points_trace',...
		'position',[370,27,420,43])

add_block('built-in/Stop Simulation',[sys,'/','Subsystem3/Subsystem2/Stop Simulation'])
set_param([sys,'/','Subsystem3/Subsystem2/Stop Simulation'],...
		'position',[350,63,390,97])

add_block('built-in/Inport',[sys,'/','Subsystem3/Subsystem2/in_1'])
set_param([sys,'/','Subsystem3/Subsystem2/in_1'],...
		'position',[15,70,35,90])
add_line([sys,'/','Subsystem3/Subsystem2'],[270,80;345,80])
add_line([sys,'/','Subsystem3/Subsystem2'],[300,80;300,35;365,35])
add_line([sys,'/','Subsystem3/Subsystem2'],[40,80;180,80])
set_param([sys,'/','Subsystem3/Subsystem2'],...
		'Mask Display','STOP')


%     Finished composite block 'Subsystem3/Subsystem2'.

set_param([sys,'/','Subsystem3/Subsystem2'],...
		'ForeGround',5,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[995,518,1080,572])

add_block('built-in/Note',[sys,'/',['Subsystem3/ARRT SUR',13,'LES ANGLES']])
set_param([sys,'/',['Subsystem3/ARRT SUR',13,'LES ANGLES']],...
		'position',[785,685,790,690])

add_block('built-in/Note',[sys,'/',['Subsystem3/ARRT SUR LE',13,'QUATERNION']])
set_param([sys,'/',['Subsystem3/ARRT SUR LE',13,'QUATERNION']],...
		'position',[795,500,800,505])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant2'])
set_param([sys,'/','Subsystem3/Constant2'],...
		'ForeGround',1,...
		'BackGround',3,...
		'Value','2',...
		'position',[440,874,515,896])

add_block('built-in/Gain',[sys,'/','Subsystem3/Gain3'])
set_param([sys,'/','Subsystem3/Gain3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'Gain','0',...
		'position',[600,870,645,900])

add_block('built-in/Note',[sys,'/',['Subsystem3/Suppression du test',13,'sur le quaternion']])
set_param([sys,'/',['Subsystem3/Suppression du test',13,'sur le quaternion']],...
		'position',[570,605,575,610])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant4'])
set_param([sys,'/','Subsystem3/Constant4'],...
		'ForeGround',1,...
		'BackGround',2,...
		'Value','2',...
		'position',[465,649,540,671])

add_block('built-in/Gain',[sys,'/','Subsystem3/Gain2'])
set_param([sys,'/','Subsystem3/Gain2'],...
		'ForeGround',1,...
		'BackGround',0,...
		'Gain','0',...
		'position',[580,645,625,675])

add_block('built-in/Relational Operator',[sys,'/',['Subsystem3/Relational',13,'Operator1']])
set_param([sys,'/',['Subsystem3/Relational',13,'Operator1']],...
		'ForeGround',1,...
		'BackGround',3,...
		'Operator','<=',...
		'position',[615,723,645,792])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant1'])
set_param([sys,'/','Subsystem3/Constant1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Value','angles_cardan*pi/180',...
		'position',[410,764,550,786])

add_block('built-in/Sum',[sys,'/','Subsystem3/Sum1'])
set_param([sys,'/','Subsystem3/Sum1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'inputs','+-',...
		'position',[705,739,740,821])

add_block('built-in/Relational Operator',[sys,'/',['Subsystem3/Relational',13,'Operator']])
set_param([sys,'/',['Subsystem3/Relational',13,'Operator']],...
		'ForeGround',1,...
		'BackGround',2,...
		'Operator','<=',...
		'position',[620,498,645,547])

add_block('built-in/MATLAB Fcn',[sys,'/',['Subsystem3/Dcalage',13,'angulaire',13,'R//R0']])
set_param([sys,'/',['Subsystem3/Dcalage',13,'angulaire',13,'R//R0']],...
		'ForeGround',1,...
		'BackGround',2,...
		'MATLAB Fcn','decalRR0(u)',...
		'Output Width','1',...
		'position',[395,438,455,472])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant'])
set_param([sys,'/','Subsystem3/Constant'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Value','angle_quater_maxi',...
		'position',[395,524,535,546])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace6'])
set_param([sys,'/','Subsystem3/To Workspace6'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'mat-name','angle_rot_R0_to_R',...
		'buffer','points_trace',...
		'position',[505,441,625,469])

add_block('built-in/Sum',[sys,'/','Subsystem3/Sum2'])
set_param([sys,'/','Subsystem3/Sum2'],...
		'ForeGround',1,...
		'BackGround',2,...
		'inputs','+-',...
		'position',[700,504,735,586])

add_block('built-in/Sum',[sys,'/','Subsystem3/Sum3'])
set_param([sys,'/','Subsystem3/Sum3'],...
		'ForeGround',1,...
		'BackGround',4,...
		'inputs','-+',...
		'position',[705,354,740,436])

add_block('built-in/Gain',[sys,'/','Subsystem3/Gain4'])
set_param([sys,'/','Subsystem3/Gain4'],...
		'ForeGround',1,...
		'BackGround',0,...
		'Gain','0',...
		'position',[605,285,650,315])

add_block('built-in/Product',[sys,'/','Subsystem3/Product'])
set_param([sys,'/','Subsystem3/Product'],...
		'inputs','3',...
		'position',[860,365,885,725])

add_block('built-in/Note',[sys,'/',['Subsystem3/ARRT SUR',13,'LES VITESSES']])
set_param([sys,'/',['Subsystem3/ARRT SUR',13,'LES VITESSES']],...
		'position',[795,350,800,355])

add_block('built-in/Constant',[sys,'/','Subsystem3/Constant3'])
set_param([sys,'/','Subsystem3/Constant3'],...
		'ForeGround',1,...
		'BackGround',4,...
		'Value','2',...
		'position',[460,289,535,311])

add_block('built-in/Demux',[sys,'/','Subsystem3/Demux'])
set_param([sys,'/','Subsystem3/Demux'],...
		'outputs','[3 4]',...
		'position',[220,440,270,560])

add_block('built-in/Inport',[sys,'/','Subsystem3/in_1'])
set_param([sys,'/','Subsystem3/in_1'],...
		'position',[105,490,125,510])

add_block('built-in/Note',[sys,'/','Subsystem3/Entre pqrQ'])
set_param([sys,'/','Subsystem3/Entre pqrQ'],...
		'position',[130,535,135,540])

add_block('built-in/Note',[sys,'/','Subsystem3/Q'])
set_param([sys,'/','Subsystem3/Q'],...
		'position',[325,535,330,540])

add_block('built-in/Note',[sys,'/','Subsystem3/pqr'])
set_param([sys,'/','Subsystem3/pqr'],...
		'position',[295,450,300,455])

add_block('built-in/MATLAB Fcn',[sys,'/',['Subsystem3/fonction ',13,'d''arrt']])
set_param([sys,'/',['Subsystem3/fonction ',13,'d''arrt']],...
		'ForeGround',1,...
		'BackGround',4,...
		'MATLAB Fcn','stopderv(u)',...
		'Output Width','1',...
		'position',[360,348,420,382])

add_block('built-in/Note',[sys,'/',['Subsystem3/Suppression du test ',13,'sur les angles']])
set_param([sys,'/',['Subsystem3/Suppression du test ',13,'sur les angles']],...
		'position',[605,830,610,835])

add_block('built-in/Note',[sys,'/','Subsystem3/STOP SIMULATION'])
set_param([sys,'/','Subsystem3/STOP SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',24,...
		'position',[595,5,600,10])

add_block('built-in/Note',[sys,'/',['Subsystem3/NB : La mise  1 d''un des gains 2, 3, 4 mettra hors jeu le contrle correspondant ',13,'La mise  1 de tous les gains empche la simulation de dmarrer']])
set_param([sys,'/',['Subsystem3/NB : La mise  1 d''un des gains 2, 3, 4 mettra hors jeu le contrle correspondant ',13,'La mise  1 de tous les gains empche la simulation de dmarrer']],...
		'ForeGround',3,...
		'Font Angle','italic')
set_param([sys,'/',['Subsystem3/NB : La mise  1 d''un des gains 2, 3, 4 mettra hors jeu le contrle correspondant ',13,'La mise  1 de tous les gains empche la simulation de dmarrer']],...
		'Font Size',18,...
		'position',[600,70,605,75])

add_block('built-in/Note',[sys,'/','Subsystem3/En effet, quand le gain vaut 1, la somme  ( sum1 ou sum2 ou sum3 ) ne peut jamais valoir 1, au maximum ce sera 0'])
set_param([sys,'/','Subsystem3/En effet, quand le gain vaut 1, la somme  ( sum1 ou sum2 ou sum3 ) ne peut jamais valoir 1, au maximum ce sera 0'],...
		'position',[565,130,570,135])

add_block('built-in/Fcn',[sys,'/','Subsystem3/Fcn'])
set_param([sys,'/','Subsystem3/Fcn'],...
		'ForeGround',1,...
		'BackGround',3,...
		'Expr','(u(1)^2+u(2)^2+u(3)^2)^0.5',...
		'position',[965,674,1040,696])

add_block('built-in/To Workspace',[sys,'/','Subsystem3/To Workspace7'])
set_param([sys,'/','Subsystem3/To Workspace7'],...
		'hide name',0,...
		'mat-name','rien',...
		'buffer','points_trace',...
		'position',[1060,670,1115,700])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem3/MATLAB Fcn2'])
set_param([sys,'/','Subsystem3/MATLAB Fcn2'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','angles(u)',...
		'Output Width','3',...
		'position',[330,721,420,759])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem3/MATLAB Fcn'])
set_param([sys,'/','Subsystem3/MATLAB Fcn'],...
		'hide name',0,...
		'MATLAB Fcn','anglmax(u)',...
		'Output Width','1',...
		'position',[500,723,560,757])
add_line([sys,'/','Subsystem3'],[545,660;575,660])
add_line([sys,'/','Subsystem3'],[540,300;600,300])
add_line([sys,'/','Subsystem3'],[520,885;595,885])
add_line([sys,'/','Subsystem3'],[745,780;830,780;830,665;855,665])
add_line([sys,'/','Subsystem3'],[745,395;830,395;830,425;855,425])
add_line([sys,'/','Subsystem3'],[740,545;855,545])
add_line([sys,'/','Subsystem3'],[655,300;665,300;665,375;700,375])
add_line([sys,'/','Subsystem3'],[425,365;580,365;580,415;700,415])
add_line([sys,'/','Subsystem3'],[630,660;670,660;670,565;695,565])
add_line([sys,'/','Subsystem3'],[650,525;695,525])
add_line([sys,'/','Subsystem3'],[650,885;680,885;680,800;700,800])
add_line([sys,'/','Subsystem3'],[650,760;700,760])
add_line([sys,'/','Subsystem3'],[460,455;500,455])
add_line([sys,'/','Subsystem3'],[475,455;475,510;615,510])
add_line([sys,'/','Subsystem3'],[275,530;365,530;365,455;390,455])
add_line([sys,'/','Subsystem3'],[890,545;990,545])
add_line([sys,'/','Subsystem3'],[555,775;610,775])
add_line([sys,'/','Subsystem3'],[540,535;615,535])
add_line([sys,'/','Subsystem3'],[130,500;215,500])
add_line([sys,'/','Subsystem3'],[290,530;290,740;325,740])
add_line([sys,'/','Subsystem3'],[175,500;175,365;355,365])
add_line([sys,'/','Subsystem3'],[1045,685;1055,685])
add_line([sys,'/','Subsystem3'],[425,740;495,740])
add_line([sys,'/','Subsystem3'],[460,740;460,685;960,685])
add_line([sys,'/','Subsystem3'],[565,740;610,740])
set_param([sys,'/','Subsystem3'],...
		'Mask Display','STOP MULTIPLE\n\nCHOIX POSSIBLES')


%     Finished composite block 'Subsystem3'.

set_param([sys,'/','Subsystem3'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'position',[850,426,1090,494])

add_block('built-in/Note',[sys,'/',['Stabilisation par la drive du champ magntique terrestre',13,'et un rappel proportionnel sur un champ de consigne, adapte au cas puls']])
set_param([sys,'/',['Stabilisation par la drive du champ magntique terrestre',13,'et un rappel proportionnel sur un champ de consigne, adapte au cas puls']],...
		'ForeGround',5,...
		'Font Weight','bold',...
		'Font Size',22)
set_param([sys,'/',['Stabilisation par la drive du champ magntique terrestre',13,'et un rappel proportionnel sur un champ de consigne, adapte au cas puls']],...
		'position',[623,100,628,105])


%     Subsystem  'Subsystem7'.

new_system([sys,'/','Subsystem7'])
set_param([sys,'/','Subsystem7'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem7/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem7/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem7/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem7/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem7/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem7/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem7/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem7/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem7/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem7/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem7/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem7/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem7/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem7/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem7/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem7/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem7/d) Lancer la simulation'])
set_param([sys,'/','Subsystem7/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem7/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem7/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem7/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem7/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem7/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem7/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem7/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem7/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem7/b) '])
set_param([sys,'/','Subsystem7/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem7/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem7/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem7/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem7/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem7/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem7/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem7'],...
		'Mask Display','Gain_essai =15',...
		'Mask Dialogue','eval(''gain_pour_essai=15'')')


%     Finished composite block 'Subsystem7'.

set_param([sys,'/','Subsystem7'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1015,695,1155,741])


%     Subsystem  'Subsystem5'.

new_system([sys,'/','Subsystem5'])
set_param([sys,'/','Subsystem5'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem5/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem5/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem5/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem5/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem5/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem5/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem5/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem5/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem5/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem5/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem5/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem5/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem5/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem5/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem5/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem5/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem5/d) Lancer la simulation'])
set_param([sys,'/','Subsystem5/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem5/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem5/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem5/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem5/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem5/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem5/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem5/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem5/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem5/b) '])
set_param([sys,'/','Subsystem5/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem5/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem5/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem5/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem5/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem5/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem5/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem5'],...
		'Mask Display','Gain_essai =5',...
		'Mask Dialogue','eval(''gain_pour_essai=5'')')


%     Finished composite block 'Subsystem5'.

set_param([sys,'/','Subsystem5'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1010,640,1150,686])


%     Subsystem  'Subsystem4'.

new_system([sys,'/','Subsystem4'])
set_param([sys,'/','Subsystem4'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem4/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem4/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem4/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem4/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem4/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem4/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem4/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem4/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem4/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem4/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem4/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem4/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem4/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem4/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem4/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem4/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem4/d) Lancer la simulation'])
set_param([sys,'/','Subsystem4/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem4/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem4/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem4/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem4/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem4/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem4/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem4/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem4/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem4/b) '])
set_param([sys,'/','Subsystem4/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem4/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem4/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem4/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem4/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem4/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem4/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem4'],...
		'Mask Display','Gain_essai =0',...
		'Mask Dialogue','eval(''gain_pour_essai=0'')')


%     Finished composite block 'Subsystem4'.

set_param([sys,'/','Subsystem4'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[845,640,985,686])


%     Subsystem  'Subsystem6'.

new_system([sys,'/','Subsystem6'])
set_param([sys,'/','Subsystem6'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem6/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem6/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem6/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem6/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem6/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem6/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem6/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem6/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem6/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem6/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem6/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem6/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem6/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem6/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem6/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem6/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem6/d) Lancer la simulation'])
set_param([sys,'/','Subsystem6/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem6/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem6/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem6/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem6/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem6/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem6/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem6/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem6/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem6/b) '])
set_param([sys,'/','Subsystem6/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem6/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem6/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem6/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem6/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem6/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem6/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem6'],...
		'Mask Display','Gain_essai =10',...
		'Mask Dialogue','eval(''gain_pour_essai=10'')')


%     Finished composite block 'Subsystem6'.

set_param([sys,'/','Subsystem6'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[845,695,985,741])


%     Subsystem  'Subsystem2'.

new_system([sys,'/','Subsystem2'])
set_param([sys,'/','Subsystem2'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem2/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem2/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem2/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem2/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem2/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem2/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem2/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem2/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem2/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem2/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem2/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem2/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem2/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem2/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem2/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem2/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem2/d) Lancer la simulation'])
set_param([sys,'/','Subsystem2/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem2/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem2/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem2/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem2/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem2/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem2/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem2/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem2/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem2/b) '])
set_param([sys,'/','Subsystem2/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem2/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem2/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem2/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem2/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem2/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem2/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem2'],...
		'Mask Display','RECUPERER LES RESULTATS\n\nDANS L''ESPACE MATLAB',...
		'Mask Dialogue','eval(''display'')')


%     Finished composite block 'Subsystem2'.

set_param([sys,'/','Subsystem2'],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[855,780,1155,841])


%     Subsystem  'Subsystem1'.

new_system([sys,'/','Subsystem1'])
set_param([sys,'/','Subsystem1'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem1/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem1/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem1/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem1/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem1/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem1/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem1/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem1/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem1/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem1/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem1/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem1/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem1/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem1/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem1/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem1/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem1/d) Lancer la simulation'])
set_param([sys,'/','Subsystem1/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem1/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem1/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem1/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem1/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem1/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem1/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem1/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem1/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem1/b) '])
set_param([sys,'/','Subsystem1/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem1/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem1/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem1/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem1/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem1/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem1/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem1/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem1/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem1/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem1'],...
		'Mask Display','LIRE LES EXPLICATIONS')


%     Finished composite block 'Subsystem1'.

set_param([sys,'/','Subsystem1'],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[45,760,395,796])

add_block('built-in/Note',[sys,'/',['Vecteur d''tat de 7 composantes']])
set_param([sys,'/',['Vecteur d''tat de 7 composantes']],...
		'position',[755,650,760,655])
add_line(sys,[725,460;745,460;745,630;35,630;35,545;115,545])
add_line(sys,[550,460;615,460])
add_line(sys,[340,545;495,545])
add_line(sys,[745,460;745,330;840,330])
add_line(sys,[595,460;595,370;840,370])
add_line(sys,[35,545;35,370;120,370])
add_line(sys,[355,350;415,350])
add_line(sys,[335,460;400,460;400,385;415,385])
add_line(sys,[450,370;495,370])
add_line(sys,[35,470;110,470])
add_line(sys,[355,390;365,390;365,420;95,420;95,445;110,445])
add_line(sys,[745,460;845,460])

drawnow

% Return any arguments.
if (nargin | nargout)
	% Must use feval here to access system in memory
	if (nargin > 3)
		if (flag == 0)
			eval(['[ret,x0,str,ts,xts]=',sys,'(t,x,u,flag);'])
		else
			eval(['ret =', sys,'(t,x,u,flag);'])
		end
	else
		[ret,x0,str,ts,xts] = feval(sys);
	end
else
	drawnow % Flash up the model and execute load callback
end
