function [ret,x0,str,ts,xts]=survfix4(t,x,u,flag);
%SURVFIX4	is the M-file description of the SIMULINK system named SURVFIX4.
%	The block-diagram can be displayed by typing: SURVFIX4.
%
%	SYS=SURVFIX4(T,X,U,FLAG) returns depending on FLAG certain
%	system values given time point, T, current state vector, X,
%	and input vector, U.
%	FLAG is used to indicate the type of output to be returned in SYS.
%
%	Setting FLAG=1 causes SURVFIX4 to return state derivatives, FLAG=2
%	discrete states, FLAG=3 system outputs and FLAG=4 next sample
%	time. For more information and other options see SFUNC.
%
%	Calling SURVFIX4 with a FLAG of zero:
%	[SIZES]=SURVFIX4([],[],[],0),  returns a vector, SIZES, which
%	contains the sizes of the state vector and other parameters.
%		SIZES(1) number of states
%		SIZES(2) number of discrete states
%		SIZES(3) number of outputs
%		SIZES(4) number of inputs
%		SIZES(5) number of roots (currently unsupported)
%		SIZES(6) direct feedthrough flag
%		SIZES(7) number of sample times
%
%	For the definition of other parameters in SIZES, see SFUNC.
%	See also, TRIM, LINMOD, LINSIM, EULER, RK23, RK45, ADAMS, GEAR.

% Note: This M-file is only used for saving graphical information;
%       after the model is loaded into memory an internal model
%       representation is used.

% the system will take on the name of this mfile:
sys = mfilename;
new_system(sys)
simver(1.3)
if (0 == (nargin + nargout))
     set_param(sys,'Location',[6,44,1260,958])
     open_system(sys)
end;
set_param(sys,'algorithm',     'RK-45')
set_param(sys,'Start time',    'instant_injection')
set_param(sys,'Stop time',     'temps_fin_simu')
set_param(sys,'Min step size', '0.1')
set_param(sys,'Max step size', '1')
set_param(sys,'Relative error','1e-3')
set_param(sys,'Return vars',   '')


%     Subsystem  'Subsystem2'.

new_system([sys,'/','Subsystem2'])
set_param([sys,'/','Subsystem2'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem2/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem2/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem2/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem2/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem2/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem2/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem2/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem2/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem2/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem2/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem2/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem2/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem2/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem2/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem2/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem2/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem2/d) Lancer la simulation'])
set_param([sys,'/','Subsystem2/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem2/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem2/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem2/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem2/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem2/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem2/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem2/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem2/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem2/b) '])
set_param([sys,'/','Subsystem2/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem2/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem2/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem2/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem2/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem2/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem2/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem2/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem2'],...
		'Mask Display','RECUPERER LES RESULTATS\n\nDANS L''ESPACE MATLAB',...
		'Mask Dialogue','eval(''display'')')


%     Finished composite block 'Subsystem2'.

set_param([sys,'/','Subsystem2'],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[105,735,405,796])

add_block('built-in/Note',[sys,'/',['Ci-dessous cliquer',13,'pour obtenir les tracs de courbes']])
set_param([sys,'/',['Ci-dessous cliquer',13,'pour obtenir les tracs de courbes']],...
		'Font Name','Arial',...
		'Font Angle','italic',...
		'Font Size',16,...
		'position',[250,685,255,690])


%     Subsystem  'Courbes'.

new_system([sys,'/','Courbes'])
set_param([sys,'/','Courbes'],'Location',[5,45,1259,959])


%     Subsystem  'Courbes/Extras8'.

new_system([sys,'/','Courbes/Extras8'])
set_param([sys,'/','Courbes/Extras8'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras8'],...
		'Mask Display','Voir courbe\ncouple de\ncommande (Nm)',...
		'Mask Dialogue','eval(''plot(temps,cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras8'.

set_param([sys,'/','Courbes/Extras8'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,155,750,221])


%     Subsystem  'Courbes/Extras9'.

new_system([sys,'/','Courbes/Extras9'])
set_param([sys,'/','Courbes/Extras9'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras9'],...
		'Mask Display','Voir les courbes\nmoments magntiques\nmx,my,mz',...
		'Mask Dialogue','eval(''plot(temps,mx,temps,my,temps,mz);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras9'.

set_param([sys,'/','Courbes/Extras9'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[330,75,555,136])


%     Subsystem  'Courbes/Extras5'.

new_system([sys,'/','Courbes/Extras5'])
set_param([sys,'/','Courbes/Extras5'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras5'],...
		'Mask Display','Voir les courbes\ndes angles (en degrs)',...
		'Mask Dialogue','eval(''plot(temps,roulis,temps,tangage,temps,lacet);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras5'.

set_param([sys,'/','Courbes/Extras5'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,75,295,141])


%     Subsystem  'Courbes/Extras7'.

new_system([sys,'/','Courbes/Extras7'])
set_param([sys,'/','Courbes/Extras7'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras7'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (/s)',...
		'Mask Dialogue','eval(''K=180/pi;plot(temps,p*K,temps,q*K,temps,r*K);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras7'.

set_param([sys,'/','Courbes/Extras7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,160,300,221])


%     Subsystem  'Courbes/Extras6'.

new_system([sys,'/','Courbes/Extras6'])
set_param([sys,'/','Courbes/Extras6'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras6'],...
		'Mask Display','Voir la courbe\n\nenergie absolue (J)',...
		'Mask Dialogue','eval(''plot(temps,energie);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras6'.

set_param([sys,'/','Courbes/Extras6'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,75,750,131])

add_block('built-in/Note',[sys,'/','Courbes/********************************************************************************************'])
set_param([sys,'/','Courbes/********************************************************************************************'],...
		'Font Name','Arial',...
		'Font Size',20,...
		'position',[670,345,675,350])

add_block('built-in/Note',[sys,'/','Courbes/COURBES PRINCIPALES'])
set_param([sys,'/','Courbes/COURBES PRINCIPALES'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[595,15,600,20])


%     Subsystem  'Courbes/Extras14'.

new_system([sys,'/','Courbes/Extras14'])
set_param([sys,'/','Courbes/Extras14'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras14'],...
		'Mask Display','Voir courbe\ndiffrence Bc - B ',...
		'Mask Dialogue','eval(''plot(temps,Bc-B);title(''''Composantes de Bc - B dans xyz'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras14'.

set_param([sys,'/','Courbes/Extras14'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,450,275,516])


%     Subsystem  'Courbes/Extras10'.

new_system([sys,'/','Courbes/Extras10'])
set_param([sys,'/','Courbes/Extras10'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras10'],...
		'Mask Display','Couleurs des courbes\nroulis=jaune\ntangage=bleu\nlacet=magenta')


%     Finished composite block 'Courbes/Extras10'.

set_param([sys,'/','Courbes/Extras10'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,230,755,311])


%     Subsystem  'Courbes/Extras26'.

new_system([sys,'/','Courbes/Extras26'])
set_param([sys,'/','Courbes/Extras26'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras26'],...
		'Mask Display','Voir la vitesse angulaire\nde lacet seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,r,''''c.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras26'.

set_param([sys,'/','Courbes/Extras26'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,165,540,226])


%     Subsystem  'Courbes/Extras25'.

new_system([sys,'/','Courbes/Extras25'])
set_param([sys,'/','Courbes/Extras25'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras25'],...
		'Mask Display','Voir la vitesse angulaire\nde roulis seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,p,''''y.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras25'.

set_param([sys,'/','Courbes/Extras25'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,240,540,301])


%     Subsystem  'Courbes/Extras11'.

new_system([sys,'/','Courbes/Extras11'])
set_param([sys,'/','Courbes/Extras11'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras11'],...
		'Mask Display','Voir la vitesse angulaire\nde tangage seul (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,q,''''m.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras11'.

set_param([sys,'/','Courbes/Extras11'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[90,235,295,296])


%     Subsystem  'Courbes/Extras22'.

new_system([sys,'/','Courbes/Extras22'])
set_param([sys,'/','Courbes/Extras22'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras22'],...
		'Mask Display','Voir l''angle\nde tangage seul ()',...
		'Mask Dialogue','eval(''plot(temps,tangage,''''m.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras22'.

set_param([sys,'/','Courbes/Extras22'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,320,300,381])


%     Subsystem  'Courbes/Extras38'.

new_system([sys,'/','Courbes/Extras38'])
set_param([sys,'/','Courbes/Extras38'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras38'],...
		'Mask Display','Voir l''angle\nde roulis seul ()',...
		'Mask Dialogue','eval(''plot(temps,roulis,''''y.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras38'.

set_param([sys,'/','Courbes/Extras38'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,315,540,376])


%     Subsystem  'Courbes/Extras12'.

new_system([sys,'/','Courbes/Extras12'])
set_param([sys,'/','Courbes/Extras12'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras12'],...
		'Mask Display','Voir courbe\ncouple commande\n roue (Nm)',...
		'Mask Dialogue','eval(''plot(temps,cpl_roue);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras12'.

set_param([sys,'/','Courbes/Extras12'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1025,450,1200,516])

add_block('built-in/Note',[sys,'/','Courbes/COURBES ANNEXES ET MISE AU POINT'])
set_param([sys,'/','Courbes/COURBES ANNEXES ET MISE AU POINT'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[595,385,600,390])


%     Subsystem  'Courbes/Extras39'.

new_system([sys,'/','Courbes/Extras39'])
set_param([sys,'/','Courbes/Extras39'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras39'],...
		'Mask Display','Voir l''angle\nde lacet seul ()',...
		'Mask Dialogue','eval(''plot(temps,lacet,''''c.'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras39'.

set_param([sys,'/','Courbes/Extras39'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[575,320,760,381])


%     Subsystem  'Courbes/Extras41'.

new_system([sys,'/','Courbes/Extras41'])
set_param([sys,'/','Courbes/Extras41'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras41'],...
		'Mask Display','Voir courbe du\ncouplage roue-satellite\nsur le roulis',...
		'Mask Dialogue','eval(''plot(temps,couplage_roue_sat(1:length(temps),1),''''y'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras41'.

set_param([sys,'/','Courbes/Extras41'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,610,1215,671])


%     Subsystem  'Courbes/Extras42'.

new_system([sys,'/','Courbes/Extras42'])
set_param([sys,'/','Courbes/Extras42'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras42'],...
		'Mask Display','Voir courbe du\ncouplage roue-satellite\nsur le lacet',...
		'Mask Dialogue','eval(''plot(temps,couplage_roue_sat(1:length(temps),3),''''c'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras42'.

set_param([sys,'/','Courbes/Extras42'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,685,1220,751])


%     Subsystem  'Courbes/Extras35'.

new_system([sys,'/','Courbes/Extras35'])
set_param([sys,'/','Courbes/Extras35'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras35'],...
		'Mask Display','Voir courbe\nunitaire(B)',...
		'Mask Dialogue','eval(''plot(temps,unitB);title(''''Unitaire de B'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras35'.

set_param([sys,'/','Courbes/Extras35'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[590,620,765,676])


%     Subsystem  'Courbes/Extras28'.

new_system([sys,'/','Courbes/Extras28'])
set_param([sys,'/','Courbes/Extras28'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras28'],...
		'Mask Display','Voir courbe\nnorme de la\ndiffrence PB - B ',...
		'Mask Dialogue','eval(''plot(temps,norm_PB_moins_B);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras28'.

set_param([sys,'/','Courbes/Extras28'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[595,685,770,751])


%     Subsystem  'Courbes/Extras24'.

new_system([sys,'/','Courbes/Extras24'])
set_param([sys,'/','Courbes/Extras24'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras24'],...
		'Mask Display','Voir courbe\ndcalage R / R0\nen degrs',...
		'Mask Dialogue','eval(''plot(temps,angle_rot_R0_to_R);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras24'.

set_param([sys,'/','Courbes/Extras24'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,530,760,596])


%     Subsystem  'Courbes/Extras23'.

new_system([sys,'/','Courbes/Extras23'])
set_param([sys,'/','Courbes/Extras23'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras23'],...
		'Mask Display','Voir courbe\ncouple total 1',...
		'Mask Dialogue','eval(''plot(temps,cpl_laurens2);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras23'.

set_param([sys,'/','Courbes/Extras23'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[585,450,760,516])


%     Subsystem  'Courbes/Extras46'.

new_system([sys,'/','Courbes/Extras46'])
set_param([sys,'/','Courbes/Extras46'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras46'],...
		'Mask Display','Voir courbe\nmoment magntique\n de rappel Mr',...
		'Mask Dialogue','eval(''plot(temps,Bc*KPD_laurens);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras46'.

set_param([sys,'/','Courbes/Extras46'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[320,625,540,691])


%     Subsystem  'Courbes/Extras29'.

new_system([sys,'/','Courbes/Extras29'])
set_param([sys,'/','Courbes/Extras29'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras29'],...
		'Mask Display','Voir courbe\nmoment magntique\n amortisseur Ma',...
		'Mask Dialogue','eval(''plot(temps,(Bc_prime-B_prime)*FPD_laurens);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras29'.

set_param([sys,'/','Courbes/Extras29'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[325,555,540,616])


%     Subsystem  'Courbes/Extras21'.

new_system([sys,'/','Courbes/Extras21'])
set_param([sys,'/','Courbes/Extras21'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras21'],...
		'Mask Display','Voir courbe\nmoment magntique\nde Laurens M1=Mr+Ma\npartie utile',...
		'Mask Dialogue','eval(''plot(temps,M1);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras21'.

set_param([sys,'/','Courbes/Extras21'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[330,450,535,541])


%     Subsystem  'Courbes/Extras47'.

new_system([sys,'/','Courbes/Extras47'])
set_param([sys,'/','Courbes/Extras47'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras47'],...
		'Mask Display','Angle entre les\naxes y et Y \nen degrs')
set_param([sys,'/','Courbes/Extras47'],...
		'Mask Dialogue','eval(''plot(temps,180/pi*acos(2*(quater(:,1).^2+quater(:,3).^2)-1));title(''''Angle entre y et Y, en degrs'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras47'.

set_param([sys,'/','Courbes/Extras47'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[590,765,765,831])


%     Subsystem  'Courbes/Extras13'.

new_system([sys,'/','Courbes/Extras13'])
set_param([sys,'/','Courbes/Extras13'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras13'],...
		'Mask Display','Voir courbe\nvitesse de la\n roue (rd/s)',...
		'Mask Dialogue','eval(''plot(temps,vit_roue);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras13'.

set_param([sys,'/','Courbes/Extras13'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1035,530,1205,596])


%     Subsystem  'Courbes/Extras51'.

new_system([sys,'/','Courbes/Extras51'])
set_param([sys,'/','Courbes/Extras51'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras51'],...
		'Mask Display','Unitaire soleil\ndans xyz satellite)',...
		'Mask Dialogue','eval(''plot(temps,Us_xyz);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras51'.

set_param([sys,'/','Courbes/Extras51'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,695,540,756])


%     Subsystem  'Courbes/Extras53'.

new_system([sys,'/','Courbes/Extras53'])
set_param([sys,'/','Courbes/Extras53'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras53'],...
		'Mask Display','Projection\nUnitaire soleil\ndans (x,z) satellite)\ndepuis le dbut',...
		'Mask Dialogue','eval(''plot(Us_xyz(:,1),Us_xyz(:,3));grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras53'.

set_param([sys,'/','Courbes/Extras53'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[95,765,300,846])


%     Subsystem  'Courbes/Extras55'.

new_system([sys,'/','Courbes/Extras55'])
set_param([sys,'/','Courbes/Extras55'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras55'],...
		'Mask Display','Position eclipse',...
		'Mask Dialogue','eval(''plot(temps,1.0001*eclipse,''''m.'''');title(['''' Temps eclipse du jour = '''' num2str(jour)]);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras55'.

set_param([sys,'/','Courbes/Extras55'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[335,765,540,826])


%     Subsystem  'Courbes/Extras15'.

new_system([sys,'/','Courbes/Extras15'])
set_param([sys,'/','Courbes/Extras15'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras15'],...
		'Mask Display','Voir courbe\ndiffrence B''c - B'' ',...
		'Mask Dialogue','eval(''plot(temps,Bc_prime-B_prime);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras15'.

set_param([sys,'/','Courbes/Extras15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[100,535,275,601])


%     Subsystem  'Courbes/Extras50'.

new_system([sys,'/','Courbes/Extras50'])
set_param([sys,'/','Courbes/Extras50'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras50'],...
		'Mask Display','Energie\nproduite (J)')
set_param([sys,'/','Courbes/Extras50'],...
		'Mask Dialogue','eval(''plot(temps,energie_spin,''''m.'''');text(1000,25000,[''''Pm= '''' num2str(puissance_moyenne(length(q))) ''''W'''']);title([''''Energie sur une priode du jour '''' num2str(jour)]);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras50'.

set_param([sys,'/','Courbes/Extras50'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[80,610,285,671])


%     Subsystem  'Courbes/Extras52'.

new_system([sys,'/','Courbes/Extras52'])
set_param([sys,'/','Courbes/Extras52'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras52'],...
		'Mask Display','Puissance\ninstantane(W)',...
		'Mask Dialogue','eval(''plot(temps,puissance_inst,''''w'''');title(['''' Puissance instantane du jour = '''' num2str(jour)]);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras52'.

set_param([sys,'/','Courbes/Extras52'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[85,690,290,751])


%     Subsystem  'Courbes/Extras34'.

new_system([sys,'/','Courbes/Extras34'])
set_param([sys,'/','Courbes/Extras34'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras34'],...
		'Mask Display','Graphe de vrification\n\ndu quaternion(2:4) calcul\n\n ou issu de la simulation')
set_param([sys,'/','Courbes/Extras34'],...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),2:4),temps,cos((vitesse_rotation*temps+beta_fix-pi/2)/2),''''w'''',temps,sin((vitesse_rotation*temps+beta_fix-pi/2)/2));title(''''Quaternion(2:4)'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras34'.

set_param([sys,'/','Courbes/Extras34'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[790,70,1020,176])


%     Subsystem  'Courbes/Extras44'.

new_system([sys,'/','Courbes/Extras44'])
set_param([sys,'/','Courbes/Extras44'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras44'],...
		'Mask Display','Voir courbes\ncouple perturbateur ',...
		'Mask Dialogue','eval(''plot(temps,perturb);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras44'.

set_param([sys,'/','Courbes/Extras44'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[810,445,985,511])


%     Subsystem  'Courbes/Extras45'.

new_system([sys,'/','Courbes/Extras45'])
set_param([sys,'/','Courbes/Extras45'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras45'],...
		'Mask Display','Voir courbes\ndes 3 stops',...
		'Mask Dialogue','eval(''plot(temps,stop1,''''y'''',temps,stop2,''''m'''',temps,stop3,''''c'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras45'.

set_param([sys,'/','Courbes/Extras45'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[815,530,985,591])


%     Subsystem  'Courbes/Extras48'.

new_system([sys,'/','Courbes/Extras48'])
set_param([sys,'/','Courbes/Extras48'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras48'],...
		'Mask Display','Angle entre les\naxes x et X \nen degrs')
set_param([sys,'/','Courbes/Extras48'],...
		'Mask Dialogue','eval(''plot(temps,180/pi*acos(2*(quater(:,1).^2+quater(:,2).^2)-1));title(''''Angle entre x et X, en degrs'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras48'.

set_param([sys,'/','Courbes/Extras48'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[815,600,990,666])


%     Subsystem  'Courbes/Extras56'.

new_system([sys,'/','Courbes/Extras56'])
set_param([sys,'/','Courbes/Extras56'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras56'],...
		'Mask Display','Angle entre les\naxes y et Y \nen degrs')
set_param([sys,'/','Courbes/Extras56'],...
		'Mask Dialogue','eval(''plot(temps,180/pi*acos(2*(quater(:,1).^2+quater(:,3).^2)-1));title(''''Angle entre y et Y, en degrs'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras56'.

set_param([sys,'/','Courbes/Extras56'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[820,680,995,746])


%     Subsystem  'Courbes/Extras49'.

new_system([sys,'/','Courbes/Extras49'])
set_param([sys,'/','Courbes/Extras49'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras49'],...
		'Mask Display','Angle entre les\naxes z et Z \nen degrs')
set_param([sys,'/','Courbes/Extras49'],...
		'Mask Dialogue','eval(''plot(temps,180/pi*acos(2*(quater(:,1).^2+quater(:,4).^2)-1));title(''''Angle entre z et Z, en degrs'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras49'.

set_param([sys,'/','Courbes/Extras49'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[820,760,995,826])


%     Subsystem  'Courbes/Extras54'.

new_system([sys,'/','Courbes/Extras54'])
set_param([sys,'/','Courbes/Extras54'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras54'],...
		'Mask Display','Projection\nUnitaire soleil\ndans (x,z) satellite)\nen fin de simu',...
		'Mask Dialogue','eval(''N=length(q);plot(Us_xyz(N-2000:N,1),Us_xyz(N-2000:N,3));grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras54'.

set_param([sys,'/','Courbes/Extras54'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,760,785,841])


%     Subsystem  'Courbes/Extras43'.

new_system([sys,'/','Courbes/Extras43'])
set_param([sys,'/','Courbes/Extras43'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras43'],...
		'Mask Display','Voir courbe\nstop multiple ',...
		'Mask Dialogue','eval(''plot(temps,stop_multiple);grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras43'.

set_param([sys,'/','Courbes/Extras43'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1040,760,1215,826])


%     Subsystem  'Courbes/Extras32'.

new_system([sys,'/','Courbes/Extras32'])
set_param([sys,'/','Courbes/Extras32'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras32'],...
		'Mask Display','Voir courbe\nquaternion(2:4)',...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),2:4));title(''''Quaternion(2:4)'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras32'.

set_param([sys,'/','Courbes/Extras32'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1055,75,1230,141])


%     Subsystem  'Courbes/Extras33'.

new_system([sys,'/','Courbes/Extras33'])
set_param([sys,'/','Courbes/Extras33'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras33'],...
		'Mask Display','Voir courbe\nquaternion(1)',...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),1));title(''''Premire composante du quaternion'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras33'.

set_param([sys,'/','Courbes/Extras33'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1055,145,1230,211])


%     Subsystem  'Courbes/Extras16'.

new_system([sys,'/','Courbes/Extras16'])
set_param([sys,'/','Courbes/Extras16'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras16'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\nabsolues ( / XYZ ) (/s)')
set_param([sys,'/','Courbes/Extras16'],...
		'Mask Dialogue','eval(''K=180/pi;plot(temps,omega_quat);title(''''Vitesses absolues / au repre orbital'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras16'.

set_param([sys,'/','Courbes/Extras16'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1055,220,1245,276])


%     Subsystem  'Courbes/Extras17'.

new_system([sys,'/','Courbes/Extras17'])
set_param([sys,'/','Courbes/Extras17'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras17'],...
		'Mask Display','Voir les courbes\nvitesses angulaires\np, q, r (rd/s)')
set_param([sys,'/','Courbes/Extras17'],...
		'Mask Dialogue','eval(''K=1;plot(temps,p*K,temps,q*K,temps,r*K);title('''' Vitesses angulaires p q r en rd/s'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras17'.

set_param([sys,'/','Courbes/Extras17'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[1045,285,1250,346])


%     Subsystem  'Courbes/Extras36'.

new_system([sys,'/','Courbes/Extras36'])
set_param([sys,'/','Courbes/Extras36'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras36'],...
		'Mask Display','Voir courbe\nincidence\nsoleil sur panneaux')
set_param([sys,'/','Courbes/Extras36'],...
		'Mask Dialogue','eval(''plot(temps,eclipse.*acos(-Us_xyz(:,2)*cos(ang)-Us_xyz(:,3)*sin(ang))*180/pi);title(''''Angle incidence sur les panneaux'''');grid;figure(gcf)'')')


%     Finished composite block 'Courbes/Extras36'.

set_param([sys,'/','Courbes/Extras36'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[820,180,995,246])


%     Subsystem  'Courbes/Extras37'.

new_system([sys,'/','Courbes/Extras37'])
set_param([sys,'/','Courbes/Extras37'],'Location',[4,42,1012,678])
set_param([sys,'/','Courbes/Extras37'],...
		'Mask Display','Optimisation des temps\n\nde basculement\n\nseul les quaternion(2)\n\n calculs ou simuls\n\nsont conservs')
set_param([sys,'/','Courbes/Extras37'],...
		'Mask Dialogue','eval(''plot(temps,quater(1:length(quater),2),temps,cos((vitesse_rotation*temps+beta_fix-pi/2)/2),''''w'''');title(''''Quaternion(2) calcul et simul'''');figure(gcf)'')')


%     Finished composite block 'Courbes/Extras37'.

set_param([sys,'/','Courbes/Extras37'],...
		'ForeGround',1,...
		'BackGround',4,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[790,250,1020,356])
set_param([sys,'/','Courbes'],...
		'Mask Display','SORTIES GRAPHIQUES')


%     Finished composite block 'Courbes'.

set_param([sys,'/','Courbes'],...
		'ForeGround',1,...
		'BackGround',5,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[120,605,395,681])


%     Subsystem  'Subsystem11'.

new_system([sys,'/','Subsystem11'])
set_param([sys,'/','Subsystem11'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem11/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem11/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem11/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem11/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem11/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem11/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem11/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem11/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem11/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem11/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem11/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem11/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem11/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem11/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem11/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem11/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem11/d) Lancer la simulation'])
set_param([sys,'/','Subsystem11/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem11/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem11/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem11/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem11/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem11/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem11/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem11/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem11/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem11/b) '])
set_param([sys,'/','Subsystem11/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem11/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem11/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem11/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem11/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem11/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem11/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem11/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem11/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem11/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem11'],...
		'Mask Display','2 - Roue BCT\nH=0.005 mN\n 1665 tours/mn',...
		'Mask Dialogue','eval(''H=0.005'')')


%     Finished composite block 'Subsystem11'.

set_param([sys,'/','Subsystem11'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[540,740,755,831])


%     Subsystem  'Subsystem10'.

new_system([sys,'/','Subsystem10'])
set_param([sys,'/','Subsystem10'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem10/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem10/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem10/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem10/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem10/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem10/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem10/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem10/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem10/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem10/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem10/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem10/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem10/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem10/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem10/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem10/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem10/d) Lancer la simulation'])
set_param([sys,'/','Subsystem10/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem10/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem10/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem10/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem10/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem10/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem10/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem10/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem10/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem10/b) '])
set_param([sys,'/','Subsystem10/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem10/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem10/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem10/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem10/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem10/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem10/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem10/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem10/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem10/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem10'],...
		'Mask Display','2 - Roue BCT\nH=0.001 mN\n 333 tours/mn',...
		'Mask Dialogue','eval(''H=0.001'')')


%     Finished composite block 'Subsystem10'.

set_param([sys,'/','Subsystem10'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[535,640,750,731])


%     Subsystem  'Subsystem6'.

new_system([sys,'/','Subsystem6'])
set_param([sys,'/','Subsystem6'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem6/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem6/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem6/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem6/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem6/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem6/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem6/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem6/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem6/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem6/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem6/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem6/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem6/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem6/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem6/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem6/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem6/d) Lancer la simulation'])
set_param([sys,'/','Subsystem6/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem6/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem6/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem6/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem6/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem6/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem6/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem6/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem6/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem6/b) '])
set_param([sys,'/','Subsystem6/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem6/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem6/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem6/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem6/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem6/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem6/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem6/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem6'],...
		'Mask Display','2 - Roue AOCS\nH=0.00058 mNs\ntype A\nWheel  8000 tr/mn',...
		'Mask Dialogue','eval(''H=0.00058'')')


%     Finished composite block 'Subsystem6'.

set_param([sys,'/','Subsystem6'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[785,745,1000,836])


%     Subsystem  'Subsystem7'.

new_system([sys,'/','Subsystem7'])
set_param([sys,'/','Subsystem7'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem7/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem7/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem7/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem7/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem7/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem7/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem7/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem7/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem7/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem7/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem7/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem7/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem7/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem7/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem7/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem7/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem7/d) Lancer la simulation'])
set_param([sys,'/','Subsystem7/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem7/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem7/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem7/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem7/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem7/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem7/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem7/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem7/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem7/b) '])
set_param([sys,'/','Subsystem7/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem7/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem7/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem7/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem7/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem7/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem7/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem7/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem7'],...
		'Mask Display','2 - Roue BCT\nH=0.002 mN\n 666 tours/mn',...
		'Mask Dialogue','eval(''H=0.002'')')


%     Finished composite block 'Subsystem7'.

set_param([sys,'/','Subsystem7'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[785,640,1000,731])


%     Subsystem  'Subsystem9'.

new_system([sys,'/','Subsystem9'])
set_param([sys,'/','Subsystem9'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem9/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem9/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem9/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem9/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem9/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem9/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem9/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem9/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem9/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem9/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem9/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem9/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem9/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem9/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem9/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem9/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem9/d) Lancer la simulation'])
set_param([sys,'/','Subsystem9/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem9/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem9/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem9/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem9/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem9/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem9/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem9/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem9/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem9/b) '])
set_param([sys,'/','Subsystem9/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem9/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem9/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem9/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem9/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem9/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem9/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem9/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem9/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem9/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem9'],...
		'Mask Display','2- Roue\nH=0.003 mNs\nBCT Micro Reaction\nWheel  1000 tr/mn',...
		'Mask Dialogue','eval(''H=0.003'')')


%     Finished composite block 'Subsystem9'.

set_param([sys,'/','Subsystem9'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1025,745,1240,836])


%     Subsystem  'Subsystem8'.

new_system([sys,'/','Subsystem8'])
set_param([sys,'/','Subsystem8'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem8/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem8/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem8/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem8/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem8/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem8/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem8/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem8/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem8/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem8/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem8/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem8/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem8/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem8/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem8/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem8/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem8/d) Lancer la simulation'])
set_param([sys,'/','Subsystem8/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem8/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem8/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem8/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem8/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem8/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem8/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem8/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem8/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem8/b) '])
set_param([sys,'/','Subsystem8/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem8/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem8/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem8/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem8/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem8/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem8/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem8/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem8/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem8/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem8'],...
		'Mask Display','2 - Roue\nHy=0.009 mNs\nBCT Micro Reaction\nWheel  3000 tr/mn',...
		'Mask Dialogue','eval(''Hy=0.009'')')


%     Finished composite block 'Subsystem8'.

set_param([sys,'/','Subsystem8'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1020,645,1235,736])


%     Subsystem  'Subsystem5'.

new_system([sys,'/','Subsystem5'])
set_param([sys,'/','Subsystem5'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem5/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem5/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem5/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem5/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem5/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem5/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem5/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem5/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem5/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem5/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem5/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem5/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem5/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem5/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem5/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem5/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem5/d) Lancer la simulation'])
set_param([sys,'/','Subsystem5/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem5/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem5/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem5/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem5/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem5/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem5/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem5/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem5/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem5/b) '])
set_param([sys,'/','Subsystem5/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem5/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem5/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem5/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem5/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem5/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem5/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem5/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem5'],...
		'Mask Display','2 - Roue\nH=0.018 mNs\nBCT Micro Reaction\nWheel  6000 tr/mn',...
		'Mask Dialogue','eval(''H=0.018'')')


%     Finished composite block 'Subsystem5'.

set_param([sys,'/','Subsystem5'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[1015,545,1230,636])


%     Subsystem  'Subsystem4'.

new_system([sys,'/','Subsystem4'])
set_param([sys,'/','Subsystem4'],'Location',[20,278,1248,1004])

add_block('built-in/Note',[sys,'/','Subsystem4/COMMENT UTILISER LA SIMULATION'])
set_param([sys,'/','Subsystem4/COMMENT UTILISER LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem4/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'])
set_param([sys,'/','Subsystem4/1 - Le choix des paramtres doit tre ralis dans un fichier  ouvrir dans notepad (ou autre)'],...
		'Font Size',14,...
		'position',[495,65,500,70])

add_block('built-in/Note',[sys,'/',['Subsystem4/Pour la simulation principale c''est DER.M']])
set_param([sys,'/',['Subsystem4/Pour la simulation principale c''est DER.M']],...
		'Font Size',14,...
		'position',[410,90,415,95])

add_block('built-in/Note',[sys,'/',['Subsystem4/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']])
set_param([sys,'/',['Subsystem4/Pour la simulation orbite polaire et ces particulier d''une matrice d''inertie de revolution autour du tangage, c''est POLEDAT.M']],...
		'Font Size',14,...
		'position',[710,115,715,120])

add_block('built-in/Note',[sys,'/',['Subsystem4/a) Un CLEAR dans l''espace Matlab est conseill']])
set_param([sys,'/',['Subsystem4/a) Un CLEAR dans l''espace Matlab est conseill']],...
		'Font Size',14,...
		'position',[405,185,410,190])

add_block('built-in/Note',[sys,'/','Subsystem4/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'])
set_param([sys,'/','Subsystem4/b) Cliquer sur le cadre magenta pour excuter le fichier de donnes derivdat ou poledat'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[560,210,565,215])

add_block('built-in/Note',[sys,'/','Subsystem4/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'])
set_param([sys,'/','Subsystem4/c) Paramtrer la simulation, choisir un temps assez long disons 30000 secondes'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[530,235,535,240])

add_block('built-in/Note',[sys,'/',['Subsystem4/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']])
set_param([sys,'/',['Subsystem4/2 - Le lancement de la simulation principale se fait dans l''ordre suivant;']],...
		'Font Size',14,...
		'position',[405,160,410,165])

add_block('built-in/Note',[sys,'/','Subsystem4/d) Lancer la simulation'])
set_param([sys,'/','Subsystem4/d) Lancer la simulation'],...
		'Font Size',14,...
		'position',[295,265,300,270])

add_block('built-in/Note',[sys,'/','Subsystem4/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'])
set_param([sys,'/','Subsystem4/3 - Attendre que le stop arrte les calculs quand les vitesses sont suffisamment rduites,'],...
		'Font Size',14,...
		'position',[480,295,485,300])

add_block('built-in/Note',[sys,'/','Subsystem4/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'])
set_param([sys,'/','Subsystem4/NB :Le stop peut tre inactiv en portant son gain  0, dans son masque'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[495,325,500,330])

add_block('built-in/Note',[sys,'/','Subsystem4/4 - Comment voir rapidement les courbes intressantes :'])
set_param([sys,'/','Subsystem4/4 - Comment voir rapidement les courbes intressantes :'],...
		'Font Name','Arial',...
		'Font Underline',1,...
		'Font Size',14,...
		'position',[360,370,365,375])

add_block('built-in/Note',[sys,'/','Subsystem4/a) En cliquant dans les boites vertes'])
set_param([sys,'/','Subsystem4/a) En cliquant dans les boites vertes'],...
		'Font Size',14,...
		'position',[355,400,360,405])

add_block('built-in/Note',[sys,'/','Subsystem4/b) '])
set_param([sys,'/','Subsystem4/b) '],...
		'Font Size',14,...
		'position',[220,435,225,440])

add_block('built-in/Note',[sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']])
set_param([sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['Subsystem4/Les sorties suivantes sont disponibles dans l''espace Matlab p q r roulis lacet tangage energie p2q2 ...',13,'(p+q = carr de la rotation transversale)']],...
		'position',[635,435,640,440])

add_block('built-in/Note',[sys,'/','Subsystem4/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'])
set_param([sys,'/','Subsystem4/c) Roulis lacet tangage temps sont aussi stockes dans angles.mat,  vous de les exploiter'],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[570,480,575,485])

add_block('built-in/Note',[sys,'/',['Subsystem4/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']])
set_param([sys,'/',['Subsystem4/NB :Les couleurs lors de 3 sorties sont',13,' roulis jaune',13,'tangage magenta',13,'lacet cyan']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[580,545,585,550])

add_block('built-in/Note',[sys,'/','Subsystem4/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem4/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem4'],...
		'Mask Display','2 - La rouede mes\nsimulations\nH=3.2 e-3 mNs',...
		'Mask Dialogue','eval(''H=0.0032'')')


%     Finished composite block 'Subsystem4'.

set_param([sys,'/','Subsystem4'],...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[785,545,1000,631])


%     Subsystem  'Subsystem'.

new_system([sys,'/','Subsystem'])
set_param([sys,'/','Subsystem'],'Location',[0,40,1264,964])

add_block('built-in/Note',[sys,'/',['Subsystem/SORTIES ET TRAITEMENT DE LA SORTIE',13,'POUR VERIFICATIONS DE FONCTIONNALITE']])
set_param([sys,'/',['Subsystem/SORTIES ET TRAITEMENT DE LA SORTIE',13,'POUR VERIFICATIONS DE FONCTIONNALITE']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[620,40,625,45])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux'])
set_param([sys,'/','Subsystem/Demux'],...
		'outputs','[3 4]',...
		'position',[470,339,520,506])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn1'])
set_param([sys,'/','Subsystem/MATLAB Fcn1'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','angles(u)',...
		'Output Width','3',...
		'position',[700,448,760,482])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace14'])
set_param([sys,'/','Subsystem/To Workspace14'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','lacet',...
		'buffer','points_trace',...
		'position',[965,490,1035,510])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace12'])
set_param([sys,'/','Subsystem/To Workspace12'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','roulis',...
		'buffer','points_trace',...
		'position',[965,420,1035,440])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux2'])
set_param([sys,'/','Subsystem/Demux2'],...
		'outputs','3',...
		'position',[860,413,905,517])

add_block('built-in/Note',[sys,'/','Subsystem/Energie'])
set_param([sys,'/','Subsystem/Energie'],...
		'position',[885,390,890,395])

add_block('built-in/Note',[sys,'/','Subsystem/Drive de Q'])
set_param([sys,'/','Subsystem/Drive de Q'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[635,665,640,670])

add_block('built-in/Note',[sys,'/',['Subsystem/Rotation atellite par',13,'rapport aux axes orbitaux']])
set_param([sys,'/',['Subsystem/Rotation atellite par',13,'rapport aux axes orbitaux']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[905,670,910,675])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn4'])
set_param([sys,'/','Subsystem/MATLAB Fcn4'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','omegaorb(u)',...
		'Output Width','3',...
		'position',[815,621,895,659])

add_block('built-in/Note',[sys,'/','Subsystem/Quaternion Q'])
set_param([sys,'/','Subsystem/Quaternion Q'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[600,435,605,440])

add_block('built-in/Mux',[sys,'/','Subsystem/Mux'])
set_param([sys,'/','Subsystem/Mux'],...
		'inputs','[4 4]',...
		'position',[735,591,770,689])

add_block('built-in/Note',[sys,'/',['Subsystem/Q''']])
set_param([sys,'/',['Subsystem/Q''']],...
		'position',[710,670,715,675])

add_block('built-in/Note',[sys,'/','Subsystem/Q'])
set_param([sys,'/','Subsystem/Q'],...
		'position',[705,590,710,595])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace11'])
set_param([sys,'/','Subsystem/To Workspace11'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','omega_quat',...
		'buffer','points_trace',...
		'position',[915,624,1000,656])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn3'])
set_param([sys,'/','Subsystem/MATLAB Fcn3'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','ENERGIE(u)',...
		'Output Width','1',...
		'position',[745,361,825,399])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux1'])
set_param([sys,'/','Subsystem/Demux1'],...
		'outputs','[3 4]',...
		'position',[470,539,520,706])

add_block('built-in/Note',[sys,'/',['Subsystem/Entre de',13,'p q r Q']])
set_param([sys,'/',['Subsystem/Entre de',13,'p q r Q']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[420,365,425,370])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace13'])
set_param([sys,'/','Subsystem/To Workspace13'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','tangage',...
		'buffer','points_trace',...
		'position',[965,455,1035,475])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace8'])
set_param([sys,'/','Subsystem/To Workspace8'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','temps',...
		'buffer','points_trace',...
		'position',[965,525,1035,545])

add_block('built-in/Clock',[sys,'/','Subsystem/Clock'])
set_param([sys,'/','Subsystem/Clock'],...
		'position',[905,525,925,545])

add_block('built-in/Demux',[sys,'/','Subsystem/Demux3'])
set_param([sys,'/','Subsystem/Demux3'],...
		'outputs','3',...
		'position',[610,158,655,262])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace15'])
set_param([sys,'/','Subsystem/To Workspace15'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','p',...
		'buffer','points_trace',...
		'position',[715,165,785,185])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace16'])
set_param([sys,'/','Subsystem/To Workspace16'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','q',...
		'buffer','points_trace',...
		'position',[715,200,785,220])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace17'])
set_param([sys,'/','Subsystem/To Workspace17'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','r',...
		'buffer','points_trace',...
		'position',[715,235,785,255])

add_block('built-in/Note',[sys,'/',['Subsystem/Vitesses angulaires absolues',13,'autour des axes x y z']])
set_param([sys,'/',['Subsystem/Vitesses angulaires absolues',13,'autour des axes x y z']],...
		'position',[895,185,900,190])

add_block('built-in/Inport',[sys,'/','Subsystem/in_2'])
set_param([sys,'/','Subsystem/in_2'],...
		'Port','2',...
		'position',[390,615,410,635])


%     Subsystem  ['Subsystem/Vitesses angulaires',13,'rd//s'].

new_system([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],'Location',[0,59,274,252])

add_block('built-in/Inport',[sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/x']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/x']],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfuny',...
		'parameters','ax, color,dt',...
		'position',[130,55,180,75])
add_line([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],[90,65;125,65])
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter plotting ranges and line type.|Time range:|y-min:|y-max:|Line type (rgbw-:*). Seperate each plot by ''/'':')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Translate','color = @4; ax = [0, @1, @2, @3]; dt = -1;')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Scope block. Look at the m-file sfuny.m to see how it works. This block can take scalar or vector input signal.')
set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'Mask Entries','10000\/-0.2\/0.2\/''y-/g--/c-./w:/m*/ro/b+''\/')


%     Finished composite block ['Subsystem/Vitesses angulaires',13,'rd//s'].

set_param([sys,'/',['Subsystem/Vitesses angulaires',13,'rd//s']],...
		'orientation',2,...
		'BackGround',2,...
		'position',[365,192,475,228])


%     Subsystem  'Subsystem/Angles en degrs'.

new_system([sys,'/','Subsystem/Angles en degrs'])
set_param([sys,'/','Subsystem/Angles en degrs'],'Location',[0,59,274,252])

add_block('built-in/Inport',[sys,'/','Subsystem/Angles en degrs/x'])
set_param([sys,'/','Subsystem/Angles en degrs/x'],...
		'position',[65,55,85,75])

add_block('built-in/S-Function',[sys,'/',['Subsystem/Angles en degrs/S-function',13,'M-file which plots',13,'lines',13,'']])
set_param([sys,'/',['Subsystem/Angles en degrs/S-function',13,'M-file which plots',13,'lines',13,'']],...
		'function name','sfuny',...
		'parameters','ax, color,dt',...
		'position',[130,55,180,75])
add_line([sys,'/','Subsystem/Angles en degrs'],[90,65;125,65])
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Display','plot(0,0,100,100,[90,10,10,10,90,90,10],[65,65,90,40,40,90,90],[90,78,69,54,40,31,25,10],[77,60,48,46,56,75,81,84])',...
		'Mask Type','Graph scope.')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Dialogue','Graph scope using MATLAB graph window.\nEnter plotting ranges and line type.|Time range:|y-min:|y-max:|Line type (rgbw-:*). Seperate each plot by ''/'':')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Translate','color = @4; ax = [0, @1, @2, @3]; dt = -1;')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Help','This block plots to the MATLAB graph window and can be used as an improved version of the Scope block. Look at the m-file sfuny.m to see how it works. This block can take scalar or vector input signal.')
set_param([sys,'/','Subsystem/Angles en degrs'],...
		'Mask Entries','10000\/-200\/200\/''y-/g--/c-./w:/m*/ro/b+''\/')


%     Finished composite block 'Subsystem/Angles en degrs'.

set_param([sys,'/','Subsystem/Angles en degrs'],...
		'orientation',2,...
		'BackGround',2,...
		'position',[705,522,815,558])

add_block('built-in/Inport',[sys,'/','Subsystem/in_1'])
set_param([sys,'/','Subsystem/in_1'],...
		'position',[290,415,310,435])

add_block('built-in/Gain',[sys,'/',['Subsystem/Conversion',13,'rd // degrs']])
set_param([sys,'/',['Subsystem/Conversion',13,'rd // degrs']],...
		'Gain','180/pi',...
		'position',[795,452,820,478])

add_block('built-in/MATLAB Fcn',[sys,'/','Subsystem/MATLAB Fcn2'])
set_param([sys,'/','Subsystem/MATLAB Fcn2'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'MATLAB Fcn','vibomega(u)',...
		'Output Width','1',...
		'position',[660,108,720,142])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace9'])
set_param([sys,'/','Subsystem/To Workspace9'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','GDomega',...
		'buffer','points_trace',...
		'position',[775,115,845,135])

add_block('built-in/To Workspace',[sys,'/','Subsystem/To Workspace7'])
set_param([sys,'/','Subsystem/To Workspace7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','energie',...
		'buffer','points_trace',...
		'position',[960,370,1030,390])
add_line([sys,'/','Subsystem'],[525,380;740,380])
add_line([sys,'/','Subsystem'],[830,380;955,380])
add_line([sys,'/','Subsystem'],[315,425;465,425])
add_line([sys,'/','Subsystem'],[525,465;695,465])
add_line([sys,'/','Subsystem'],[930,535;960,535])
add_line([sys,'/','Subsystem'],[415,625;465,625])
add_line([sys,'/','Subsystem'],[525,665;730,665])
add_line([sys,'/','Subsystem'],[675,465;675,615;730,615])
add_line([sys,'/','Subsystem'],[900,640;910,640])
add_line([sys,'/','Subsystem'],[765,465;790,465])
add_line([sys,'/','Subsystem'],[825,465;855,465])
add_line([sys,'/','Subsystem'],[910,430;960,430])
add_line([sys,'/','Subsystem'],[910,465;960,465])
add_line([sys,'/','Subsystem'],[910,500;960,500])
add_line([sys,'/','Subsystem'],[550,380;550,210;605,210])
add_line([sys,'/','Subsystem'],[660,175;710,175])
add_line([sys,'/','Subsystem'],[660,210;710,210])
add_line([sys,'/','Subsystem'],[660,245;710,245])
add_line([sys,'/','Subsystem'],[550,210;480,210])
add_line([sys,'/','Subsystem'],[840,465;840,540;820,540])
add_line([sys,'/','Subsystem'],[550,210;550,125;655,125])
add_line([sys,'/','Subsystem'],[725,125;770,125])
add_line([sys,'/','Subsystem'],[775,640;810,640])
set_param([sys,'/','Subsystem'],...
		'Mask Display','SORTIE DES ANGLES\n\nEN DEGRES & ENERGIE')


%     Finished composite block 'Subsystem'.

set_param([sys,'/','Subsystem'],...
		'hide name',0,...
		'position',[845,259,1085,341])

add_block('built-in/Note',[sys,'/',['Vecteur d''tat de 7 composantes']])
set_param([sys,'/',['Vecteur d''tat de 7 composantes']],...
		'position',[365,560,370,565])


%     Subsystem  'UnNamed2'.

new_system([sys,'/','UnNamed2'])
set_param([sys,'/','UnNamed2'],'Location',[588,239,931,333])

add_block('built-in/Note',[sys,'/',['UnNamed2/Drive du quaternion d''attitude']])
set_param([sys,'/',['UnNamed2/Drive du quaternion d''attitude']],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[185,5,190,10])

add_block('built-in/Outport',[sys,'/','UnNamed2/Outport'])
set_param([sys,'/','UnNamed2/Outport'],...
		'position',[260,45,280,65])

add_block('built-in/Inport',[sys,'/','UnNamed2/Inport'])
set_param([sys,'/','UnNamed2/Inport'],...
		'position',[80,45,100,65])

add_block('built-in/MATLAB Fcn',[sys,'/','UnNamed2/deriv_qt.m'])
set_param([sys,'/','UnNamed2/deriv_qt.m'],...
		'MATLAB Fcn','deriv_qt(u)',...
		'Output Width','4',...
		'position',[150,39,220,71])
add_line([sys,'/','UnNamed2'],[225,55;255,55])
add_line([sys,'/','UnNamed2'],[105,55;145,55])
set_param([sys,'/','UnNamed2'],...
		'Mask Display','F(X , t)\n\nPOUR LE QUATERNION')


%     Finished composite block 'UnNamed2'.

set_param([sys,'/','UnNamed2'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[120,443,335,547])

add_block('built-in/Note',[sys,'/','3'])
set_param([sys,'/','3'],...
		'position',[515,330,520,335])

add_block('built-in/Note',[sys,'/','4'])
set_param([sys,'/','4'],...
		'position',[510,470,515,475])


%     Subsystem  'F(X,t) pour la rotation'.

new_system([sys,'/','F(X,t) pour la rotation'])
set_param([sys,'/','F(X,t) pour la rotation'],'Location',[5,45,1259,959])
open_system([sys,'/','F(X,t) pour la rotation'])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/pqr'])
set_param([sys,'/','F(X,t) pour la rotation/pqr'],...
		'position',[325,375,330,380])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Demux'])
set_param([sys,'/','F(X,t) pour la rotation/Demux'],...
		'outputs','[3 4]',...
		'position',[225,375,275,495])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/tau',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/tau',13,'']],...
		'position',[85,240,90,245])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Vecteur d''tat ',13,'7 composantes',13,'p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Vecteur d''tat ',13,'7 composantes',13,'p q r Q']],...
		'position',[105,335,110,340])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Inport'])
set_param([sys,'/','F(X,t) pour la rotation/Inport'],...
		'position',[75,300,95,320])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Suppression de',13,'toutes les',13,'perturbations']])
set_param([sys,'/',['F(X,t) pour la rotation/Suppression de',13,'toutes les',13,'perturbations']],...
		'position',[515,325,520,330])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']])
set_param([sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',22)
set_param([sys,'/',['F(X,t) pour la rotation/F(X,t) POUR LA PARTIE CONCERNANT LA SEULE ROTATION',13,'AVEC LA METHODE INITIALE DE M LAURENS']],...
		'position',[630,40,635,45])


%     Subsystem  ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1'].

new_system([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],'Location',[4,176,1271,782])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/B']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/B']],...
		'position',[510,425,515,430])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Vecteur d''tat',13,'7 composantes',13,'p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Vecteur d''tat',13,'7 composantes',13,'p q r Q']],...
		'position',[140,515,145,520])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain3']],...
		'hide name',0,...
		'Gain','-1',...
		'position',[800,382,835,408])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple magntique rsiduel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple magntique rsiduel']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[880,399,885,404])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple arodynamique']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Couple arodynamique']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[690,277,695,282])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel1']],...
		'MATLAB Fcn','norm(u,2)',...
		'Output Width','1',...
		'position',[770,438,830,472])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Gain']],...
		'Gain','mom_magn_res',...
		'position',[375,315,490,365])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace']],...
		'hide name',0,...
		'mat-name','moment_residuel',...
		'buffer','5000',...
		'position',[895,440,1015,470])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Moment magntique rsiduel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Moment magntique rsiduel']],...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[245,246,250,251])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Produit',13,'vectoriel']],...
		'MATLAB Fcn','prodvect(u)',...
		'Output Width','3',...
		'position',[640,378,700,412])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux5']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux5']],...
		'inputs','[3 3]',...
		'position',[540,286,585,504])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux3']],...
		'ForeGround',1,...
		'BackGround',0,...
		'inputs','3',...
		'position',[320,288,350,392])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant12']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant12']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','alpha',...
		'position',[130,293,275,317])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant11']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant11']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','beta',...
		'position',[130,328,275,352])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant10']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant10']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','gamma',...
		'position',[130,363,275,387])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/M']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/M']],...
		'position',[525,315,530,320])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant6']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant6']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero3',...
		'position',[785,318,930,342])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant5']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant5']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero2',...
		'position',[785,283,930,307])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant4']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant4']],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'Value','cpl_aero1',...
		'position',[785,248,930,272])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux1']],...
		'ForeGround',1,...
		'BackGround',0,...
		'inputs','3',...
		'position',[975,243,1005,347])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Termes rsidus de gradient',13,'et forces inertie']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Termes rsidus de gradient',13,'et forces inertie']],...
		'ForeGround',4,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[895,195,900,200])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant3']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','Irt*vitesse_rotation^2',...
		'position',[795,158,940,182])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant2']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant2']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','3*Irl*vitesse_rotation^2',...
		'position',[795,123,940,147])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Constant1']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'Value','-4*Itl*vitesse_rotation^2',...
		'position',[795,88,940,112])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Mux']],...
		'ForeGround',7,...
		'BackGround',0,...
		'inputs','3',...
		'position',[985,83,1015,187])


%     Subsystem  ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite'].

new_system([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],'Location',[195,288,1041,757])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']],...
		'hide name',0,...
		'MATLAB Fcn','champmgn(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL',...
		'position',[250,95,425,145])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Clock']],...
		'position',[180,104,215,136])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/BX BY BZ']],...
		'position',[340,70,345,75])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Vecteur d''tat ',13,'( 7 composantes)']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Vecteur d''tat ',13,'( 7 composantes)']],...
		'position',[70,235,75,240])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/in_1']],...
		'position',[50,190,70,210])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/out_1']],...
		'position',[705,150,725,170])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Sortie Bx By Bz']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Sortie Bx By Bz']],...
		'position',[725,200,730,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Equateur au temps 0 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Equateur au temps 0 s']],...
		'position',[150,75,155,80])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/MATLAB Fcn1']],...
		'MATLAB Fcn','champsat(u)',...
		'Output Width','3',...
		'position',[560,143,620,177])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite/Mux1']],...
		'inputs','[3 7]',...
		'position',[485,80,530,240])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[220,120;245,120])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[430,120;480,120])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[625,160;700,160])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[75,200;480,200])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],[535,160;555,160])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],...
		'Mask Display','CHAMP MAGNETIQUE EN\nAXES SATELLITE\nEntre=vecteur d''tat\nSortie Bx By Bz')


%     Finished composite block ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite'].

set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Champ magntique',13,'en axes satellite']],...
		'position',[240,458,440,532])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'Font Size',16)
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Ensemble des couples perturbateurs externes ou externes',13,'',13,'Il faut fournir les 3 composantes ',13,'du vecteur unitaire de la direction',13,'du moment magntique rsiduel ',13,'( si on le connat)']],...
		'position',[385,95,390,100])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Inport']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Inport']],...
		'position',[20,485,40,505])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Sum']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Sum']],...
		'inputs','++++',...
		'position',[1090,254,1125,361])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Outport']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Outport']],...
		'position',[1200,300,1220,320])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'ForeGround',2,...
		'Font Name','Arial',...
		'Font Size',14)
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'position',[665,505,670,510])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Partie principale du gradient de gravit']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/Partie principale du gradient de gravit']],...
		'position',[945,540,950,545])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']],...
		'ForeGround',7,...
		'BackGround',0,...
		'hide name',0,...
		'MATLAB Fcn','gradgrav(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/MATLAB Fcn']],...
		'Mask Display','GRADIENT PRINCIPAL\n\nDE GRAVITE',...
		'position',[630,554,805,606])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace1']])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1/To Workspace1']],...
		'hide name',0,...
		'mat-name','perturb',...
		'buffer','points_trace',...
		'position',[1180,188,1235,212])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[45,495;235,495])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[840,395;1055,395;1055,320;1085,320])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[705,395;735,395;735,455;765,455])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[735,395;795,395])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[835,455;890,455])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[590,395;635,395])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[445,495;470,495;470,450;535,450])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[495,340;535,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[355,340;370,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,305;315,305])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,340;315,340])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[280,375;315,375])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,100;980,100])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,135;980,135])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[945,170;980,170])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,260;970,260])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,295;970,295])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1010,295;1085,295])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[935,330;970,330])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1020,135;1075,135;1085,270])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[75,495;75,580;625,580])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1130,310;1195,310])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[810,580;1070,580;1070,345;1085,345])
add_line([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],[1155,310;1155,200;1175,200])
set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],...
		'Mask Display','Couples\nperturbateurs Mp')


%     Finished composite block ['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1'].

set_param([sys,'/',['F(X,t) pour la rotation/Perturbations',13,'arodynamiques1']],...
		'ForeGround',2,...
		'hide name',0,...
		'position',[285,282,415,338])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain3'])
set_param([sys,'/','F(X,t) pour la rotation/Gain3'],...
		'hide name',0,...
		'position',[885,248,925,282])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/out_1'],...
		'position',[1125,325,1145,345])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain1'])
set_param([sys,'/','F(X,t) pour la rotation/Gain1'],...
		'hide name',0,...
		'position',[1070,322,1095,348])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Sum1'])
set_param([sys,'/','F(X,t) pour la rotation/Sum1'],...
		'position',[970,194,1010,476])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Suppression de',13,'tous les couples']])
set_param([sys,'/',['F(X,t) pour la rotation/Suppression de',13,'tous les couples']],...
		'position',[900,205,905,210])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain'])
set_param([sys,'/','F(X,t) pour la rotation/Gain'],...
		'hide name',0,...
		'position',[460,293,500,327])

add_block('built-in/Sum',[sys,'/','F(X,t) pour la rotation/Sum2'])
set_param([sys,'/','F(X,t) pour la rotation/Sum2'],...
		'position',[680,128,720,372])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Q'])
set_param([sys,'/','F(X,t) pour la rotation/Q'],...
		'position',[280,445,285,450])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Couple roue unidimensionnel sur tangage'])
set_param([sys,'/','F(X,t) pour la rotation/Couple roue unidimensionnel sur tangage'],...
		'position',[705,435,710,440])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/out_2'])
set_param([sys,'/','F(X,t) pour la rotation/out_2'],...
		'Port','2',...
		'position',[1080,495,1100,515])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Sortie vitesse roue'])
set_param([sys,'/','F(X,t) pour la rotation/Sortie vitesse roue'],...
		'position',[985,515,990,520])

add_block('built-in/State-Space',[sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']])
set_param([sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'A','[]',...
		'B','[]',...
		'C','[]',...
		'D','inv(Asat)',...
		'Mask Display','Inv(Asat)',...
		'Mask Type','Matrix Gain')
set_param([sys,'/',['F(X,t) pour la rotation/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'Mask Help','Multiplies input vector by entered matrix to produce output vector (y=Au).',...
		'position',[750,242,850,288])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/magneto.m1'])
set_param([sys,'/','F(X,t) pour la rotation/magneto.m1'],...
		'hide name',0,...
		'MATLAB Fcn','derivrot(u)',...
		'Output Width','3',...
		'Mask Display','DERIVROT(u)',...
		'position',[375,379,500,431])


%     Subsystem  'F(X,t) pour la rotation/Calcul des commandes, moments et roue'.

new_system([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],'Location',[5,45,1259,959])
open_system([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Weight','bold')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'Font Size',18)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/REGULATION PAR LA DERIVEE AVEC UN GAIN LINEAIRE',13,'ET UN RAPPEL PROPORTIONNEL SUR UN CHAMP MAGNETIQUE DE CONSIGNE']],...
		'position',[595,10,600,15])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'ForeGround',2,...
		'Font Name','Arial')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/NB : Dans le vol rel, le champ magntique est celui ',13,'rgnant  la position du satellite']],...
		'Font Size',14,...
		'position',[605,82,610,87])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/pqrQ'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/pqrQ'],...
		'position',[445,205,450,210])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/in_1'],...
		'hide name',0,...
		'position',[430,240,450,260])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple total magntique + roue',13,'de la rgulation de Laurens amliore']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple total magntique + roue',13,'de la rgulation de Laurens amliore']],...
		'position',[900,200,905,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Vecteur d''tat']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Vecteur d''tat']],...
		'position',[375,235,380,240])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_7'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_7'],...
		'hide name',0,...
		'Port','2',...
		'position',[750,265,770,285])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_5'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/out_5'],...
		'hide name',0,...
		'position',[750,215,770,235])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple roue uniaxial'])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue/Couple roue uniaxial'],...
		'position',[850,260,855,265])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],'Location',[4,44,1268,968])
open_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','cpl_laurens2')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace9']],...
		'buffer','points_trace',...
		'position',[1030,324,1150,346])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace12']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace12']],...
		'hide name',0,...
		'mat-name','mz',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace12']],...
		'position',[1110,279,1145,301])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace11']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace11']],...
		'hide name',0,...
		'mat-name','my',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace11']],...
		'position',[1110,254,1145,276])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace10']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace10']],...
		'hide name',0,...
		'mat-name','mx',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace10']],...
		'position',[1110,229,1145,251])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'ForeGround',1,...
		'BackGround',2)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'MATLAB Fcn','prodvect(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple ',13,'magntique']],...
		'position',[1065,418,1125,452])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc']],...
		'position',[485,340,490,345])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B ']],...
		'position',[505,295,510,300])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''c']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''c']],...
		'position',[515,380,520,385])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative']],...
		'position',[455,399,500,421])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux']],...
		'hide name',0,...
		'inputs','[3 3 3 3]',...
		'position',[530,251,565,434])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B''']],...
		'position',[445,250,450,255])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','Bc_prime')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace5']],...
		'buffer','points_trace',...
		'position',[425,439,490,461])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']],...
		'hide name',0,...
		'mat-name','Bc',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace4']],...
		'position',[430,469,495,491])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux4']],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'outputs','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux4']],...
		'position',[1005,227,1045,303])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ rel']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ rel']],...
		'position',[860,590,865,595])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux2']],...
		'hide name',0,...
		'inputs','3',...
		'position',[1055,660,1080,730])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant']],...
		'hide name',0,...
		'Value','0',...
		'position',[1000,660,1020,680])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant1']],...
		'hide name',0,...
		'Value','0',...
		'position',[1000,710,1020,730])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']],...
		'hide name',0,...
		'mat-name','M4',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace17']],...
		'position',[1150,364,1185,386])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux4']],...
		'hide name',0,...
		'inputs','[3 3]',...
		'position',[1015,386,1040,479])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'outputs','[3 1]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Demux3']],...
		'position',[905,392,945,468])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_2']],...
		'Port','2',...
		'position',[1125,760,1145,780])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace13']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace13']],...
		'hide name',0,...
		'mat-name','cpl_roue',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace13']],...
		'position',[1060,799,1130,821])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'uniaxial']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'uniaxial']],...
		'position',[1100,825,1105,830])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'dimension 3',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple roue',13,'dimension 3',13,'']],...
		'position',[1165,665,1170,670])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Gain2']],...
		'hide name',0,...
		'position',[660,332,685,358])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'MATLAB Fcn','laur0(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment',13,' magntique',13,'de Laurens',13,'en colonne']],...
		'position',[580,328,640,362])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','M1')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace7']],...
		'buffer','points_trace',...
		'position',[615,289,650,311])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum4']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum4']],...
		'hide name',0,...
		'inputs','+-',...
		'position',[520,139,550,206])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple',13,'de commande',13,' raliser']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Couple',13,'de commande',13,' raliser']],...
		'position',[1235,525,1240,530])

add_block('built-in/Sum',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Sum3']],...
		'hide name',0,...
		'position',[1155,395,1185,555])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/out_3']],...
		'position',[1225,465,1245,485])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Possibilit de',13,'supprimer M1 ou M3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Possibilit de',13,'supprimer M1 ou M3']],...
		'position',[635,205,640,210])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Somme des',13,'moments magntiques',13,'M1+M2+M3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Somme des',13,'moments magntiques',13,'M1+M2+M3']],...
		'position',[735,235,740,240])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace16']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace16']],...
		'hide name',0,...
		'mat-name','norm_PB_moins_B')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace16']],...
		'buffer','points_trace',...
		'position',[725,161,845,189])

add_block('built-in/Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Fcn']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Fcn']],...
		'Expr','(u(1)^2+u(2)^2+u(3)^2)^0.5',...
		'position',[655,165,695,185])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Mux1']],...
		'hide name',0,...
		'inputs','[3 1]',...
		'position',[730,338,765,522])

add_block('built-in/Constant',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Constant2']],...
		'hide name',0,...
		'Value','0',...
		'position',[630,465,650,485])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pas de roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Pas de roue']],...
		'position',[645,485,650,490])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'ForeGround',1)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'BackGround',2)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'MATLAB Fcn','mom_prop(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'Output Width','4')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Moment magntique ',13,'et',13,'couple roue',13,'plafonns']],...
		'position',[805,413,865,447])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace6']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace6']],...
		'hide name',0,...
		'mat-name','B',...
		'buffer','points_trace')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace6']],...
		'position',[310,329,375,351])

add_block('built-in/Derivative',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Derivative1']],...
		'position',[320,264,365,286])

add_block('built-in/To Workspace',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace15']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace15']],...
		'orientation',2,...
		'hide name',0,...
		'mat-name','B_prime')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/To Workspace15']],...
		'buffer','points_trace',...
		'position',[315,219,380,241])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B']],...
		'position',[475,515,480,520])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/in_1']],...
		'position',[5,264,20,286])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Entre vecteur d''tat ',13,'7 composantes p q r Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Entre vecteur d''tat ',13,'7 composantes p q r Q']],...
		'position',[70,370,75,375])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B+pqr+Q']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/B+pqr+Q']],...
		'position',[55,410,60,415])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],'Location',[382,182,1228,651])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'hide name',0)
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'MATLAB Fcn','champmgn(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn3']],...
		'position',[250,95,425,145])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Clock']],...
		'position',[180,104,215,136])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/BX BY BZ']],...
		'position',[340,70,345,75])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Vecteur d''tat ',13,'( 7 composantes)']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Vecteur d''tat ',13,'( 7 composantes)']],...
		'position',[70,235,75,240])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/in_1']],...
		'position',[50,190,70,210])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/out_1']],...
		'position',[705,150,725,170])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Sortie Bx By Bz']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Sortie Bx By Bz']],...
		'position',[725,200,730,205])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Equateur au temps 0 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Equateur au temps 0 s']],...
		'position',[150,75,155,80])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']],...
		'MATLAB Fcn','champsat(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/MATLAB Fcn1']],...
		'position',[560,143,620,177])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Mux1']],...
		'inputs','[3 7]')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_/Mux1']],...
		'position',[485,80,530,240])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],[220,120;245,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],[430,120;480,120])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],[625,160;700,160])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],[75,200;480,200])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],[535,160;555,160])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],...
		'Mask Display','CHAMP MAGNETIQUE EN\nAXES SATELLITE\nEntre=vecteur d''tat\nSortie Bx By Bz')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Champ magntique',13,'en axes satellite_']],...
		'position',[50,236,210,314])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc '].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],'Location',[4,44,1268,968])
open_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Fin']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Fin']],...
		'position',[565,280,570,285])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Survie ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Survie ']],...
		'position',[550,455,555,460])


%     Subsystem  ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir'].

new_system([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],'Location',[295,1114256,595,1114496])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Mux2']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Mux2']],...
		'inputs','3',...
		'position',[215,60,255,210])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Gain1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Gain1']],...
		'Gain','-1',...
		'position',[125,162,150,188])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Gain']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Gain']],...
		'Gain','-1',...
		'position',[125,117,150,143])

add_block('built-in/Demux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Demux']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/Demux']],...
		'outputs','3',...
		'position',[55,64,95,196])

add_block('built-in/Inport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/in_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/in_1']],...
		'position',[15,120,35,140])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir/out_1']],...
		'position',[285,125,305,145])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[155,175;175,175;175,185;210,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[155,130;175,130;175,135;210,135])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[100,175;120,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[100,130;120,130])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[100,85;210,85])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[40,130;50,130])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],[260,135;280,135])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],...
		'Mask Display','Pointage\n\nnadir')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Pointage Nadir']],...
		'position',[325,163,490,207])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /14000 <t < 14600  attitude quasi inertielle']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /14000 <t < 14600  attitude quasi inertielle']],...
		'position',[635,665,640,670])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /4000 < t < 10000  pointage Nadir']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /4000 < t < 10000  pointage Nadir']],...
		'position',[640,620,645,625])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /t < 4000  attitude quasi inertielle']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /t < 4000  attitude quasi inertielle']],...
		'position',[645,575,650,580])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /BX BY BZ']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /BX BY BZ']],...
		'position',[165,335,170,340])

add_block('built-in/Clock',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Clock']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Clock']],...
		'position',[5,369,40,401])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'hide name',0,...
		'MATLAB Fcn','champmgn(u)')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'Mask Display','CHAMP MAGNETIQUE\nREPERE ORBITAL')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /MATLAB Fcn3']],...
		'position',[75,360,250,410])

add_block('built-in/Mux',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Mux1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Mux1']],...
		'inputs','[1 3]',...
		'position',[295,273,340,422])

add_block('built-in/MATLAB Fcn',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Survie']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Survie']],...
		'MATLAB Fcn','Bc_Beta(u)',...
		'Output Width','3')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Survie']],...
		'Mask Display','Bc_Beta',...
		'position',[375,331,530,369])

add_block('built-in/Switch',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,' 4000 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,' 4000 s']],...
		'Threshold','4000',...
		'position',[620,260,665,510])

add_block('built-in/Switch',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'10000 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'10000 s']],...
		'Threshold','10000',...
		'position',[705,50,750,310])

add_block('built-in/Switch',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'14600 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'14600 s']],...
		'Threshold','14600',...
		'position',[845,60,890,320])

add_block('built-in/Switch',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'26740 s']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'26740 s']],...
		'Threshold','26740')
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Switch',13,'',13,'26740 s']],...
		'position',[980,60,1025,320])

add_block('built-in/Outport',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /out_1']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /out_1']],...
		'position',[1100,180,1120,200])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Bcx']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Bcx']],...
		'position',[1140,150,1145,155])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Bcy']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Bcy']],...
		'position',[1140,180,1145,185])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Bcz']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /Bcz']],...
		'position',[1145,210,1150,215])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /14600 < t < 26740 mode fin']])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc /14600 < t < 26740 mode fin']],...
		'position',[630,700,635,705])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[45,385;70,385])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[45,385;45,310;290,310])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[255,385;290,385])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[345,350;370,350])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[535,350;575,350;575,470;615,470])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[45,385;45,461;450,461;450,385;615,385])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[670,385;690,385;700,265])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[255,385;255,185;320,185])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[495,185;550,185;550,300;615,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[595,385;595,180;700,180])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[575,350;575,95;700,95])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[450,461;450,555;805,555;807,190;840,190])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[550,185;550,30;790,30;790,105;840,105])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[755,180;785,180;785,275;840,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[805,555;935,554;935,190;975,190])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[895,190;912,188;912,275;975,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[575,95;577,94;575,15;940,17;940,105;975,105])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],[1030,190;1095,190])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],...
		'Mask Display','CALCUL\nBc')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc '].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS/Bc ']],...
		'position',[275,366,395,414])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[505,410;495,450])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[400,390;430,390;430,365;525,365])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[400,390;400,480;425,480])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[430,390;430,410;450,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[505,410;525,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[570,345;575,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[25,275;45,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[770,430;800,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[870,430;900,430])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[950,410;1010,410])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1045,435;1060,435])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1130,435;1150,435])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1190,475;1220,475])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1025,670;1050,670])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1025,720;1050,720])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1085,695;1115,695;1115,515;1150,515])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1190,475;1190,335;1155,335])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[970,410;970,265;1000,265])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1050,240;1105,240])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1050,265;1105,265])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1050,290;1105,290])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[950,450;967,450;967,695;1050,695])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1130,435;1130,375;1145,375])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[970,695;970,770;1120,770])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[1030,770;1030,810;1055,810])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[645,345;655,345])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[690,345;705,345;705,385;725,385])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[690,345;690,300;655,300])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[465,365;465,155;515,155])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[215,275;245,275;245,325;250,325;480,320;480,190;515,190])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[555,175;650,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[700,175;720,175])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[655,475;725,475])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[370,275;525,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[215,275;315,275])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[423,275;425,230;385,230])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[480,320;525,320])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[266,325;265,340;305,340])
add_line([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],[245,325;245,540;986,540;986,460;1010,455])
set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],...
		'Mask Display','CALCUL DES MOMENTS\n\nMAGNETIQUES\n\nET COUPLE ROUE')


%     Finished composite block ['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS'].

set_param([sys,'/',['F(X,t) pour la rotation/Calcul des commandes, moments et roue/Moment magntique et',13,'couple roue en ',13,'rgulation de LAURENS']],...
		'position',[510,199,725,301])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[455,250;505,250])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[730,225;745,225])
add_line([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],[730,275;745,275])
set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],...
		'Mask Display','REGULATION BOUSSOLE\n\nTECHNIQUE LAURENS\n\nPULSEE TYPE 2')


%     Finished composite block 'F(X,t) pour la rotation/Calcul des commandes, moments et roue'.

set_param([sys,'/','F(X,t) pour la rotation/Calcul des commandes, moments et roue'],...
		'position',[205,171,435,249])

add_block('built-in/Demux',[sys,'/','F(X,t) pour la rotation/Demux1'])
set_param([sys,'/','F(X,t) pour la rotation/Demux1'],...
		'outputs','[1 3]',...
		'position',[355,536,395,569])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/unitaire',13,'de la rotation Q']])
set_param([sys,'/',['F(X,t) pour la rotation/unitaire',13,'de la rotation Q']],...
		'position',[550,580,555,585])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/To Workspace7'])
set_param([sys,'/','F(X,t) pour la rotation/To Workspace7'],...
		'hide name',0,...
		'mat-name','unitQ',...
		'buffer','points_trace',...
		'position',[530,545,585,575])

add_block('built-in/MATLAB Fcn',[sys,'/','F(X,t) pour la rotation/MATLAB Fcn1'])
set_param([sys,'/','F(X,t) pour la rotation/MATLAB Fcn1'],...
		'hide name',0,...
		'MATLAB Fcn','VectUnit(u)',...
		'Output Width','3',...
		'position',[435,543,495,577])


%     Subsystem  'F(X,t) pour la rotation/Subsystem'.

new_system([sys,'/','F(X,t) pour la rotation/Subsystem'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],'Location',[4,50,970,451])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Subsystem/Inversion',13,'signe',13,'(action//roue)',13,'']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Inversion',13,'signe',13,'(action//roue)',13,'']],...
		'ForeGround',2,...
		'Gain','-1',...
		'position',[115,122,145,148])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Subsystem/Gain3'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/Gain3'],...
		'ForeGround',2,...
		'Gain','30/pi',...
		'position',[445,118,470,152])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace5'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace5'],...
		'ForeGround',2,...
		'BackGround',7,...
		'hide name',0,...
		'Font Size',14,...
		'mat-name','vit_roue',...
		'buffer','points_trace',...
		'position',[515,120,595,150])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue ',13,'en rd//s']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue ',13,'en rd//s']],...
		'ForeGround',2,...
		'position',[520,210,525,215])

add_block('built-in/Inport',[sys,'/','F(X,t) pour la rotation/Subsystem/in_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/in_1'],...
		'ForeGround',2,...
		'position',[20,125,40,145])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue',13,'en tours//mn']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Vitesse roue',13,'en tours//mn']],...
		'ForeGround',2,...
		'position',[455,70,460,75])

add_block('built-in/Gain',[sys,'/',['F(X,t) pour la rotation/Subsystem/Inertie',13,'de la ',13,'roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Inertie',13,'de la ',13,'roue']],...
		'ForeGround',2,...
		'BackGround',7,...
		'Gain','1/J_ROUE',...
		'position',[195,115,230,155])

add_block('built-in/Note',[sys,'/','F(X,t) pour la rotation/Subsystem/SUIVI DE LA ROUE'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/SUIVI DE LA ROUE'],...
		'ForeGround',2,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[390,10,395,15])

add_block('built-in/Integrator',[sys,'/','F(X,t) pour la rotation/Subsystem/Integrator2'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/Integrator2'],...
		'ForeGround',2,...
		'BackGround',7,...
		'Initial','rotation_initiale',...
		'position',[280,113,325,157])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'],...
		'ForeGround',2,...
		'BackGround',7,...
		'hide name',0,...
		'Font Size',14,...
		'mat-name','vitesseroue',...
		'buffer','points_trace')
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To Workspace6'],...
		'position',[445,179,590,211])

add_block('built-in/Outport',[sys,'/','F(X,t) pour la rotation/Subsystem/out_1'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/out_1'],...
		'hide name',0,...
		'position',[610,330,630,350])

add_block('built-in/To File',[sys,'/','F(X,t) pour la rotation/Subsystem/To File'])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem/To File'],...
		'ForeGround',2,...
		'BackGround',7,...
		'File name','vit_roue.mat',...
		'Matrix name','vit_roue_rd_s',...
		'position',[455,247,570,273])

add_block('built-in/Note',[sys,'/',['F(X,t) pour la rotation/Subsystem/Sortie de la vitesse ',13,'de la roue']])
set_param([sys,'/',['F(X,t) pour la rotation/Subsystem/Sortie de la vitesse ',13,'de la roue']],...
		'position',[485,320,490,325])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[475,135;510,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[150,135;190,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[45,135;110,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[235,135;275,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[330,135;440,135])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[395,135;395,195;440,195])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[395,195;395,345;605,340])
add_line([sys,'/','F(X,t) pour la rotation/Subsystem'],[420,195;420,260;450,260])
set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],...
		'Mask Display','EVOLUTION DE LA\nVITESSE DE LA ROUE')


%     Finished composite block 'F(X,t) pour la rotation/Subsystem'.

set_param([sys,'/','F(X,t) pour la rotation/Subsystem'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'position',[710,474,855,536])

add_block('built-in/To Workspace',[sys,'/','F(X,t) pour la rotation/To Workspace4'])
set_param([sys,'/','F(X,t) pour la rotation/To Workspace4'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','quater',...
		'buffer','points_trace',...
		'position',[365,450,430,480])

add_block('built-in/Gain',[sys,'/','F(X,t) pour la rotation/Gain2'])
set_param([sys,'/','F(X,t) pour la rotation/Gain2'],...
		'position',[550,175,595,205])
add_line([sys,'/','F(X,t) pour la rotation'],[420,310;455,310])
add_line([sys,'/','F(X,t) pour la rotation'],[280,405;370,405])
add_line([sys,'/','F(X,t) pour la rotation'],[280,465;360,465])
add_line([sys,'/','F(X,t) pour la rotation'],[1015,335;1065,335])
add_line([sys,'/','F(X,t) pour la rotation'],[1100,335;1120,335])
add_line([sys,'/','F(X,t) pour la rotation'],[100,310;280,310])
add_line([sys,'/','F(X,t) pour la rotation'],[190,310;190,435;220,435])
add_line([sys,'/','F(X,t) pour la rotation'],[190,310;200,210])
add_line([sys,'/','F(X,t) pour la rotation'],[505,310;675,310])
add_line([sys,'/','F(X,t) pour la rotation'],[725,250;745,265])
add_line([sys,'/','F(X,t) pour la rotation'],[855,265;880,265])
add_line([sys,'/','F(X,t) pour la rotation'],[930,265;965,265])
add_line([sys,'/','F(X,t) pour la rotation'],[505,405;965,405])
add_line([sys,'/','F(X,t) pour la rotation'],[600,190;675,190])
add_line([sys,'/','F(X,t) pour la rotation'],[440,230;580,230;580,505;705,505])
add_line([sys,'/','F(X,t) pour la rotation'],[860,505;1075,505])
add_line([sys,'/','F(X,t) pour la rotation'],[440,190;545,190])
add_line([sys,'/','F(X,t) pour la rotation'],[295,465;295,555;350,555])
add_line([sys,'/','F(X,t) pour la rotation'],[400,560;430,560])
add_line([sys,'/','F(X,t) pour la rotation'],[500,560;525,560])
set_param([sys,'/','F(X,t) pour la rotation'],...
		'Mask Display','F(X , t ) \n\nPOUR LA ROTATION')


%     Finished composite block 'F(X,t) pour la rotation'.

set_param([sys,'/','F(X,t) pour la rotation'],...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Size',14,...
		'position',[125,278,350,362])

add_block('built-in/Mux',[sys,'/','Mux'])
set_param([sys,'/','Mux'],...
		'hide name',0,...
		'inputs','[3 4]',...
		'position',[500,233,545,582])

add_block('built-in/Note',[sys,'/',['Entres',13,'drive rotation (3)']])
set_param([sys,'/',['Entres',13,'drive rotation (3)']],...
		'position',[435,235,440,240])

add_block('built-in/Note',[sys,'/','Drive quaternion (4)'])
set_param([sys,'/','Drive quaternion (4)'],...
		'position',[425,495,430,500])

add_block('built-in/Sum',[sys,'/','Sum'])
set_param([sys,'/','Sum'],...
		'position',[420,280,445,355])

add_block('built-in/Integrator',[sys,'/','Integrator'])
set_param([sys,'/','Integrator'],...
		'hide name',0,...
		'Initial','etat_init',...
		'position',[620,384,720,436])

add_block('built-in/Note',[sys,'/',['p'' q'' r'' Q''']])
set_param([sys,'/',['p'' q'' r'' Q''']],...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[785,315,790,320])

add_block('built-in/Note',[sys,'/','p q r Q'])
set_param([sys,'/','p q r Q'],...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[785,250,790,255])

add_block('built-in/Note',[sys,'/','1  3 = rotation inertielle'])
set_param([sys,'/','1  3 = rotation inertielle'],...
		'position',[180,575,185,580])

add_block('built-in/Note',[sys,'/',['4  7 = quaternion d''attitude dans le repre orbital']])
set_param([sys,'/',['4  7 = quaternion d''attitude dans le repre orbital']],...
		'position',[585,580,590,585])


%     Subsystem  'Couplage roue-satellite'.

new_system([sys,'/','Couplage roue-satellite'])
set_param([sys,'/','Couplage roue-satellite'],'Location',[4,44,1195,548])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/CALCUL DU COUPLE DE COUPLAGE ROUE SATELLITE'])
set_param([sys,'/','Couplage roue-satellite/CALCUL DU COUPLE DE COUPLAGE ROUE SATELLITE'],...
		'ForeGround',3,...
		'BackGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[530,50,535,55])

add_block('built-in/Gain',[sys,'/','Couplage roue-satellite/Gain1'])
set_param([sys,'/','Couplage roue-satellite/Gain1'],...
		'hide name',0,...
		'position',[475,172,500,198])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/Fonction jw_rotot.m'])
set_param([sys,'/','Couplage roue-satellite/Fonction jw_rotot.m'],...
		'position',[375,220,380,225])

add_block('built-in/Inport',[sys,'/','Couplage roue-satellite/in_3'])
set_param([sys,'/','Couplage roue-satellite/in_3'],...
		'position',[95,140,115,160])

add_block('built-in/Inport',[sys,'/','Couplage roue-satellite/in_4'])
set_param([sys,'/','Couplage roue-satellite/in_4'],...
		'Port','2',...
		'position',[100,209,125,231])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/pqrQ'])
set_param([sys,'/','Couplage roue-satellite/pqrQ'],...
		'position',[80,205,85,210])

add_block('built-in/MATLAB Fcn',[sys,'/','Couplage roue-satellite/magneto.m2'])
set_param([sys,'/','Couplage roue-satellite/magneto.m2'],...
		'ForeGround',1,...
		'BackGround',0,...
		'hide name',0,...
		'MATLAB Fcn','jwrotot2(u)',...
		'Output Width','3',...
		'Mask Display','Participation\nde la roue\nau second membre')
set_param([sys,'/','Couplage roue-satellite/magneto.m2'],...
		'position',[300,161,440,209])

add_block('built-in/Note',[sys,'/',['Couplage roue-satellite/Sortie du couplage',13,'roue satellite']])
set_param([sys,'/',['Couplage roue-satellite/Sortie du couplage',13,'roue satellite']],...
		'position',[600,220,605,225])

add_block('built-in/Note',[sys,'/','Couplage roue-satellite/Vitesseroue'])
set_param([sys,'/','Couplage roue-satellite/Vitesseroue'],...
		'position',[100,110,105,115])

add_block('built-in/Outport',[sys,'/','Couplage roue-satellite/out_1'])
set_param([sys,'/','Couplage roue-satellite/out_1'],...
		'position',[590,175,610,195])

add_block('built-in/Mux',[sys,'/','Couplage roue-satellite/Mux'])
set_param([sys,'/','Couplage roue-satellite/Mux'],...
		'inputs','[1 7]',...
		'position',[195,113,250,257])

add_block('built-in/To Workspace',[sys,'/','Couplage roue-satellite/To Workspace7'])
set_param([sys,'/','Couplage roue-satellite/To Workspace7'],...
		'ForeGround',1,...
		'BackGround',3,...
		'hide name',0,...
		'mat-name','couplage_roue_sat',...
		'buffer','points_trace',...
		'position',[685,294,845,316])

add_block('built-in/State-Space',[sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']])
set_param([sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'A','[]',...
		'B','[]',...
		'C','[]',...
		'D','Asat',...
		'Mask Display','Asat',...
		'Mask Type','Matrix Gain')
set_param([sys,'/',['Couplage roue-satellite/Matrix',13,'Gain',13,'applique',13,'aux couples',13,'externes']],...
		'Mask Help','Multiplies input vector by entered matrix to produce output vector (y=Au).',...
		'position',[555,282,655,328])
add_line([sys,'/','Couplage roue-satellite'],[255,185;295,185])
add_line([sys,'/','Couplage roue-satellite'],[120,150;190,150])
add_line([sys,'/','Couplage roue-satellite'],[130,220;190,220])
add_line([sys,'/','Couplage roue-satellite'],[445,185;470,185])
add_line([sys,'/','Couplage roue-satellite'],[505,185;585,185])
add_line([sys,'/','Couplage roue-satellite'],[660,305;680,305])
add_line([sys,'/','Couplage roue-satellite'],[530,185;530,305;550,305])
set_param([sys,'/','Couplage roue-satellite'],...
		'Mask Display','COUPLAGE ROUE-SATELLITE')


%     Finished composite block 'Couplage roue-satellite'.

set_param([sys,'/','Couplage roue-satellite'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'position',[115,380,330,435])


%     Subsystem  'Subsystem1'.

new_system([sys,'/','Subsystem1'])
set_param([sys,'/','Subsystem1'],'Location',[196,101,1144,611])

add_block('built-in/Note',[sys,'/','Subsystem1/QUE FAIT LA SIMULATION'])
set_param([sys,'/','Subsystem1/QUE FAIT LA SIMULATION'],...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[625,10,630,15])

add_block('built-in/Note',[sys,'/','Subsystem1/Robert Guiziou dcembre 2013'])
set_param([sys,'/','Subsystem1/Robert Guiziou dcembre 2013'],...
		'position',[1115,690,1120,695])
set_param([sys,'/','Subsystem1'],...
		'Mask Display','LIRE LES EXPLICATIONS\nDANS L''ESPACE MATLAB\nAPRES AVOIR INITIALISE',...
		'Mask Dialogue','eval(''nanoexp0'')')


%     Finished composite block 'Subsystem1'.

set_param([sys,'/','Subsystem1'],...
		'ForeGround',7,...
		'BackGround',3,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[105,805,405,881])


%     Subsystem  'Energie'.

new_system([sys,'/','Energie'])
set_param([sys,'/','Energie'],'Location',[4,63,164,72])

add_block('built-in/Inport',[sys,'/','Energie/in_1'])
set_param([sys,'/','Energie/in_1'],...
		'position',[90,275,110,295])


%     Subsystem  'Energie/Energie'.

new_system([sys,'/','Energie/Energie'])
set_param([sys,'/','Energie/Energie'],'Location',[4,44,1268,968])

add_block('built-in/To Workspace',[sys,'/','Energie/Energie/To Workspace10'])
set_param([sys,'/','Energie/Energie/To Workspace10'],...
		'hide name',0,...
		'mat-name','energie_spin',...
		'buffer','points_trace',...
		'position',[650,284,795,306])

add_block('built-in/To Workspace',[sys,'/','Energie/Energie/To Workspace12'])
set_param([sys,'/','Energie/Energie/To Workspace12'],...
		'hide name',0,...
		'mat-name','puissance_inst',...
		'buffer','points_trace',...
		'position',[440,209,545,231])


%     Subsystem  ['Energie/Energie/Inner',13,'Product'].

new_system([sys,'/',['Energie/Energie/Inner',13,'Product']])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product']],'Location',[33,479,253,634])

add_block('built-in/Inport',[sys,'/',['Energie/Energie/Inner',13,'Product/in_1']])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product/in_1']],...
		'position',[15,25,35,45])

add_block('built-in/Inport',[sys,'/',['Energie/Energie/Inner',13,'Product/in_2']])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product/in_2']],...
		'Port','2',...
		'position',[15,90,35,110])

add_block('built-in/Outport',[sys,'/',['Energie/Energie/Inner',13,'Product/out_1']])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product/out_1']],...
		'position',[185,55,205,75])

add_block('built-in/Sum',[sys,'/',['Energie/Energie/Inner',13,'Product/Sum']])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product/Sum']],...
		'inputs','+',...
		'position',[125,55,145,75])

add_block('built-in/Product',[sys,'/',['Energie/Energie/Inner',13,'Product/Product']])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product/Product']],...
		'position',[65,55,90,75])
add_line([sys,'/',['Energie/Energie/Inner',13,'Product']],[150,65;180,65])
add_line([sys,'/',['Energie/Energie/Inner',13,'Product']],[95,65;120,65])
add_line([sys,'/',['Energie/Energie/Inner',13,'Product']],[40,35;60,60])
add_line([sys,'/',['Energie/Energie/Inner',13,'Product']],[40,100;60,70])
set_param([sys,'/',['Energie/Energie/Inner',13,'Product']],...
		'Mask Display','.\n',...
		'Mask Type','Dot Product',...
		'Mask Dialogue','Inner (dot) product.\ny=sum(u1.*u2)')
set_param([sys,'/',['Energie/Energie/Inner',13,'Product']],...
		'Mask Help','Computes the inner (dot) product of the inputs: sum(u1.*u2).  The inputs are vectors of the same length. Output is a scalar.')


%     Finished composite block ['Energie/Energie/Inner',13,'Product'].

set_param([sys,'/',['Energie/Energie/Inner',13,'Product']],...
		'position',[385,265,415,320])

add_block('built-in/Integrator',[sys,'/','Energie/Energie/Integrator'])
set_param([sys,'/','Energie/Energie/Integrator'],...
		'position',[475,260,505,330])

add_block('built-in/Note',[sys,'/','Energie/Energie/K_eclipse'])
set_param([sys,'/','Energie/Energie/K_eclipse'],...
		'position',[220,340,225,345])

add_block('built-in/Mux',[sys,'/','Energie/Energie/Mux6'])
set_param([sys,'/','Energie/Energie/Mux6'],...
		'hide name',0,...
		'inputs','[1 3]',...
		'position',[185,224,220,331])

add_block('built-in/Inport',[sys,'/','Energie/Energie/in_1'])
set_param([sys,'/','Energie/Energie/in_1'],...
		'Port','2',...
		'position',[70,240,90,260])

add_block('built-in/Inport',[sys,'/','Energie/Energie/in_2'])
set_param([sys,'/','Energie/Energie/in_2'],...
		'Port','3',...
		'position',[70,295,90,315])

add_block('built-in/MATLAB Fcn',[sys,'/',['Energie/Energie/Couple magntique',13,'5']])
set_param([sys,'/',['Energie/Energie/Couple magntique',13,'5']],...
		'hide name',0,...
		'MATLAB Fcn','PrecPuis(u)',...
		'Output Width','1',...
		'Mask Display','Puissance',...
		'position',[245,261,335,299])

add_block('built-in/MATLAB Fcn',[sys,'/','Energie/Energie/1//u'])
set_param([sys,'/','Energie/Energie/1//u'],...
		'MATLAB Fcn','1/u',...
		'Mask Display','INVERSE',...
		'position',[485,131,560,169])

add_block('built-in/To Workspace',[sys,'/','Energie/Energie/To Workspace11'])
set_param([sys,'/','Energie/Energie/To Workspace11'],...
		'hide name',0,...
		'mat-name','puissance_moyenne',...
		'buffer','points_trace',...
		'position',[705,159,835,181])

add_block('built-in/Product',[sys,'/','Energie/Energie/Product'])
set_param([sys,'/','Energie/Energie/Product'],...
		'position',[610,128,650,212])

add_block('built-in/Inport',[sys,'/','Energie/Energie/in_3'])
set_param([sys,'/','Energie/Energie/in_3'],...
		'position',[70,355,90,375])

add_block('built-in/Constant',[sys,'/','Energie/Energie/Constant'])
set_param([sys,'/','Energie/Energie/Constant'],...
		'hide name',0,...
		'Value','temps_energie',...
		'position',[220,152,320,178])

add_block('built-in/Sum',[sys,'/','Energie/Energie/Sum'])
set_param([sys,'/','Energie/Energie/Sum'],...
		'inputs','+-',...
		'position',[375,122,395,178])

add_block('built-in/Note',[sys,'/',['Energie/Energie/ENERGIE CALCULEE A PARTIR DES REGULATIONS ALTERNEES',13,'temps > temps_phase3']])
set_param([sys,'/',['Energie/Energie/ENERGIE CALCULEE A PARTIR DES REGULATIONS ALTERNEES',13,'temps > temps_phase3']],...
		'Font Name','Arial Narrow',...
		'Font Size',18,...
		'position',[375,15,380,20])
add_line([sys,'/','Energie/Energie'],[510,295;645,295])
add_line([sys,'/','Energie/Energie'],[575,295;575,190;605,190])
add_line([sys,'/','Energie/Energie'],[420,295;420,220;435,220])
add_line([sys,'/','Energie/Energie'],[420,295;470,295])
add_line([sys,'/','Energie/Energie'],[340,280;380,280])
add_line([sys,'/','Energie/Energie'],[95,365;350,365;350,305;380,305])
add_line([sys,'/','Energie/Energie'],[655,170;700,170])
add_line([sys,'/','Energie/Energie'],[565,150;605,150])
add_line([sys,'/','Energie/Energie'],[400,150;480,150])
add_line([sys,'/','Energie/Energie'],[325,165;370,165])
add_line([sys,'/','Energie/Energie'],[95,250;180,250])
add_line([sys,'/','Energie/Energie'],[150,250;150,135;370,135])
add_line([sys,'/','Energie/Energie'],[95,305;180,305])
add_line([sys,'/','Energie/Energie'],[225,280;240,280])
set_param([sys,'/','Energie/Energie'],...
		'Mask Display','ENERGIE PRODUITE')


%     Finished composite block 'Energie/Energie'.

set_param([sys,'/','Energie/Energie'],...
		'position',[410,162,590,218])

add_block('built-in/MATLAB Fcn',[sys,'/',['Energie/Couple magntique',13,'3']])
set_param([sys,'/',['Energie/Couple magntique',13,'3']],...
		'hide name',0,...
		'MATLAB Fcn','soleil(u)',...
		'Output Width','1',...
		'Mask Display','Soleil',...
		'position',[245,76,335,114])

add_block('built-in/Note',[sys,'/',['Energie/Sortie B',13,'axes satellite']])
set_param([sys,'/',['Energie/Sortie B',13,'axes satellite']],...
		'hide name',0,...
		'position',[225,104,230,109])

add_block('built-in/Clock',[sys,'/','Energie/Clock1'])
set_param([sys,'/','Energie/Clock1'],...
		'position',[95,77,130,113])


%     Subsystem  'Energie/Unitaire soleil'.

new_system([sys,'/','Energie/Unitaire soleil'])
set_param([sys,'/','Energie/Unitaire soleil'],'Location',[4,63,164,72])

add_block('built-in/Note',[sys,'/','Energie/Unitaire soleil/UX UY UZ'])
set_param([sys,'/','Energie/Unitaire soleil/UX UY UZ'],...
		'ForeGround',2,...
		'Font Weight','bold',...
		'Font Size',12,...
		'position',[310,105,315,110])

add_block('built-in/Mux',[sys,'/','Energie/Unitaire soleil/Mux1'])
set_param([sys,'/','Energie/Unitaire soleil/Mux1'],...
		'inputs','[3 7]',...
		'position',[490,120,535,280])

add_block('built-in/Note',[sys,'/','Energie/Unitaire soleil/Equateur au temps 0 s'])
set_param([sys,'/','Energie/Unitaire soleil/Equateur au temps 0 s'],...
		'position',[155,115,160,120])

add_block('built-in/Note',[sys,'/','Energie/Unitaire soleil/Axes satellite'])
set_param([sys,'/','Energie/Unitaire soleil/Axes satellite'],...
		'position',[715,155,720,160])

add_block('built-in/Note',[sys,'/','Energie/Unitaire soleil/Sortie Ux Uy Uz'])
set_param([sys,'/','Energie/Unitaire soleil/Sortie Ux Uy Uz'],...
		'position',[720,135,725,140])

add_block('built-in/Outport',[sys,'/','Energie/Unitaire soleil/out_1'])
set_param([sys,'/','Energie/Unitaire soleil/out_1'],...
		'position',[725,190,745,210])

add_block('built-in/Inport',[sys,'/','Energie/Unitaire soleil/in_1'])
set_param([sys,'/','Energie/Unitaire soleil/in_1'],...
		'position',[145,230,165,250])

add_block('built-in/Note',[sys,'/',['Energie/Unitaire soleil/Vecteur d''tat ',13,'7 composantes',13,'pqrQ)']])
set_param([sys,'/',['Energie/Unitaire soleil/Vecteur d''tat ',13,'7 composantes',13,'pqrQ)']],...
		'position',[145,275,150,280])

add_block('built-in/Clock',[sys,'/','Energie/Unitaire soleil/Clock'])
set_param([sys,'/','Energie/Unitaire soleil/Clock'],...
		'position',[135,144,170,176])

add_block('built-in/MATLAB Fcn',[sys,'/','Energie/Unitaire soleil/Unit_Sat'])
set_param([sys,'/','Energie/Unitaire soleil/Unit_Sat'],...
		'MATLAB Fcn','Unit_Sat(u)',...
		'Output Width','3',...
		'position',[590,183,650,217])

add_block('built-in/MATLAB Fcn',[sys,'/','Energie/Unitaire soleil/SunitXYZ'])
set_param([sys,'/','Energie/Unitaire soleil/SunitXYZ'],...
		'Font Size',12,...
		'MATLAB Fcn','SunitXYZ(u)',...
		'Output Width','3',...
		'Mask Display','UNITAIRE SOLEIL\nEN REPERE ORBITAL',...
		'position',[230,135,405,185])

add_block('built-in/Note',[sys,'/',['Energie/Unitaire soleil/CALCUL DE L''UNITAIRE SOLEIL Us EN AXES SATELLITE xyz']])
set_param([sys,'/',['Energie/Unitaire soleil/CALCUL DE L''UNITAIRE SOLEIL Us EN AXES SATELLITE xyz']],...
		'ForeGround',2,...
		'Font Weight','bold',...
		'Font Size',20,...
		'position',[490,0,495,5])
add_line([sys,'/','Energie/Unitaire soleil'],[410,160;485,160])
add_line([sys,'/','Energie/Unitaire soleil'],[175,160;225,160])
add_line([sys,'/','Energie/Unitaire soleil'],[655,200;720,200])
add_line([sys,'/','Energie/Unitaire soleil'],[170,240;485,240])
add_line([sys,'/','Energie/Unitaire soleil'],[540,200;585,200])
set_param([sys,'/','Energie/Unitaire soleil'],...
		'Mask Display','UNITAIRE SOLEIL\nEN\nAXES SATELLITE')


%     Finished composite block 'Energie/Unitaire soleil'.

set_param([sys,'/','Energie/Unitaire soleil'],...
		'position',[140,248,325,322])

add_block('built-in/To Workspace',[sys,'/','Energie/To Workspace12'])
set_param([sys,'/','Energie/To Workspace12'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'mat-name','Us_xyz',...
		'buffer','points_trace',...
		'position',[445,275,515,295])

add_block('built-in/To Workspace',[sys,'/','Energie/To Workspace13'])
set_param([sys,'/','Energie/To Workspace13'],...
		'ForeGround',1,...
		'BackGround',2,...
		'hide name',0,...
		'mat-name','eclipse',...
		'buffer','points_trace',...
		'position',[440,85,510,105])
add_line([sys,'/','Energie'],[115,285;135,285])
add_line([sys,'/','Energie'],[330,285;375,285;375,210;405,210])
add_line([sys,'/','Energie'],[340,95;375,95;375,170;405,170])
add_line([sys,'/','Energie'],[135,95;240,95])
add_line([sys,'/','Energie'],[185,95;185,190;405,190])
add_line([sys,'/','Energie'],[375,285;440,285])
add_line([sys,'/','Energie'],[375,95;435,95])
set_param([sys,'/','Energie'],...
		'Mask Display','MODULE\n\nENERGIE')


%     Finished composite block 'Energie'.

set_param([sys,'/','Energie'],...
		'position',[590,496,740,564])

add_block('built-in/Note',[sys,'/',['Stabilisation de M Philippe Laurens par une rgulation PD',13,' sur un champ de consigne optimal ou consigne nadir']])
set_param([sys,'/',['Stabilisation de M Philippe Laurens par une rgulation PD',13,' sur un champ de consigne optimal ou consigne nadir']],...
		'ForeGround',5,...
		'Font Weight','bold',...
		'Font Size',22,...
		'position',[633,95,638,100])

add_block('built-in/Note',[sys,'/','Simulation survfix3.m ( ensemble des fichiers dans survfix1.zip )'])
set_param([sys,'/','Simulation survfix3.m ( ensemble des fichiers dans survfix1.zip )'],...
		'Font Name','Arial Narrow',...
		'Font Size',18,...
		'position',[635,195,640,200])

add_block('built-in/Note',[sys,'/','TOUS LES MODES AVEC UNE ROUE A VITESSE CONSTANTE'])
set_param([sys,'/','TOUS LES MODES AVEC UNE ROUE A VITESSE CONSTANTE'],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',16,...
		'position',[600,170,605,175])

add_block('built-in/Note',[sys,'/',['SYNTHESE : BASCULEMENTS DE SURVIE EN POINTAGE NADIR',13,' AVEC QUATERNION D''ATTITUDE ',13,'',13,'',13,'']])
set_param([sys,'/',['SYNTHESE : BASCULEMENTS DE SURVIE EN POINTAGE NADIR',13,' AVEC QUATERNION D''ATTITUDE ',13,'',13,'',13,'']],...
		'ForeGround',3,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',26)
set_param([sys,'/',['SYNTHESE : BASCULEMENTS DE SURVIE EN POINTAGE NADIR',13,' AVEC QUATERNION D''ATTITUDE ',13,'',13,'',13,'']],...
		'position',[715,10,720,15])


%     Subsystem  'Extras1'.

new_system([sys,'/','Extras1'])
set_param([sys,'/','Extras1'],'Location',[4,42,1012,678])
set_param([sys,'/','Extras1'],...
		'Mask Display','1 - INITIALISATION\npar double clic souris\nCAS GENERAL',...
		'Mask Dialogue','eval(''beta3dat'')')


%     Finished composite block 'Extras1'.

set_param([sys,'/','Extras1'],...
		'ForeGround',1,...
		'BackGround',6,...
		'hide name',0,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[840,410,1085,471])

add_block('built-in/Note',[sys,'/',['L''initialisation lance beta3dat.m',13,'']])
set_param([sys,'/',['L''initialisation lance beta3dat.m',13,'']],...
		'BackGround',6,...
		'Font Name','Arial',...
		'Font Weight','bold',...
		'Font Size',14,...
		'position',[975,480,980,485])
add_line(sys,[725,410;745,410;745,580;35,580;35,495;115,495])
add_line(sys,[550,410;615,410])
add_line(sys,[340,495;495,495])
add_line(sys,[745,410;745,280;840,280])
add_line(sys,[595,410;595,320;840,320])
add_line(sys,[35,495;35,320;120,320])
add_line(sys,[355,300;415,300])
add_line(sys,[335,410;400,410;400,335;415,335])
add_line(sys,[450,320;495,320])
add_line(sys,[35,420;110,420])
add_line(sys,[355,340;365,340;365,370;95,370;95,395;110,395])
add_line(sys,[565,580;565,530;585,530])

drawnow

% Return any arguments.
if (nargin | nargout)
	% Must use feval here to access system in memory
	if (nargin > 3)
		if (flag == 0)
			eval(['[ret,x0,str,ts,xts]=',sys,'(t,x,u,flag);'])
		else
			eval(['ret =', sys,'(t,x,u,flag);'])
		end
	else
		[ret,x0,str,ts,xts] = feval(sys);
	end
else
	drawnow % Flash up the model and execute load callback
end
